"""This file is auto-generated with stub_generation.py"""
from typing import List,Any,overload,Callable,Union
from enum import Enum
import numpy as np
from shyft.time_series._time_series import *
# import Boost.Python
nan = float('nan')

class CatchmentPropertyType(Enum):
    """
    int([x]) -> integer
    int(x, base=10) -> integer
    
    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating point
    numbers, this truncates towards zero.
    
    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """
    DISCHARGE: int
    SNOW_COVERED_AREA: int
    SNOW_WATER_EQUIVALENT: int
    ROUTED_DISCHARGE: int
    CELL_CHARGE: int


class OKCovarianceType(Enum):
    """
    int([x]) -> integer
    int(x, base=10) -> integer
    
    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating point
    numbers, this truncates towards zero.
    
    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """
    GAUSSIAN: int
    EXPONENTIAL: int


class OptimizerMethod(Enum):
    """
    Shyft calibration allow the user to select between one of these parameter optimizer methods.
    """
    BOBYQA: int
    GLOBAL: int
    DREAM: int
    SCEUA: int


class RegionModelType(Enum):
    """
    Ref to DrmClient, used do specify what remote region-model type to create
    """
    PT_GS_K: int
    PT_GS_K_OPT: int
    PT_SS_K: int
    PT_SS_K_OPT: int
    PT_HS_K: int
    PT_HS_K_OPT: int
    PT_HPS_K: int
    PT_HPS_K_OPT: int
    R_PM_GS_K: int
    R_PM_GS_K_OPT: int
    PT_ST_K: int
    PT_ST_K_OPT: int
    PT_ST_HBV: int
    PT_ST_K_HBV: int
    R_PT_GS_K: int
    R_PT_GS_K_OPT: int


class TargetSpecCalcType(Enum):
    """
    int([x]) -> integer
    int(x, base=10) -> integer
    
    Convert a number or string to an integer, or return 0 if no arguments
    are given.  If x is a number, return x.__int__().  For floating point
    numbers, this truncates towards zero.
    
    If x is not a number or if base is given, then x must be a string,
    bytes, or bytearray instance representing an integer literal in the
    given base.  The literal can be preceded by '+' or '-' and be surrounded
    by whitespace.  The base defaults to 10.  Valid bases are 0 and 2-36.
    Base 0 means to interpret the base from the string as an integer literal.
    >>> int('0b100', base=0)
    4
    """
    NASH_SUTCLIFFE: int
    KLING_GUPTA: int
    ABS_DIFF: int
    RMSE: int


class stat_scope(Enum):
    """
    Defines the scope of the indexes passed to statistics functions
    .cell      : the indexes are considered as the i'th cell
    .catchment : the indexes are considered catchment identifiers
    
    The statistics is then covering the cells that matches the selected criteria
    """
    cell: int
    catchment: int


class ARegionEnvironment:
    """
    Contains all geo-located sources to be used by a Shyft core model
    """
    @property
    def precipitation(self)->PrecipitationSourceVector:
        """
         precipitation sources
        """
        ...
    @precipitation.setter
    def precipitation(self, value:PrecipitationSourceVector)->None:
        ...
    @property
    def radiation(self)->RadiationSourceVector:
        """
         radiation sources
        """
        ...
    @radiation.setter
    def radiation(self, value:RadiationSourceVector)->None:
        ...
    @property
    def rel_hum(self)->RelHumSourceVector:
        """
         rel-hum sources
        """
        ...
    @rel_hum.setter
    def rel_hum(self, value:RelHumSourceVector)->None:
        ...
    @property
    def temperature(self)->TemperatureSourceVector:
        """
         temperature sources
        """
        ...
    @temperature.setter
    def temperature(self, value:TemperatureSourceVector)->None:
        ...
    @property
    def wind_speed(self)->WindSpeedSourceVector:
        """
         wind-speed sources
        """
        ...
    @wind_speed.setter
    def wind_speed(self, value:WindSpeedSourceVector)->None:
        ...
    def __init__(self) -> None:
        ...

    @staticmethod
    def create_from_geo_ts_matrix(m: GeoTsMatrix, t: object = 0, e: object = 0, v: Union[IntVector,list[int],range] = _time_series.IntVector) -> ARegionEnvironment:
        """
        Construct ARegionEnvironment from a specified slice of a GeoTsMatrix
            
            Args:
                m (GeoTsMatrix): The matrix from which to extract temperature,precipitation,radiation,wind-speed,rel-hum data
            
                t (int): the forecast index to extract
            
                e (int): the ensemble index to extract
            
            Returns:
                ARegionEnvironment: ARegionEnvironment. A ready to use region environment for interpolation
            
                v (IntVector): variable indicies that in order of appearance selects temperature,precipitation,radiation,wind-speed,rel-hum.If empty/default, same as [0,1,2,3,4]
            
        """
        ...

    @staticmethod
    def deserialize(blob: ByteVector) -> ARegionEnvironment:
        """
        convert a blob, as returned by .serialize() into a ARegionEnvironment
        """
        ...

    def serialize(self) -> ByteVector:
        """
        convert ARegionEnvironment into a binary blob that later can be restored with the .deserialize(blob) method
            
        """
        ...



class ActualEvapotranspirationParameter:
    @property
    def ae_scale_factor(self)->float:
        """
         typical value 1.5
        """
        ...
    @ae_scale_factor.setter
    def ae_scale_factor(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, ae_scale_factor: object) -> None:
        """
        a new object with specified parameters
        """
        ...



class ActualEvapotranspirationResponse:
    @property
    def ae(self)->float:
        """
         actual evapotranspiration
        """
        ...
    @ae.setter
    def ae(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class BTKParameter:
    """
    BTKParameter class with time varying gradient based on day no
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, temperature_gradient: object, temperature_gradient_sd: object) -> None:
        """
        specifying default temp.grad(not used) and std.dev[C/100m]
        
        """
        ...
    @overload
    def __init__(self, temperature_gradient: object, temperature_gradient_sd: object, sill: object, nugget: object, range: object, zscale: object) -> None:
        """
        full specification of all parameters
        """
        ...

    def nug(self) -> float:
        """
        Nugget magnitude,default=0.5
        """
        ...

    def range(self) -> float:
        """
        Point where semivariogram flattens out,default=200000.0
        """
        ...

    def sill(self) -> float:
        """
        Value of semivariogram at range default=25.0
        """
        ...

    def temperature_gradient(self, p: UtcPeriod) -> float:
        """
        return default temp.gradient based on day of year calculated for midst of utcperiod p
        """
        ...

    def temperature_gradient_sd(self) -> float:
        """
        returns Prior standard deviation of temperature gradient in [C/m]
        """
        ...

    def zscale(self) -> float:
        """
        Height scale used during distance computations,default=20.0
        """
        ...



class CalibrationOption:
    """
    The CalibrationOption controls how the calibration optmizer is run regarding method and termination criterias.
    Notice that the different optmizers do have variations in the termination criterias.
    All optimization algorithms expect at least one-dimensional paramteter space.
    """
    @property
    def max_iterations(self)->int:
        """
         stop after this number of iterations (default 1500)
        """
        ...
    @max_iterations.setter
    def max_iterations(self, value:int)->None:
        ...
    @property
    def method(self)->int:
        """
         OptimizerMethod,  which parameter optimizer to use
        """
        ...
    @method.setter
    def method(self, value:int)->None:
        ...
    @property
    def solver_epsilon(self)->float:
        """
         GLOBAL: select which parameter optimizer to use (default 0.001)
        """
        ...
    @solver_epsilon.setter
    def solver_epsilon(self, value:float)->None:
        ...
    @property
    def time_limit(self)->time:
        """
         GLOBAL: don't run the global search longer than this (default 0, off)
        """
        ...
    @time_limit.setter
    def time_limit(self, value:time)->None:
        ...
    @property
    def tr_start(self)->float:
        """
         BOBYQA: trust region start (default 0.1)
        """
        ...
    @tr_start.setter
    def tr_start(self, value:float)->None:
        ...
    @property
    def tr_stop(self)->float:
        """
         BOBYQA: trust region stop/end (default 1e-5)
        """
        ...
    @tr_stop.setter
    def tr_stop(self, value:float)->None:
        ...
    @property
    def x_epsilon(self)->float:
        """
         SCEUA: stop when normalized parameters x is within this range in variation
        """
        ...
    @x_epsilon.setter
    def x_epsilon(self, value:float)->None:
        ...
    @property
    def y_epsilon(self)->float:
        """
         SCEUA: stop when goal function is stable within this range (not improving any further)
        """
        ...
    @y_epsilon.setter
    def y_epsilon(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, method: OptimizerMethod, max_iterations: object, time_limit: Union[time,float,int], solver_epsilon: object, x_epsilon: object, y_epsilon: object, tr_start: object, tr_stop: object) -> None:
        """
        Initialises a ParameterOptimizer instance.
            
                method (OptimizerMethod): select which parameter optimizer to use.
            
                max_iterations (int): stop after this number of iterations (default 1500).
            
                time_limit (float): GLOBAL: don't run the global search longer than this (default 0, off).
            
                solver_epsilon (float): GLOBAL: select which parameter optimizer to use (default 0.001).
            
                x_epsilon (float): SCEUA: stop when normalized parameters x is within this range in variation.
            
                y_epsilon (float): SCEUA: stop when goal function is stable within this range (not improving any further).
            
                tr_start (float): BOBYQA: trust region start (default 0.1).
            
                tr_stop (float): BOBYQA: trust region stop/end (default 1e-5).
            
        """
        ...

    @staticmethod
    def bobyqa(max_iterations: object, tr_start: object, tr_stop: object) -> CalibrationOption:
        """
        Construct bobyqa options.
            
                max_iterations (int): stop after this number of iterations (default 1500).
            
                tr_start (float): BOBYQA: trust region start (default 0.1).
            
                tr_stop (float): trust region stop/end (default 1e-5).
            
            Returns:
                CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
            
        """
        ...

    @staticmethod
    def dream(max_iterations: object) -> CalibrationOption:
        """
        Construct dream options
            
                max_iterations (int): stop after this number of iterations (default 1500).
            
            Returns:
                CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
            
        """
        ...

    @staticmethod
    def global_search(max_iterations: object, time_limit: Union[time,float,int], solver_epsilon: object) -> CalibrationOption:
        """
        Construct global search options
            
                max_iterations (int): stop after this number of iterations (default 1500).
            
                time_limit (float): don't run the global search longer than this (default 0, off).
            
                solver_epsilon (float): select which parameter optimizer to use (default 0.001).
            
            Returns:
                CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
            
        """
        ...

    @staticmethod
    def sceua(max_iterations: object, x_epsilon: object, y_epsilon: object) -> CalibrationOption:
        """
        Construct sceua options
            
                max_iterations (int): stop after this number of iterations (default 1500).
            
                x_epsilon (float): stop when normalized parameters x is within this range in variation.
            
                y_epsilon (float): stop when goal function is stable within this range (not improving any further).
            
            Returns:
                CalibrationOption: co. An instance of CalibrationOption for the relevant optmization algorithm.
            
        """
        ...



class CalibrationStatus:
    """
    The CalibrationStatus is returned from drms, or local call to the region model calibration object.It provides status, progress, and current trace of parameters and goal function values
    """
    @property
    def running(self)->bool:
        """
         True if the calibration is still running
        """
        ...
    @property
    def trace_goal_function_values(self)->DoubleVector:
        """
         goal function values obtained so far
        """
        ...
    @property
    def trace_size(self)->int:
        """
         returns the size of the parameter-trace
        
        See also:
            trace_goal_function_value,trace_parameter
        """
        ...
    def __init__(*args, **kwargs):
        """
        This class cannot be instantiated from Python
        """
        ...

    @staticmethod
    def result(*args, **kwargs):
        """
        returns resulting calibrated parameter if available
            
        """
        ...

    def trace_goal_function_value(self, i: object) -> float:
        """
        returns the i'th goal function value
            
        """
        ...

    @staticmethod
    def trace_parameter(*args, **kwargs):
        """
        returns the i'th parameter tried, corresponding to the 
            i'th trace_goal_function value
            
        """
        ...



class CellEnvironment:
    """
    Contains all ts projected to a certain cell-model using interpolation step (if needed)
    """
    @property
    def precipitation(self)->TsFixed:
        """
         precipitation
        """
        ...
    @precipitation.setter
    def precipitation(self, value:TsFixed)->None:
        ...
    @property
    def radiation(self)->TsFixed:
        """
         radiation
        """
        ...
    @radiation.setter
    def radiation(self, value:TsFixed)->None:
        ...
    @property
    def rel_hum(self)->TsFixed:
        """
         relhum
        """
        ...
    @rel_hum.setter
    def rel_hum(self, value:TsFixed)->None:
        ...
    @property
    def temperature(self)->TsFixed:
        """
         temperature
        """
        ...
    @temperature.setter
    def temperature(self, value:TsFixed)->None:
        ...
    @property
    def wind_speed(self)->TsFixed:
        """
         wind speed
        """
        ...
    @wind_speed.setter
    def wind_speed(self, value:TsFixed)->None:
        ...
    def __init__(self) -> None:
        ...

    def has_nan_values(self) -> bool:
        """
        scans all time-series for nan-values
            
            Returns:
                bool: has_nan. true if any nan is encounted, otherwise false
            
        """
        ...

    def init(self, ta: TimeAxisFixedDeltaT) -> None:
        """
        zero all series, set time-axis ta
        """
        ...



class CellStateId:
    """
    Unique cell pseudo identifier of  a state
    
    A lot of Shyft models can appear in different geometries and
    resolutions.In a few rare cases we would like to store the *state* of
    a model.This would be typical for example for hydrological new - year 1 - sept in Norway.
    To ensure that each cell - state have a unique identifier so that we never risk
    mixing state from different cells or different geometries, we create a pseudo unique
    id that helps identifying unique cell - states given this usage and context.
    
    The primary usage is to identify which cell a specific identified state belongs to.
    """
    @property
    def area(self)->int:
        """
         area in [m^2]
        """
        ...
    @area.setter
    def area(self, value:int)->None:
        ...
    @property
    def cid(self)->int:
        """
         catchment identifier
        """
        ...
    @cid.setter
    def cid(self, value:int)->None:
        ...
    @property
    def x(self)->int:
        """
         x position in [m]
        """
        ...
    @x.setter
    def x(self, value:int)->None:
        ...
    @property
    def y(self)->int:
        """
         y position in [m]
        """
        ...
    @y.setter
    def y(self, value:int)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, cid: object, x: object, y: object, area: object) -> None:
        """
        construct CellStateId with specified parameters
        """
        ...



class DrmClient:
    """
    Distributed region model client provides all needed functionality to transfer
    Shyft region-models, and then run simulations/optimizations
    """
    total_clients: int
    @property
    def host_port(self)->str:
        """
         Endpoint network address of the remote server.
        """
        ...
    @property
    def is_open(self)->bool:
        """
         If the connection to the remote server is (still) open.
        """
        ...
    @property
    def reconnect_count(self)->int:
        """
         Number of reconnects to the remote server that have been performed.
        """
        ...
    @property
    def timeout_ms(self)->int:
        """
         Timout for remote server operations, in number milliseconds.
        """
        ...
    def __init__(self, host_port: str, timeout_ms: object) -> None:
        """
        Creates a client with supplied arguments
        """
        ...

    def adjust_q(self, mid: str, indexes: Union[IntVector,list[int],range], wanted_q: object, start_step: object = 0, scale_range: object = 3.0, scale_eps: object = 0.001, max_iter: object = 300, n_steps: object = 1) -> FlowAdjustResult:
        """
        State adjustment to achieve wanted/observed flow
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): cell ids that should be adjusted
            
                wanted_q (float): the average flow first time-step we want to achieve, in m**3/s
            
                start_step (int): time-axis start step index, default=0
            
                scale_range (float): default=3, min,max= scale0/scale_range.. scale0*scale_range
            
                scale_eps (float): default 1e-3, stop iteration when scale-change is less than this
            
                max_iter (int): default=300, stop searching for solution after this limit is reached
            
                n_steps (int): default=1, number of steps on time-axis to average to match wanted flow
            
            Returns:
                q_adjust_result: . obtained flow in m**3/s. This can deviate from wanted flow due to model and state constraints. q_r is optimized q and q_0 is unadjusted.
            
        """
        ...

    def cancel_calibration(self, mid: str) -> bool:
        """
        Cancels any ongoing not finished calibration for the specified model
            
            Args:
                mid (str): model identifier
            
            Returns:
                bool: . indicating success
            
        """
        ...

    def check_calibration(self, mid: str) -> CalibrationStatus:
        """
        Check ongoing calibration, return intermediate status with trace, or final status with trace and resulting parameters
            
            Args:
                mid (): model identifier
            
            Returns:
                : . Calibration Status class with information and results
            
        """
        ...

    def clone_model(self, mid: str, new_mid: str) -> bool:
        """
        clone the specified model
            
            Args:
                mid (str): original model
            
                new_mid (str): the model-id for the cloned model
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
            
        """
        ...

    def copy_model(self, mid: str, new_mid: str) -> bool:
        """
        full copy of the specified model
            
            Args:
                mid (str): original model
            
                new_mid (str): the model-id for the copied model
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def create_model(self, mid: str, mtype: RegionModelType, gcd: GeoCellDataVector) -> bool:
        """
        create the model
            
            Args:
                mid (str): model identifier
            
                mtype (enum rmodel_type): rmodel_type.pt_gs_k rmodel_type.pt_gs_k_opt or similar model type
            
                gcd (GeoCellDataVector): strongly typed list of GeoCellData
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def fx(self, mid: str, fx_args: str) -> bool:
        """
        Invoke the serverside fx callback, on the exlusively locked model given by mid
            
            Args:
                mid (str): model identifier
            
                fx_args (str): args to the callback, typically a json string
            
            Returns:
                bool: . the server side evaluation result, as pr. callback return value
            
        """
        ...

    @staticmethod
    def get_catchment_parameter(*args, **kwargs):
        """
        Get the effective catchment parameter for the specified catchment
            
            Args:
                mid (str): model identifier
            
                cid (int): catchment identifier
            
            Returns:
                XXXParameter: . The catchment parameter for specified catchment
            
        """
        ...

    def get_charge(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the charge
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_description(self, mid: str) -> str:
        """
        returns the user specified description string, typically a json formatted string.
            
            Args:
                mid (str): model identifier
            
            Returns:
                str: . The  description(json string) for the specified model
            
        """
        ...

    def get_discharge(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the discharge
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_geo_cell_data(self, mid: str) -> GeoCellDataVector:
        """
        returns the geo cell data vector for the specified model.
            
            Args:
                mid (str): model identifier
            
            Returns:
                GeoCellDataVector: . The GeoCellDataVector for the specified model
            
        """
        ...

    def get_interpolation_parameter(self, mid: str) -> InterpolationParameter:
        """
        returns the current interpolation parameters  used to interpolate region-env to the cells.
            
            Args:
                mid (str): model identifier
            
            Returns:
                InterpolationParameter: . The interpolation parameter for the specified model
            
        """
        ...

    def get_model_ids(self) -> StringVector:
        """
        returns a list of model ids (mids) that is alive and known at the remote server
            
            Returns:
                StringList: . Strongly typed list of strings naming the available models on the drms
            
        """
        ...

    def get_precipitation(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the precipitation
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_radiation(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the radiation
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_region_env(self, mid: str) -> ARegionEnvironment:
        """
        returns the current region-env for the specified model.
            
            Args:
                mid (str): model identifier
            
            Returns:
                ARegionEnvironment: . The region envirionment for the specified model
            
        """
        ...

    @staticmethod
    def get_region_parameter(*args, **kwargs):
        """
        Get the region model parameter settings
            
            Args:
                mid (str): model identifier
            
            Returns:
                XXXParameter: . The region parameter for the model
            
        """
        ...

    def get_rel_hum(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the rel_hum
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_server_version(self) -> str:
        """
        returns the server version
            
            Returns:
                str: version. Server version string
            
        """
        ...

    def get_snow_sca(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the snow_sca
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_snow_swe(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the snow_swe
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    @staticmethod
    def get_state(*args, **kwargs):
        """
        Extract cell state for the optionaly specified catchment ids, cids
            
            Args:
                mid (str): model identifier
            
                indexes (IntVector): list of catchment-id's, if empty, extract all
            
            Returns:
                xStateWithIdVector: . strongly typed list of stack state with identifier for the cells
            
        """
        ...

    def get_temperature(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the temperature
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def get_time_axis(self, mid: str) -> TimeAxis:
        """
        Get the current time-axis of the model
            
            Args:
                mid (str): model identifier
            
            Returns:
                TimeAxis: . The region model time-axis
            
        """
        ...

    def get_wind_speed(self, mid: str, indexes: Union[IntVector,list[int],range], index_type: stat_scope = stat_scope.catchment) -> TimeSeries:
        """
        get the wind speed
            
            Args:
                mid (str): model identifier
            
                indexes (List[int]): indexes to be retrieved, the index_type tells if it's cells or catchment indexes
            
                index_type (stat_scope): stat_scope.(cell|catchment) denotes the type of indexes passed. cell returns sum of cell indexes, catchment returns sum of catchments
            
            Returns:
                TimeSeries: . returns sum  for catcment_ids
            
        """
        ...

    def has_catchment_parameter(self, mid: str, cid: object) -> bool:
        """
        Get the current time-axis of the model
            
            Args:
                mid (str): model identifier
            
                cid (int): catchment identifier
            
            Returns:
                bool: . True if catchment has its own parameter setting
            
        """
        ...

    def is_calculated(self, mid: str, catchment_id: object) -> bool:
        """
        using the catchment_calculation_filter to decide if discharge etc. are calculated.
            
            Args:
                mid (str): model identifier
            
                cid (int): catchment id
            
            Returns:
                bool: . true if catchment id is calculated during runs
            
        """
        ...

    def is_cell_env_ts_ok(self, mid: str) -> bool:
        """
        check if all values in cell.env_ts for selected calculation-filter is non-nan, i.e. valid numbers
            
            Args:
                mid (str): model identifier
            
            Returns:
                bool: . true if ok, false if not ok
            
        """
        ...

    def remove_catchment_parameter(self, mid: str, cid: object) -> None:
        """
        Remove the specified catchments local parameter, and set it to the region parameter
            
            Args:
                mid (str): model identifier
            
                cid (int): catchment identifier
            
        """
        ...

    def remove_model(self, mid: str) -> bool:
        """
        remove the specified model
            
            Args:
                mid (str): model identifier
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def rename_model(self, mid: str, new_mid: str) -> bool:
        """
        rename the specified model
            
            Args:
                mid (str): model identifier
            
                new_mid (str): the new wanted model-id for the model
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def revert_to_initial_state(self, mid: str) -> bool:
        """
        revert model to initial state, notice that this 
            require that the set_current_state_as_initial_state() or run_cells() is called.
            The latter will set initial state if not already done.
            
            Args:
                mid (str): model identifier
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def run_cells(self, mid: str, use_ncore: object = 0, start_step: object = 0, n_steps: object = 0) -> bool:
        """
        Run calculation over the specified time axis. Requires that cells and environment have been initialized and interpolated.
            
            Args:
                mid (str): model identifier
            
                use_ncore (int): default 0, means autodetect core, otherwise you can specify 1..n
            
                start_step (int): default 0, from start step of time-axis
            
                n_steps (int): default 0, means run entire time-axis, 1..n means number of steps from start_step
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def run_interpolation(self, mid: str, ip_parameter: InterpolationParameter, ta: TimeAxis, r_env: ARegionEnvironment, best_effort: object) -> bool:
        """
        initializes the cell environment and project region environment time_series to cells.
            
            Args:
                mid (str): model identifier
            
                ip_parameter (InterpolationParameter): interpolation parameter
            
                ta (TimeAxis): time axis
            
                r_env (ARegionEnvironment): region environment containing all geo-located sources
            
                best_effort (bool): best effort type interpolation
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_catchment_calculation_filter(self, mid: str, catchment_id_list: Union[IntVector,list[int],range]) -> bool:
        """
        set catchment calculation filter
            
            Args:
                mid (str): model identifier
            
                catchment_id_list (List[int]): list of catchment indexes
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_catchment_parameter(self, mid: str, parameter: object, cid: object) -> bool:
        """
        set catchment parameter
            
            Args:
                mid (str): model identifier
            
                parameter (XXXXParameter): strongly typed method stack parameter
            
                cid (int): catchment id that should get the supplied parameter
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_cell_environment(self, mid: str, ta: TimeAxis, r_env: ARegionEnvironment) -> bool:
        """
        Set the forcing data cell enviroment (cell.env_ts.* )
            
            The method initializes the cell environment, that keeps temperature, precipitation etc
            for all the cells.
            The region-model time-axis is set to the supplied time-axis, so that
            the the region model is ready to run cells, using this time-axis.
            
            There are strict requirements to the content of the `region_env` parameter:
            
             - rm.cells[i].mid_point()== region_env.temperature[i].mid_point() for all i
             - similar for precipitation,rel_hum,radiation,wind_speed
            
            So same number of forcing data, in the same order and geo position as the cells.
            Tip: If time_axis is equal to the forcing time-axis, it is twice as fast.
            
            
            Args:
                mid (str): model identifier
            
                ta (TimeAxis): specifies the time-axisfor the region-model, and thus the cells
            
                r_env (ARegionEnvironment): A region environment with ready to use forcing data for all the cells.
            
            Returns:
                bool: success. true if successfull, raises exception otherwise
            
        """
        ...

    def set_current_state_as_initial_state(self, mid: str) -> bool:
        """
        set current state as initial state for the region model.
            the method .revert_to_initial_state will restore this state.
            
            
            Args:
                mid (str): model identifier
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_description(self, mid: str, description: str) -> bool:
        """
        Set the user specified description string, typically a json formatted string.
            
            Args:
                mid (str): model identifier
            
            Returns:
                bool: . True if successful
            
        """
        ...

    def set_region_parameter(self, mid: str, parameter: object) -> bool:
        """
        set region parameter
            
            Args:
                mid (str): model identifier
            
                parameter (XXXXParameter): strongly typed method stack parameter
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_snow_sca_swe_collection(self, mid: str, catchment_id: object, on_or_off: object) -> bool:
        """
        enable/disable collection of snow sca|sca for calibration purposes
            
            Args:
                mid (str): model identifier
            
                catchment_id (int): to enable snow calibration for, -1 means turn on/off for all
            
                on_or_off (bool): on_or_off true|or false
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_state(self, mid: str, state: object) -> bool:
        """
        set the cell state
            
            Args:
                mid (str): model identifier
            
                state (StateWithIdVector): strongly typed list of stack state with id vector
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def set_state_collection(self, mid: str, catchment_id: object, on_or_off: object) -> bool:
        """
        enable state collection for specified or all cells
            
            Args:
                mid (str): model identifier
            
                catchment_id (int): to enable state collection for, -1 means turn on/off for all
            
                on_or_off (bool): on_or_off true|or false
            
            Returns:
                bool: . true if succeeded
            
        """
        ...

    def start_calibration(self, mid: str, p_start: object, p_min: object, p_max: object, spec: TargetSpecificationVector, opt: CalibrationOption) -> bool:
        """
        Start calibration with specified parameters for the model given by 'mid'
            
            Args:
                mid (): model identifier
            
                p_start (): starting parameters
            
                p_min (): min-range for parameters
            
                p_max (): max-range for parameters
            
                spec (): goal function specification, including reference series and weights
            
                opt (): optimizer options, like BOBYQA, GLOBAL,SCEUA or DREAM etc.
            
            Returns:
                bool: . indicating success
            
        """
        ...



class DrmServer:
    """
    The DRMS, distributed region model server provides the server part that implements
    the needed functionality to create, run and get results from hydrology region-models
    .Togehter with the DrmClient, it allow the Shyft hydrology region-model operations to be
    scaled out on several servers, giving providing support for system and analysis of any size
    """
    @property
    def alive_connections(self)->int:
        """
         returns currently alive connections to the server
        """
        ...
    @property
    def fx(self)->Any:
        """
        Callable[[str,str],bool]: server-side callable function(lambda) that takes two parameters:
        mid :  the model id
        fx_arg: arbitrary string to pass to the server-side function
        The server-side fx is called when the client (or web-api) invoke the c.fx(mid,fx_arg).
        The signature of the callback function should be fx_cb(mid:str, fx_arg:str)->bool
        This feature is simply enabling the users to tailor server-side functionality in python!
        NOTE: Currently we do take exclusive lock on the model id, so that we avoid race conditions
        Examples:
        
        >>> from shyft.hydrology import DrmServer
        >>> s=DrmServer()
        >>> def my_fx(mid:str, fx_arg:str)->bool:
        >>>     print(f'invoked with mid={mid} fx_arg={fx_arg}')
        >>>   # note we can use captured Server s here!>>>     return True
        >>> # and then bind the function to the callback
        >>> s.fx=my_fx
        >>> s.start_server()
        >>> : # later using client from anywhere to invoke the call
        >>> fx_result=c.fx('my_model_id', 'my_args')
        """
        ...
    @fx.setter
    def fx(self, value:Any)->None:
        ...
    def __init__(self) -> None:
        """
        Creates a server object .
            
        """
        ...

    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
        ...

    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
            
        """
        ...

    @staticmethod
    def get_model(*args, **kwargs):
        """
        returns the named model, so it can be inspected/modified
            WIP: needs to be sure that there is no activity on the model from the remote side
            
            
            Returns:
                Model: . Of the type as set by the client
            
        """
        ...

    def get_model_ids(self) -> StringVector:
        """
        returns a list of model ids (mids) that is alive and known in the server
            
            Returns:
                StringList: . Strongly typed list of strings naming the available models on the drms
            
        """
        ...

    def is_running(self) -> bool:
        """
        true if server is listening and running
            
            See also:
                start_server()
            
        """
        ...

    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
            
            Args:
                ip (str): ip or host-name to start listening on
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_listening_port(self, port_no: object) -> None:
        """
        set the listening port for the service
            
            Args:
                port_no (int): a valid and available tcp-ip port number to listen on.
                typically it could be 20000 (avoid using official reserved numbers)
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_max_connections(self, max_connect: object) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
            
            Args:
                max_connect (int): maximum number of connections before denying more connections
            
            See also:
                get_max_connections()
            
        """
        ...

    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),is_running
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
        """
        ...

    def stop_server(self, ms: object = 2000) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                start_server()
            
        """
        ...



class FlowAdjustResult:
    """
    The result type of region-model .adjust_state_to_target_flow(..) method
    """
    @property
    def diagnostics(self)->str:
        """
         If tuning failed, the diagnostics of failure, zero length/empty if success
        """
        ...
    @diagnostics.setter
    def diagnostics(self, value:str)->None:
        ...
    @property
    def q_0(self)->float:
        """
         The flow m3/s from selected catchments before tuning
        """
        ...
    @q_0.setter
    def q_0(self, value:float)->None:
        ...
    @property
    def q_r(self)->float:
        """
         The obtaioned flow m3/s after tuning
        """
        ...
    @q_r.setter
    def q_r(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class GammaSnowCalculator:
    def __init__(self) -> None:
        ...

    def step(self, state: GammaSnowState, response: GammaSnowResponse, t: Union[time,float,int], dt: Union[time,float,int], parameter: GammaSnowParameter, temperature: object, radiation: object, precipitation: object, wind_speed: object, rel_hum: object, forest_fraction: object, altitude: object) -> None:
        """
        Step the snow model forward from time t to t+dt, given state, parameters and input.
            
            Updates the state and response upon return.
            
            
            Args:
                state (GammaSnowState):  param state state of type S,in/out, ref template parameters
            
                response (GammaSnowResponse): the response , out parameter.
            
                t (float): ttemperature degC, considered constant over timestep dt
            
                dt (time): length of time-step, seconds
            
                param (GammaSnowParameter):  response result of type R, output only, ref. template parameters
            
            
                radiation (float):  in W/m2, considered constant over timestep
            
                precipitation (float):  in mm/h
            
                wind_speed (float):  in m/s
            
                rel_hum (float):  range  0..1
            
                forest_fraction (float): range 0..1, influences calculation of effective snow_cv
            
                altitude (float):  0..x [m], influences calculation of effective_snow_cv
            
        """
        ...



class GammaSnowParameter:
    @property
    def calculate_iso_pot_energy(self)->float:
        """
         Whether or not to calculate the potential energy flux,default=false
        """
        ...
    @calculate_iso_pot_energy.setter
    def calculate_iso_pot_energy(self, value:float)->None:
        ...
    @property
    def fast_albedo_decay_rate(self)->float:
        """
         Albedo decay rate during melt [days],default=5.0
        """
        ...
    @fast_albedo_decay_rate.setter
    def fast_albedo_decay_rate(self, value:float)->None:
        ...
    @property
    def glacier_albedo(self)->float:
        """
         Glacier ice fixed albedo, default=0.4
        """
        ...
    @glacier_albedo.setter
    def glacier_albedo(self, value:float)->None:
        ...
    @property
    def initial_bare_ground_fraction(self)->float:
        """
         Bare ground fraction at melt onset,default= 0.04
        """
        ...
    @initial_bare_ground_fraction.setter
    def initial_bare_ground_fraction(self, value:float)->None:
        ...
    @property
    def max_albedo(self)->float:
        """
         Maximum albedo value,default=0.9
        """
        ...
    @max_albedo.setter
    def max_albedo(self, value:float)->None:
        ...
    @property
    def max_water(self)->float:
        """
         Maximum liquid water content,default=0.1
        """
        ...
    @max_water.setter
    def max_water(self, value:float)->None:
        ...
    @property
    def min_albedo(self)->float:
        """
         Minimum albedo value,default=0.6
        """
        ...
    @min_albedo.setter
    def min_albedo(self, value:float)->None:
        ...
    @property
    def n_winter_days(self)->int:
        """
         number of winter-days, default 221
        """
        ...
    @n_winter_days.setter
    def n_winter_days(self, value:int)->None:
        ...
    @property
    def slow_albedo_decay_rate(self)->float:
        """
         Albedo decay rate in cold conditions [days],default=5.0
        """
        ...
    @slow_albedo_decay_rate.setter
    def slow_albedo_decay_rate(self, value:float)->None:
        ...
    @property
    def snow_cv(self)->float:
        """
         Spatial coefficient variation of fresh snowfall, default= 0.4
        """
        ...
    @snow_cv.setter
    def snow_cv(self, value:float)->None:
        ...
    @property
    def snow_cv_altitude_factor(self)->float:
        """
         default=0.0, [1/m]
        the effective snow_cv gets an additional value of altitude[m]* snow_cv_altitude_factor
        """
        ...
    @snow_cv_altitude_factor.setter
    def snow_cv_altitude_factor(self, value:float)->None:
        ...
    @property
    def snow_cv_forest_factor(self)->float:
        """
         default=0.0, [ratio]
        the effective snow_cv gets an additional value of geo.forest_fraction()*snow_cv_forest_factor
        """
        ...
    @snow_cv_forest_factor.setter
    def snow_cv_forest_factor(self, value:float)->None:
        ...
    @property
    def snowfall_reset_depth(self)->float:
        """
         Snowfall required to reset albedo [mm],default=5.0
        """
        ...
    @snowfall_reset_depth.setter
    def snowfall_reset_depth(self, value:float)->None:
        ...
    @property
    def surface_magnitude(self)->float:
        """
         Surface layer magnitude,default=30.0
        """
        ...
    @surface_magnitude.setter
    def surface_magnitude(self, value:float)->None:
        ...
    @property
    def tx(self)->float:
        """
         default= -0.5 [degC]
        """
        ...
    @tx.setter
    def tx(self, value:float)->None:
        ...
    @property
    def wind_const(self)->float:
        """
         Intercept in turbulent wind function,default=1.0
        """
        ...
    @wind_const.setter
    def wind_const(self, value:float)->None:
        ...
    @property
    def wind_scale(self)->float:
        """
         Slope in turbulent wind function default=2.0 [m/s]
        """
        ...
    @wind_scale.setter
    def wind_scale(self, value:float)->None:
        ...
    @property
    def winter_end_day_of_year(self)->int:
        """
         Last day of accumulation season,default= 100
        """
        ...
    @winter_end_day_of_year.setter
    def winter_end_day_of_year(self, value:int)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, winter_end_day_of_year: object, initial_bare_ground_fraction: object, snow_cv: object, tx: object, wind_scale: object, wind_const: object, max_water: object, surface_magnitude: object, max_albedo: object, min_albedo: object, fast_albedo_decay_rate: object, slow_albedo_decay_rate: object, snowfall_reset_depth: object, glacier_albedo: object) -> None:
        """
        specifying most of the parameters
        """
        ...

    def effective_snow_cv(self, forest_fraction: object, altitude: object) -> float:
        """
        returns the effective snow cv, taking the forest_fraction and altitude into the equations using corresponding factors
        """
        ...

    def is_snow_season(self, t: Union[time,float,int]) -> bool:
        """
        returns true if specified t is within the snow season, e.g. sept.. winder_end_day_of_year
        """
        ...

    def is_start_melt_season(self, t: Union[time,float,int], dt: Union[time,float,int] = 0) -> bool:
        """
        true if specified interval t day of year is wind_end_day_of_year
        """
        ...



class GammaSnowResponse:
    """
    The response(output) from gamma-snow for one time-step
    """
    @property
    def outflow(self)->float:
        """
        Water out from the snow pack [mm/h]
        """
        ...
    @outflow.setter
    def outflow(self, value:float)->None:
        ...
    @property
    def sca(self)->float:
        """
        Snow covered area [0..1]
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def storage(self)->float:
        """
        Snow storage in [mm] over the area
        """
        ...
    @storage.setter
    def storage(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class GammaSnowState:
    """
    The state description of the GammaSnow routine
    """
    @property
    def acc_melt(self)->float:
        """
        acc_melt (Accumulated melt depth) [mm],default = 0.0
        """
        ...
    @acc_melt.setter
    def acc_melt(self, value:float)->None:
        ...
    @property
    def albedo(self)->float:
        """
        albedo (Broadband snow reflectivity fraction),default = 0.4
        """
        ...
    @albedo.setter
    def albedo(self, value:float)->None:
        ...
    @property
    def alpha(self)->float:
        """
        alpha (Dynamic shape state in the SDC),default = 1.26
        """
        ...
    @alpha.setter
    def alpha(self, value:float)->None:
        ...
    @property
    def iso_pot_energy(self)->float:
        """
        iso_pot_energy (Accumulated energy assuming isothermal snow surface) [J/m2],default = 0.0
        """
        ...
    @iso_pot_energy.setter
    def iso_pot_energy(self, value:float)->None:
        ...
    @property
    def lwc(self)->float:
        """
        lwc (liquid water content) [mm],default = 0.1
        """
        ...
    @lwc.setter
    def lwc(self, value:float)->None:
        ...
    @property
    def sdc_melt_mean(self)->float:
        """
        sdc_melt_mean  (Mean snow storage at melt onset) [mm],default = 0.0
        """
        ...
    @sdc_melt_mean.setter
    def sdc_melt_mean(self, value:float)->None:
        ...
    @property
    def surface_heat(self)->float:
        """
        surface_heat (Snow surface cold content) [J/m2],default = 30000.0
        """
        ...
    @surface_heat.setter
    def surface_heat(self, value:float)->None:
        ...
    @property
    def temp_swe(self)->float:
        """
        temp_swe (Depth of temporary new snow layer during spring) [mm],default = 0.0
        """
        ...
    @temp_swe.setter
    def temp_swe(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, albedo: object, lwc: object, surface_heat: object, alpha: object, sdc_melt_mean: object, acc_melt: object, iso_pot_energy: object, temp_swe: object) -> None:
        """
        Construct gamma snow state with supplied parameters
        """
        ...



class GeoCellData:
    """
    Represents common constant geo_cell properties across several possible models and cell assemblies.
    The idea is that most of our algorithms uses one or more of these properties,
    so we provide a common aspect that keeps this together.
    Currently it keep these items:
    - mid-point geo_point, (x,y,z) (default 0)
    - TIN data 
    - the area in m^2, (default 1000 x 1000 m^2)
    - land_type_fractions (unspecified=1)
    - catchment_id   def (-1)
    - radiation_slope_factor def 0.9
    - routing_info def(0,0.0), i.e. not routed and hydrological distance=0.0m
    """
    @property
    def epsg_id(self)->int:
        """
         the epsg-id that should consistently identify the geo projection for all cells in a region model
        """
        ...
    @epsg_id.setter
    def epsg_id(self, value:int)->None:
        ...
    @property
    def routing_info(self)->RoutingInfo:
        """
         routing information for the cell keep destination id and hydrological distance to destination
        """
        ...
    @routing_info.setter
    def routing_info(self, value:RoutingInfo)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, mid_point: GeoPoint, area: object, catchment_id: object, radiation_slope_factor: object, land_type_fractions: LandTypeFractions, routing_info: RoutingInfo) -> None:
        """
        Constructs a GeoCellData with all parameters specified
            
            Args:
                mid_point (GeoPoint): specifies the x,y,z mid-point of the cell-area
            
                area (float): area in unit [m^2]
            
                catchment_id (int): catchment-id that this cell is a part of
            
                radiation_slope_factor (float): aspect dependent factor used to calculate the effective radiation for this cell,range 0.0..1.0
            
                land_type_fractions (LandTypeFractions): specifies the fractions of glacier, forrest, lake and reservoirs for this cell
            
                routing_info (RoutingInfo): Specifies the destination routing network-node and velocity for this cell
            
        
        """
        ...
    @overload
    def __init__(self, p1: GeoPoint, p2: GeoPoint, p3: GeoPoint, epsg_id: object, catchment_id: object, land_type_fractions: LandTypeFractions, routing_info: RoutingInfo) -> None:
        """
        Constructs a TIN-based GeoCellData with all parameters specified, slope/aspect etc. is based on TIN
            
            Args:
                p1 (GeoPoint): specifies the first vertex in TIN
            
                p2 (GeoPoint): specifies the second vertex in TIN
            
                p3 (GeoPoint): specifies the third vertex in TIN
            
                epsg_id (int): specifies the geo projection as epsg-id, used for projection from cartesian coord to long-lat  algorithms that needs this(e.g. radiation, TIN-models)
            
                catchment_id (int): catchment-id that this cell is a part of
            
                land_type_fractions (LandTypeFractions): specifies the fractions of glacier, forrest, lake and reservoirs for this cell
            
                routing_info (RoutingInfo): Specifies the destination routing network-node and velocity for this cell
            
        """
        ...

    def area(self) -> float:
        """
        returns the effective area, as projected to the horisontal plane, in m^2
        """
        ...

    def aspect(self) -> float:
        """
        returns aspect, deg
        """
        ...

    def catchment_id(self) -> int:
        """
        returns the current catchment_id
        """
        ...

    def is_tin(self) -> bool:
        """
        returns true if the geo-cell is fully specified as a TIN model
        """
        ...

    def land_type_fractions_info(self) -> LandTypeFractions:
        """
        land_type_fractions
        """
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns the mid_point
        """
        ...

    def radiation_slope_factor(self) -> float:
        """
        radiation slope factor
        """
        ...

    def set_catchment_id(self, catchment_id: object) -> None:
        """
        Set the catchment_id to specified value
            Note: Only use this method *before* a region-model is created
            
            Args:
                catchment_id (int): Catchment-id for this cell
            
        """
        ...

    def set_land_type_fractions(self, ltf: LandTypeFractions) -> None:
        """
        set new LandTypeFractions
        """
        ...

    def set_tin_data(self, vertexes: GeoPointVector) -> None:
        """
        set TIN data from vector of vertexes
        """
        ...

    def slope(self) -> float:
        """
        returns slope, deg
        """
        ...

    def surface_area(self) -> float:
        """
        returns the real surface area in m^2
        """
        ...

    def vertexes(self) -> GeoPointVector:
        """
        returns vector of tin vertexes
        """
        ...



class GeoCellDataClient:
    """
    The client api for the shyft.hydrology geo_cell_data model service.
    """
    @property
    def host_port(self)->str:
        """
         Endpoint network address of the remote server.
        """
        ...
    @property
    def is_open(self)->bool:
        """
         If the connection to the remote server is (still) open.
        """
        ...
    @property
    def reconnect_count(self)->int:
        """
         Number of reconnects to the remote server that have been performed.
        """
        ...
    @property
    def timeout_ms(self)->int:
        """
         Timout for remote server operations, in number milliseconds.
        """
        ...
    def __init__(self, host_port: str, timeout_ms: object) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
        ...

    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
            
        """
        ...

    def get_model_infos(self, mids: Union[IntVector,list[int],range], created_in: UtcPeriod = "[not-valid-period>") -> ModelInfoVector:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
            
            Args:
                mids (IntVector): empty = all, or a list of known exisiting model-identifiers
            
                created_in (UtcPeriod): For which period you are interested in model-infos.
            
            Returns:
                ModelInfoVector: model_infos. Strongly typed list of ModelInfo
            
        """
        ...

    def read_model(self, mid: object) -> GeoCellDataModel:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def read_models(self, mids: Union[IntVector,list[int],range]) -> GeoCellDataModelVector:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mids (Int64Vector): A strongly typed list of ints, the model-identifers for the wanted models
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def remove_model(self, mid: object) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                int: ec. 0 or error-code?
            
        """
        ...

    def store_model(self, m: GeoCellDataModel, mi: ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
            
            Args:
                m (Model): The model to store
            
                mi (ModelInfo): The model-info to store for the model
            
            Returns:
                int: mid. model-identifier for the stored model and model-info
            
        """
        ...

    def update_model_info(self, mid: object, mi: ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
            
            Args:
                mid (int): model-identifer
            
                mi (ModelInfo): The new updated model-info
            
            Returns:
                bool: ok. true if success
            
        """
        ...



class GeoCellDataModel:
    """
    A model, with mandatory identifier and geo-cell-data vector.
    The optional filled in epsg, polygon and json members can provide
    useful information on how it was created, or can be presented.
    """
    @property
    def epsg_id(self)->int:
        """
         EPSG code for the polygon coordinate reference system
        """
        ...
    @epsg_id.setter
    def epsg_id(self, value:int)->None:
        ...
    @property
    def geo_cell_data(self)->GeoCellDataVector:
        """
         a strongly typed list of geo-cell-data, the geo-personality of Shyft cells
        """
        ...
    @geo_cell_data.setter
    def geo_cell_data(self, value:GeoCellDataVector)->None:
        ...
    @property
    def id(self)->int:
        """
         unique model identifier
        """
        ...
    @id.setter
    def id(self, value:int)->None:
        ...
    @property
    def json(self)->str:
        """
         extra useful json formatted information
        """
        ...
    @json.setter
    def json(self, value:str)->None:
        ...
    @property
    def polygon(self)->GeoPointVector:
        """
         convex hull for the geo_cell_data
        """
        ...
    @polygon.setter
    def polygon(self, value:GeoPointVector)->None:
        ...
    def __init__(self) -> None:
        ...



class GeoCellDataModelVector:
    """
    A strongly typed list of GeoCellDataModel's
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: GeoCellDataModelVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class GeoCellDataServer:
    """
    The server-side component for the shyft.hydrology geo_cell_data model repository.
    """
    def __init__(self, root_dir: str) -> None:
        """
        Creates a server object that serves models from root_dir.
            The root_dir will be create if it does not exists.
            
            Args:
                root_dir (str): Path to the root-directory that keeps/will keep the model-files
            
        """
        ...

    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
        ...

    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
            
        """
        ...

    def is_running(self) -> bool:
        """
        true if server is listening and running
            
            See also:
                start_server()
            
        """
        ...

    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
            
            Args:
                ip (str): ip or host-name to start listening on
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_listening_port(self, port_no: object) -> None:
        """
        set the listening port for the service
            
            Args:
                port_no (int): a valid and available tcp-ip port number to listen on.
                typically it could be 20000 (avoid using official reserved numbers)
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_max_connections(self, max_connect: object) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
            
            Args:
                max_connect (int): maximum number of connections before denying more connections
            
            See also:
                get_max_connections()
            
        """
        ...

    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),is_running
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
        """
        ...

    def stop_server(self, timeout: object = 1000) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                start_server()
            
        """
        ...



class GeoCellDataVector:
    """
    A vector, list, of GeoCellData
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, const_ref_v: GeoCellDataVector) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    @staticmethod
    def deserialize(*args, **kwargs):
        """
        deserialize from a binary to GeoCellDataVector
            
        """
        ...

    def extend(self, arg2: object) -> None:
        ...

    def polygon_buffer(self, distance: object) -> GeoPointVector:
        """
        Computes the polygon_buffer of tin points, or mid-points, using boost::geometry::buffer algorithm
            If number of points is large, currently > 50,  a reduction algorithm using convex_hull is first applied
            
            Args:
                distance (float): metric[m] buffer distance
            
            Returns:
                GeoPointVector: polygon. A closed polygon envelope around the cells in the model
            
        """
        ...

    @staticmethod
    def serialize(*args, **kwargs):
        """
        serialize to a binary byte vector
            
        """
        ...



class GeoPointSource:
    """
    GeoPointSource contains common properties, functions
    for the point sources in Shyft.
    Typically it contains a GeoPoint (3d position), plus a time-series
    """
    @property
    def geo_ts(self)->GeoTimeSeries:
        """
         returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
        ...
    @property
    def mid_point_(self)->Any:
        """
        reference to internal mid_point
        """
        ...
    @mid_point_.setter
    def mid_point_(self, value:Any)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier, string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, midpoint: GeoPoint, ts: TimeSeries) -> None:
        ...
    @overload
    def __init__(self, geo_ts: GeoTimeSeries) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns a copy of mid_point
        """
        ...



class GeoPointSourceVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, src: GeoPointSourceVector) -> None:
        """
        clone src
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class GlacierMeltParameter:
    @property
    def direct_response(self)->float:
        """
         fraction that goes as direct response, (1-fraction) is routed through soil/kirchner routine,default=0.0
        """
        ...
    @direct_response.setter
    def direct_response(self, value:float)->None:
        ...
    @property
    def dtf(self)->float:
        """
         degree timestep factor, default=6.0 [mm/day/degC]
        """
        ...
    @dtf.setter
    def dtf(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, dtf: object, direct_response: object = 0.0) -> None:
        """
        create parameter object with specified values
        """
        ...



class GoalFunctionClient:
    """
    The client api for the shyft.hydrology goal-function model service.
    """
    @property
    def host_port(self)->str:
        """
         Endpoint network address of the remote server.
        """
        ...
    @property
    def is_open(self)->bool:
        """
         If the connection to the remote server is (still) open.
        """
        ...
    @property
    def reconnect_count(self)->int:
        """
         Number of reconnects to the remote server that have been performed.
        """
        ...
    @property
    def timeout_ms(self)->int:
        """
         Timout for remote server operations, in number milliseconds.
        """
        ...
    def __init__(self, host_port: str, timeout_ms: object) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
        ...

    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
            
        """
        ...

    def get_model_infos(self, mids: Union[IntVector,list[int],range], created_in: UtcPeriod = "[not-valid-period>") -> ModelInfoVector:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
            
            Args:
                mids (IntVector): empty = all, or a list of known exisiting model-identifiers
            
                created_in (UtcPeriod): For which period you are interested in model-infos.
            
            Returns:
                ModelInfoVector: model_infos. Strongly typed list of ModelInfo
            
        """
        ...

    def read_model(self, mid: object) -> GoalFunctionModel:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def read_models(self, mids: Union[IntVector,list[int],range]) -> GoalFunctionModelVector:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mids (Int64Vector): A strongly typed list of ints, the model-identifers for the wanted models
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def remove_model(self, mid: object) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                int: ec. 0 or error-code?
            
        """
        ...

    def store_model(self, m: GoalFunctionModel, mi: ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
            
            Args:
                m (Model): The model to store
            
                mi (ModelInfo): The model-info to store for the model
            
            Returns:
                int: mid. model-identifier for the stored model and model-info
            
        """
        ...

    def update_model_info(self, mid: object, mi: ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
            
            Args:
                mid (int): model-identifer
            
                mi (ModelInfo): The new updated model-info
            
            Returns:
                bool: ok. true if success
            
        """
        ...



class GoalFunctionModel:
    """
    A model identifier and goal-funcions, aka `TargetSpecificationVector`  used in Shyft region model calibrations. This plays the role of the goal-function specification for shyft region-models, so that they
    can be calibrated, and measured against observed quantities.
    """
    @property
    def goal_functions(self)->TargetSpecificationVector:
        """
         The goal functions
        """
        ...
    @goal_functions.setter
    def goal_functions(self, value:TargetSpecificationVector)->None:
        ...
    @property
    def id(self)->int:
        """
         unique model id
        """
        ...
    @id.setter
    def id(self, value:int)->None:
        ...
    def __init__(self) -> None:
        ...



class GoalFunctionModelVector:
    """
    A strongly typed list of GoalFunctionModels
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: GoalFunctionModelVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class GoalFunctionServer:
    """
    The server-side component for the goal-function model repository.
    """
    def __init__(self, root_dir: str) -> None:
        """
        Creates a server object that serves models from root_dir.
            The root_dir will be create if it does not exists.
            
            Args:
                root_dir (str): Path to the root-directory that keeps/will keep the model-files
            
        """
        ...

    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
        ...

    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
            
        """
        ...

    def is_running(self) -> bool:
        """
        true if server is listening and running
            
            See also:
                start_server()
            
        """
        ...

    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
            
            Args:
                ip (str): ip or host-name to start listening on
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_listening_port(self, port_no: object) -> None:
        """
        set the listening port for the service
            
            Args:
                port_no (int): a valid and available tcp-ip port number to listen on.
                typically it could be 20000 (avoid using official reserved numbers)
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_max_connections(self, max_connect: object) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
            
            Args:
                max_connect (int): maximum number of connections before denying more connections
            
            See also:
                get_max_connections()
            
        """
        ...

    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),is_running
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
        """
        ...

    def stop_server(self, timeout: object = 1000) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                start_server()
            
        """
        ...



class HbvActualEvapotranspirationParameter:
    @property
    def lp(self)->float:
        """
         typical value 150
        """
        ...
    @lp.setter
    def lp(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, lp: object) -> None:
        """
        a new object with specified parameters
        """
        ...



class HbvActualEvapotranspirationResponse:
    @property
    def ae(self)->float:
        """
         ae value
        """
        ...
    @ae.setter
    def ae(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class HbvPhysicalSnowCalculator:
    """
    Generalized quantile based HBV Physical Snow model method
    
    This algorithm uses arbitrary quartiles to model snow. No checks are performed to assert valid input.
    The starting points of the quantiles have to partition the unity, 
    include the end points 0 and 1 and must be given in ascending order.
    """
    def __init__(self, parameter: HbvPhysicalSnowParameter) -> None:
        """
        creates a calculator with given parameter and initial state, notice that state is updated in this call(hmm)
        """
        ...

    def step(self, state: HbvPhysicalSnowState, response: HbvPhysicalSnowResponse, t: Union[time,float,int], dt: Union[time,float,int], temperature: object, rad: object, prec_mm_h: object, wind_speed: object, rel_hum: object) -> None:
        """
        steps the model forward from t to t+dt, updating state and response
        """
        ...



class HbvPhysicalSnowParameter:
    @property
    def calculate_iso_pot_energy(self)->float:
        """
         whether or not to calculate the potential energy flux
        """
        ...
    @calculate_iso_pot_energy.setter
    def calculate_iso_pot_energy(self, value:float)->None:
        ...
    @property
    def cfr(self)->float:
        """
         cfr
        """
        ...
    @cfr.setter
    def cfr(self, value:float)->None:
        ...
    @property
    def fast_albedo_decay_rate(self)->float:
        """
         albedo decay rate during melt [days]
        """
        ...
    @fast_albedo_decay_rate.setter
    def fast_albedo_decay_rate(self, value:float)->None:
        ...
    @property
    def intervals(self)->DoubleVector:
        """
         snow quantiles list default 0, 0.25 0.5 1.0
        """
        ...
    @intervals.setter
    def intervals(self, value:DoubleVector)->None:
        ...
    @property
    def lw(self)->float:
        """
         max liquid water content of the snow
        """
        ...
    @lw.setter
    def lw(self, value:float)->None:
        ...
    @property
    def max_albedo(self)->float:
        """
         maximum albedo value
        """
        ...
    @max_albedo.setter
    def max_albedo(self, value:float)->None:
        ...
    @property
    def min_albedo(self)->float:
        """
         minimum albedo value
        """
        ...
    @min_albedo.setter
    def min_albedo(self, value:float)->None:
        ...
    @property
    def s(self)->DoubleVector:
        """
         snow redistribution factors,default =1.0..
        """
        ...
    @s.setter
    def s(self, value:DoubleVector)->None:
        ...
    @property
    def slow_albedo_decay_rate(self)->float:
        """
         albedo decay rate in cold conditions [days]
        """
        ...
    @slow_albedo_decay_rate.setter
    def slow_albedo_decay_rate(self, value:float)->None:
        ...
    @property
    def snowfall_reset_depth(self)->float:
        """
         snowfall required to reset albedo [mm]
        """
        ...
    @snowfall_reset_depth.setter
    def snowfall_reset_depth(self, value:float)->None:
        ...
    @property
    def surface_magnitude(self)->float:
        """
         surface layer magnitude
        """
        ...
    @surface_magnitude.setter
    def surface_magnitude(self, value:float)->None:
        ...
    @property
    def tx(self)->float:
        """
         threshold temperature determining if precipitation is rain or snow
        """
        ...
    @tx.setter
    def tx(self, value:float)->None:
        ...
    @property
    def wind_const(self)->float:
        """
         intercept in turbulent wind function
        """
        ...
    @wind_const.setter
    def wind_const(self, value:float)->None:
        ...
    @property
    def wind_scale(self)->float:
        """
         slope in turbulent wind function [m/s]
        """
        ...
    @wind_scale.setter
    def wind_scale(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, tx: object, lw: object, cfr: object, wind_scale: object, wind_const: object, surface_magnitude: object, max_albedo: object, min_albedo: object, fast_albedo_decay_rate: object, slow_albedo_decay_rate: object, snowfall_reset_depth: object, calculate_iso_pot_energy: object) -> None:
        """
        create parameter object with specifed values
        
        """
        ...
    @overload
    def __init__(self, snow_redist_factors: Union[DoubleVector,list[float],np.ndarray], quantiles: Union[DoubleVector,list[float],np.ndarray], tx: object, lw: object, cfr: object, wind_scale: object, wind_const: object, surface_magnitude: object, max_albedo: object, min_albedo: object, fast_albedo_decay_rate: object, slow_albedo_decay_rate: object, snowfall_reset_depth: object, calculate_iso_pot_energy: object) -> None:
        """
        create a parameter with snow re-distribution factors, quartiles and optionally the other parameters
        """
        ...

    def set_snow_quantiles(self, quantiles: Union[DoubleVector,list[float],np.ndarray]) -> None:
        ...

    def set_snow_redistribution_factors(self, snow_redist_factors: Union[DoubleVector,list[float],np.ndarray]) -> None:
        ...



class HbvPhysicalSnowResponse:
    @property
    def hps_state(self)->HbvPhysicalSnowState:
        """
         current state instance
        """
        ...
    @hps_state.setter
    def hps_state(self, value:HbvPhysicalSnowState)->None:
        ...
    @property
    def outflow(self)->float:
        """
         from snow-routine in [mm]
        """
        ...
    @outflow.setter
    def outflow(self, value:float)->None:
        ...
    @property
    def sca(self)->float:
        """
         snow-covered area
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def storage(self)->float:
        """
         snow storage [mm]
        """
        ...
    @storage.setter
    def storage(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class HbvPhysicalSnowState:
    @property
    def albedo(self)->float:
        """
         albedo (Broadband snow reflectivity fraction)
        """
        ...
    @albedo.setter
    def albedo(self, value:float)->None:
        ...
    @property
    def iso_pot_energy(self)->float:
        """
         iso_pot_energy (Accumulated energy assuming isothermal snow surface) [J/m2]
        """
        ...
    @iso_pot_energy.setter
    def iso_pot_energy(self, value:float)->None:
        ...
    @property
    def sca(self)->float:
        """
         snow covered area [0..1]
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def sp(self)->DoubleVector:
        """
         snow dry[mm]
        """
        ...
    @sp.setter
    def sp(self, value:DoubleVector)->None:
        ...
    @property
    def surface_heat(self)->float:
        """
         surface_heat (Snow surface cold content) [J/m2]
        """
        ...
    @surface_heat.setter
    def surface_heat(self, value:float)->None:
        ...
    @property
    def sw(self)->DoubleVector:
        """
         snow water[mm]
        """
        ...
    @sw.setter
    def sw(self, value:DoubleVector)->None:
        ...
    @property
    def swe(self)->float:
        """
         snow water equivalent[mm]
        """
        ...
    @swe.setter
    def swe(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, albedo: Union[DoubleVector,list[float],np.ndarray], iso_pot_energy: Union[DoubleVector,list[float],np.ndarray], surface_heat: object, swe: object, sca: object) -> None:
        """
        create a state with specified values
        """
        ...

    def distribute(self, p: HbvPhysicalSnowParameter, force: object = True) -> None:
        """
        Distribute state according to parameter settings.
            
            Args:
                p (HbvPhysicalSnowParameter): descr
            
                force (bool): default true, if false then only distribute if state vectors are of different size than parameters passed
            
            Returns:
                None: . 
            
        """
        ...



class HbvSnowCalculator:
    """
    Generalized quantile based HBV Snow model method
    
    This algorithm uses arbitrary quartiles to model snow. No checks are performed to assert valid input.
    The starting points of the quantiles have to partition the unity, 
    include the end points 0 and 1 and must be given in ascending order.
    """
    def __init__(self, parameter: HbvSnowParameter) -> None:
        """
        creates a calculator with given parameter
        """
        ...

    def step(self, state: HbvSnowState, response: HbvSnowResponse, t0: Union[time,float,int], t1: Union[time,float,int], precipitation: object, temperature: object) -> None:
        """
        steps the model forward from t0 to t1, updating state and response
            
        """
        ...



class HbvSnowParameter:
    @property
    def cfr(self)->float:
        """
         cfr
        """
        ...
    @cfr.setter
    def cfr(self, value:float)->None:
        ...
    @property
    def cx(self)->float:
        """
         temperature index, i.e., melt = cx(t - ts) in mm per degree C
        """
        ...
    @cx.setter
    def cx(self, value:float)->None:
        ...
    @property
    def intervals(self)->DoubleVector:
        """
         snow quantiles list default 0, 0.25 0.5 1.0
        """
        ...
    @intervals.setter
    def intervals(self, value:DoubleVector)->None:
        ...
    @property
    def lw(self)->float:
        """
         max liquid water content of the snow
        """
        ...
    @lw.setter
    def lw(self, value:float)->None:
        ...
    @property
    def s(self)->DoubleVector:
        """
         snow redistribution factors,default =1.0..
        """
        ...
    @s.setter
    def s(self, value:DoubleVector)->None:
        ...
    @property
    def ts(self)->float:
        """
         threshold temperature for melt onset
        """
        ...
    @ts.setter
    def ts(self, value:float)->None:
        ...
    @property
    def tx(self)->float:
        """
         threshold temperature determining if precipitation is rain or snow
        """
        ...
    @tx.setter
    def tx(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, tx: object, cx: object, ts: object, lw: object, cfr: object) -> None:
        """
        create parameter object with specifed values
        
        """
        ...
    @overload
    def __init__(self, snow_redist_factors: Union[DoubleVector,list[float],np.ndarray], quantiles: Union[DoubleVector,list[float],np.ndarray], tx: object, cx: object, ts: object, lw: object, cfr: object) -> None:
        """
        create a parameter with snow re-distribution factors, quartiles and optionally the other parameters
        """
        ...

    def set_snow_quantiles(self, quantiles: Union[DoubleVector,list[float],np.ndarray]) -> None:
        ...

    def set_snow_redistribution_factors(self, snow_redist_factors: Union[DoubleVector,list[float],np.ndarray]) -> None:
        ...



class HbvSnowResponse:
    @property
    def outflow(self)->float:
        """
         from snow-routine in [mm]
        """
        ...
    @outflow.setter
    def outflow(self, value:float)->None:
        ...
    @property
    def snow_state(self)->HbvSnowState:
        """
         swe and snow covered area
        """
        ...
    @snow_state.setter
    def snow_state(self, value:HbvSnowState)->None:
        ...
    def __init__(self) -> None:
        ...



class HbvSnowState:
    @property
    def sca(self)->float:
        """
         snow covered area [0..1]
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def sp(self)->DoubleVector:
        """
         snow dry[mm]
        """
        ...
    @sp.setter
    def sp(self, value:DoubleVector)->None:
        ...
    @property
    def sw(self)->DoubleVector:
        """
         snow water[mm]
        """
        ...
    @sw.setter
    def sw(self, value:DoubleVector)->None:
        ...
    @property
    def swe(self)->float:
        """
         snow water equivalent[mm]
        """
        ...
    @swe.setter
    def swe(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, swe: object, sca: object) -> None:
        """
        create a state with specified values
        """
        ...

    def distribute(self, p: HbvSnowParameter, force: object = True) -> None:
        """
        Distribute state according to parameter settings.
            
            Args:
                p (HbvSnowParameter): descr
            
                force (bool): default true, if false then only distribute if state vectors are of different size than parameters passed
            
            Returns:
                None: . 
            
        """
        ...



class HbvSoilCalculator:
    """
    Computing water through the soil moisture zone of the HBV model.
    
    
    Reference:
    
     * Nils Roar Shlthun: The Nordic HBV model 1996 https://publikasjoner.nve.no/publication/1996/publication1996_07.pdf
    
    
    
    
    Notes:
        Lake and glacier are treated in the stack
    """
    def __init__(self, parameter: HbvSoilParameter, land_fraction: object) -> None:
        """
        creates a calculator with given parameter
        """
        ...

    def step(self, state: HbvSoilState, response: HbvSoilResponse, insoil: object, pe: object, sca: object) -> None:
        """
        One step of the model, given state, parameters and input.
            
            Updates the state and response.
            
            
            Args:
                state (HbvSoilState):  param of type S, in/out, ref template parameters
            
                response (HbvSoilResponse):  param of type R, in/out, ref template parameters
            
                insoil (float): inflow to soil
            
                pe (float): potential evaporation
            
                sca (float): the response , out parameter.
            
                land_fraction (float): Land-fraction of the cell. Not directly used/desribed in ref. 'Nils Roar', but introduced here to adopt to the stack.
            
        """
        ...



class HbvSoilParameter:
    @property
    def beta(self)->float:
        """
         (unitless) exponent in the nonlinear relationship between soil moisture and field capacity. default=1.50738409
        """
        ...
    @beta.setter
    def beta(self, value:float)->None:
        ...
    @property
    def fc(self)->float:
        """
         (mm) maximum water content of the soil (Field Capacity), default=32.2755689
        """
        ...
    @fc.setter
    def fc(self, value:float)->None:
        ...
    @property
    def infmax(self)->float:
        """
         (mm/h) maximum input rate to the soil moisture zone. Excess water will go directly to upper ground water zone. default=2.0
        """
        ...
    @infmax.setter
    def infmax(self, value:float)->None:
        ...
    @property
    def lpdel(self)->float:
        """
         (unitless) dimensionless parameter (<1), from which level evapotranspiration is potential. Some publications use fcdel instead. default= 0.848557794
        """
        ...
    @lpdel.setter
    def lpdel(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, fc: object, beta: object) -> None:
        """
        create parameter object with specifed values
        """
        ...



class HbvSoilResponse:
    @property
    def ae(self)->float:
        """
         (mm/h) Actual evapotranspiration. default=0
        """
        ...
    @ae.setter
    def ae(self, value:float)->None:
        ...
    @property
    def inuz(self)->float:
        """
         (mm/h) Perculation to upper ground water zone. Called `cuz` by Nils Roar. default=0
        """
        ...
    @inuz.setter
    def inuz(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, inuz: object, ae: object) -> None:
        """
        create response object with specifed values
        """
        ...



class HbvSoilState:
    @property
    def sm(self)->float:
        """
         Soil  moisture [mm]. default=10
        """
        ...
    @sm.setter
    def sm(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, sm: object) -> None:
        """
        create a state with specified values
        """
        ...



class HbvTankCalculator:
    """
    Computing water through the HBV ground water zone of the HBV model.
    
    
    Reference:
    
     * Nils Roar Shlthun: The Nordic HBV model 1996 https://publikasjoner.nve.no/publication/1996/publication1996_07.pdf
    
    
    
    
    Notes:
        Lake and glacier are treated in the stack
    """
    def __init__(self, parameter: HbvTankParameter, lake_fraction: object) -> None:
        """
        creates a calculator with given parameters
        """
        ...

    def step(self, state: HbvTankState, response: HbvTankResponse, from_soil: object, precip: object, t2m: object) -> None:
        """
        One step of the model, given state, parameters and input.
            
            Updates the state and response.
            
            
            Args:
                state (HbvSoilState):  param of type S, in/out, ref template parameters
            
                response (HbvSoilResponse):  param of type R, in/out, ref template parameters
            
                from_soil (float): [mm/step] Inflow to upper zone (from soil moisture)
            
                precip (float): [mm/step] Routed precipitation. Assumed that it is corrected for lake fraction already.
            
                t2m (float): [deg C] Air-temperature at 2m (proxy for water temperature)
            
                lake_fraction (float): [unitless] Lake-fraction of the cell. Not directly used/desribed in ref.'Nils Roar', but introduced here to adopt to the stack.
            
        """
        ...



class HbvTankParameter:
    @property
    def ce(self)->float:
        """
         [ mm/deg/day ] Evapotranspiration constant (degree-day-factor). default=0.17/24
        """
        ...
    @ce.setter
    def ce(self, value:float)->None:
        ...
    @property
    def cevpl(self)->float:
        """
         [ unitless ], adjustment factor for potential evapotranspiration on lakes. default=1.1
        """
        ...
    @cevpl.setter
    def cevpl(self, value:float)->None:
        ...
    @property
    def klz(self)->float:
        """
         [ 1/step ] Slow response coefficient lower ground water zone. default=0.05
        """
        ...
    @klz.setter
    def klz(self, value:float)->None:
        ...
    @property
    def kuz0(self)->float:
        """
         [ 1/step ] Slow response coefficient upper ground water zone. default=0.05
        """
        ...
    @kuz0.setter
    def kuz0(self, value:float)->None:
        ...
    @property
    def kuz1(self)->float:
        """
         [ 1/step ] Mid response coefficient upper ground water zone. default=0.1
        """
        ...
    @kuz1.setter
    def kuz1(self, value:float)->None:
        ...
    @property
    def kuz2(self)->float:
        """
         [ 1/step ] Fast response coefficient upper ground water zone. default=0 (not active)
        """
        ...
    @kuz2.setter
    def kuz2(self, value:float)->None:
        ...
    @property
    def perc(self)->float:
        """
         [ mm/step ] Perculation to lower ground water zone. default=0.6
        """
        ...
    @perc.setter
    def perc(self, value:float)->None:
        ...
    @property
    def uz1(self)->float:
        """
         [ mm ] Mid-threshold for upper ground water zone. default=10
        """
        ...
    @uz1.setter
    def uz1(self, value:float)->None:
        ...
    @property
    def uz2(self)->float:
        """
         [ mm ] High-threshold for upper ground water zone. default=50
        """
        ...
    @uz2.setter
    def uz2(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, uz1: object, uz2: object, kuz0: object, kuz1: object, kuz2: object, perc: object, klz: object, ce: object, cevpl: object) -> None:
        """
        create parameter object with specifed values
        """
        ...



class HbvTankResponse:
    @property
    def elake(self)->float:
        """
         [ mm/step ] Evaporation from lake
        """
        ...
    @elake.setter
    def elake(self, value:float)->None:
        ...
    @property
    def perculation(self)->float:
        """
         [ mm/step ] Effective perculation from upper to lower
        """
        ...
    @perculation.setter
    def perculation(self, value:float)->None:
        ...
    @property
    def q(self)->float:
        """
         [mm/step] Total discharge from tank
        """
        ...
    @property
    def qlz(self)->float:
        """
         [mm/step] Total for lower zone
        """
        ...
    @qlz.setter
    def qlz(self, value:float)->None:
        ...
    @property
    def quz(self)->float:
        """
         [mm/step] Total for upper zone
        """
        ...
    @property
    def quz0(self)->float:
        """
         [mm/step] Upper zone slow response
        """
        ...
    @quz0.setter
    def quz0(self, value:float)->None:
        ...
    @property
    def quz1(self)->float:
        """
         [mm/step] Upper zone mid response
        """
        ...
    @quz1.setter
    def quz1(self, value:float)->None:
        ...
    @property
    def quz2(self)->float:
        """
         [mm/step] Upper zone fast response
        """
        ...
    @quz2.setter
    def quz2(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, quz0: object, quz1: object, quz2: object, qlz: object, elake: object, perculation: object) -> None:
        """
        create response object with specifed values
        """
        ...



class HbvTankState:
    @property
    def lz(self)->float:
        """
         Lower ground water zone content [mm]
        """
        ...
    @lz.setter
    def lz(self, value:float)->None:
        ...
    @property
    def uz(self)->float:
        """
         Upper ground water zone content [mm]
        """
        ...
    @uz.setter
    def uz(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, uz: object, lz: object) -> None:
        """
        create a state with specified values
        """
        ...



class IDWParameter:
    """
    IDWParameter is a simple place-holder for IDW parameters used during interpolation process.
    The two most common is max_distance and max_members.
    Additionally it keep distance measure-factor,
    so that the IDW distance is computed as 1 over pow(euclid distance,distance_measure_factor).
    zscale is used to discriminate neighbors that are at different elevation than target point.
    """
    @property
    def distance_measure_factor(self)->float:
        """
         IDW distance is computed as 1 over pow(euclid distance,distance_measure_factor), default=2.0
        """
        ...
    @distance_measure_factor.setter
    def distance_measure_factor(self, value:float)->None:
        ...
    @property
    def max_distance(self)->float:
        """
         [meter] only neighbours within max distance is used for each destination-cell,default= 200000.0
        """
        ...
    @max_distance.setter
    def max_distance(self, value:float)->None:
        ...
    @property
    def max_members(self)->int:
        """
         maximum members,neighbors used to interpolate into a point,default=10
        """
        ...
    @max_members.setter
    def max_members(self, value:int)->None:
        ...
    @property
    def zscale(self)->float:
        """
         Use to weight neighbors having same elevation, default=1.0
        """
        ...
    @zscale.setter
    def zscale(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, max_members: object, max_distance: object, distance_measure_factor: object) -> None:
        """
        create IDW from supplied parameters
        """
        ...



class IDWPrecipitationParameter(IDWParameter):
    """
    For precipitation,the scaling model needs the scale_factor.
    adjusted_precipitation = precipitation* (scale_factor)^(z-distance-in-meters/100.0)
    Ref to IDWParameter for the other parameters
    """
    @property
    def distance_measure_factor(self)->float:
        """
         IDW distance is computed as 1 over pow(euclid distance,distance_measure_factor), default=2.0
        """
        ...
    @distance_measure_factor.setter
    def distance_measure_factor(self, value:float)->None:
        ...
    @property
    def max_distance(self)->float:
        """
         [meter] only neighbours within max distance is used for each destination-cell,default= 200000.0
        """
        ...
    @max_distance.setter
    def max_distance(self, value:float)->None:
        ...
    @property
    def max_members(self)->int:
        """
         maximum members,neighbors used to interpolate into a point,default=10
        """
        ...
    @max_members.setter
    def max_members(self, value:int)->None:
        ...
    @property
    def scale_factor(self)->float:
        """
         ref. formula for adjusted_precipitation,  default=1.02
        """
        ...
    @scale_factor.setter
    def scale_factor(self, value:float)->None:
        ...
    @property
    def zscale(self)->float:
        """
         Use to weight neighbors having same elevation, default=1.0
        """
        ...
    @zscale.setter
    def zscale(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, scale_factor: object, max_members: object, max_distance: object) -> None:
        """
        create IDW from supplied parameters
        """
        ...



class IDWTemperatureParameter(IDWParameter):
    """
    For temperature inverse distance, also provide default temperature gradient to be used
    when the gradient can not be computed, or is outside specified limits.
    if gradient_by_equation is set true, and number of points >3, the temperature gradient computer
    will try to use the 4 closest points and determine the 3d gradient including the vertical gradient.
    In scenarios with constant gradients(vertical/horizontal), this is accurate) 
    If the number of valide nearby points is to few to do the full gradient equation, or the gradient_by_equation is set to False,
    then the two points with maximum vertical distance (required to be more than the minimum_z_distance) is used to compute the gradient.
    If all of the above fails, the default-gradient is used.
    Please note:
    Carefully selecting the maximum horizontal distances, and number of neighbors is required to get
    optimal results.
    In case of non-grid, or sparse-grid scenarios, you might consider setting the parameters to enforce default-gradient
    or you can also consider using the bayesian temperature krieging algorithm if there are few temperature sources
    """
    @property
    def default_temp_gradient(self)->float:
        """
         [degC/m], default=-0.006
        """
        ...
    @default_temp_gradient.setter
    def default_temp_gradient(self, value:float)->None:
        ...
    @property
    def distance_measure_factor(self)->float:
        """
         IDW distance is computed as 1 over pow(euclid distance,distance_measure_factor), default=2.0
        """
        ...
    @distance_measure_factor.setter
    def distance_measure_factor(self, value:float)->None:
        ...
    @property
    def gradient_by_equation(self)->bool:
        """
         if true, gradient is computed using 4 closest neighbors, solving equations to find 3D temperature gradients.
        """
        ...
    @gradient_by_equation.setter
    def gradient_by_equation(self, value:bool)->None:
        ...
    @property
    def gradient_max(self)->float:
        """
         [degC/m] default +0.01, if computed gradient larger than this, the default-gradient is used instead
        """
        ...
    @gradient_max.setter
    def gradient_max(self, value:float)->None:
        ...
    @property
    def gradient_min(self)->float:
        """
         [degC/m] default -0.01, if computed gradient less than this, the default-gradient is used instead
        """
        ...
    @gradient_min.setter
    def gradient_min(self, value:float)->None:
        ...
    @property
    def max_distance(self)->float:
        """
         [meter] only neighbours within max distance is used for each destination-cell,default= 200000.0
        """
        ...
    @max_distance.setter
    def max_distance(self, value:float)->None:
        ...
    @property
    def max_members(self)->int:
        """
         maximum members,neighbors used to interpolate into a point,default=10
        """
        ...
    @max_members.setter
    def max_members(self, value:int)->None:
        ...
    @property
    def minimum_z_distance(self)->float:
        """
         [m] default 50m, the minimum vertical distance requirement for two points to be used for gradient computation
        """
        ...
    @minimum_z_distance.setter
    def minimum_z_distance(self, value:float)->None:
        ...
    @property
    def zscale(self)->float:
        """
         Use to weight neighbors having same elevation, default=1.0
        """
        ...
    @zscale.setter
    def zscale(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, default_gradient: object, max_members: object, max_distance: object, gradient_by_equation: object, minimum_z_distance: object, gradient_min: object, gradient_max: object) -> None:
        """
        construct IDW for temperature as specified by arguments
        """
        ...



class InterpolationParameter:
    """
    The InterpolationParameter keep parameters needed to perform the
    interpolation steps, IDW,BTK etc
    It is used as parameter  in the model.run_interpolation() method
    """
    @property
    def precipitation(self)->IDWPrecipitationParameter:
        """
         IDW parameters for precipitation
        """
        ...
    @precipitation.setter
    def precipitation(self, value:IDWPrecipitationParameter)->None:
        ...
    @property
    def radiation(self)->IDWParameter:
        """
         IDW parameters for radiation
        """
        ...
    @radiation.setter
    def radiation(self, value:IDWParameter)->None:
        ...
    @property
    def rel_hum(self)->IDWParameter:
        """
         IDW parameters for relative humidity
        """
        ...
    @rel_hum.setter
    def rel_hum(self, value:IDWParameter)->None:
        ...
    @property
    def temperature(self)->BTKParameter:
        """
         BTK for temperature (in case .use_idw_for_temperature is false)
        """
        ...
    @temperature.setter
    def temperature(self, value:BTKParameter)->None:
        ...
    @property
    def temperature_idw(self)->IDWTemperatureParameter:
        """
         IDW for temperature(in case .use_idw_for_temperature is true)
        """
        ...
    @temperature_idw.setter
    def temperature_idw(self, value:IDWTemperatureParameter)->None:
        ...
    @property
    def use_idw_for_temperature(self)->bool:
        """
         if true, the IDW temperature is used instead of BTK, useful for grid-input scenarios
        """
        ...
    @use_idw_for_temperature.setter
    def use_idw_for_temperature(self, value:bool)->None:
        ...
    @property
    def wind_speed(self)->IDWParameter:
        """
         IDW parameters for wind_speed
        """
        ...
    @wind_speed.setter
    def wind_speed(self, value:IDWParameter)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, temperature: BTKParameter, precipitation: IDWPrecipitationParameter, wind_speed: IDWParameter, radiation: IDWParameter, rel_hum: IDWParameter) -> None:
        """
        using BTK for temperature
        
        """
        ...
    @overload
    def __init__(self, temperature: IDWTemperatureParameter, precipitation: IDWPrecipitationParameter, wind_speed: IDWParameter, radiation: IDWParameter, rel_hum: IDWParameter) -> None:
        """
        using smart IDW for temperature, typically grid inputs
        """
        ...



class KalmanBiasPredictor:
    """
    A bias predictor using a daily pattern KalmanFilter for temperature (etc.)
    (tbd)
    """
    @property
    def filter(self)->KalmanFilter:
        """
          kalman filter with parameters
        """
        ...
    @property
    def state(self)->KalmanState:
        """
         state of the predictor
        """
        ...
    @state.setter
    def state(self, value:KalmanState)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self) -> None:
        """
        Constructs a bias predictor with default filter, parameters and state
        
        """
        ...
    @overload
    def __init__(self, filter: KalmanFilter) -> None:
        """
        create a bias predictor with specified filter
        
        """
        ...
    @overload
    def __init__(self, filter: KalmanFilter, state: KalmanState) -> None:
        """
        create a bias predictor with specified filter and initial state
        """
        ...

    @staticmethod
    def compute_running_bias_ts(bias_predictor: KalmanBiasPredictor, forecast_ts: TimeSeries, observation_ts: TimeSeries, time_axis: TimeAxis) -> TimeSeries:
        """
        compute the running bias timeseries,
            using one 'merged' - forecasts and one observation time - series.
            
            Before each day - period, the bias - values are copied out to form
            a continuous bias prediction time-series.
            
            
            Args:
                bias_predictor (KalmanBiasPredictor): The bias predictor object it self
            
                forecast_ts (TimeSeries): a merged forecast ts, with period covering the observation_ts and time_axis supplied
            
                observation_ts (TimeSeries): the observation time-series
            
                time_axis (TimeAxis): covering the period/timesteps to be updated, e.g. yesterday, 3h resolution steps, according to the points in the filter
            
            Returns:
                TimeSeries: bias_ts. With ts( time_axis,bias_vector,POINT_AVERAGE),  computed running bias-ts
            
        """
        ...

    @staticmethod
    def update_with_forecast_vector(bias_predictor: KalmanBiasPredictor, temperature_sources: TsVector, observation_ts: TimeSeries, time_axis: TimeAxis) -> None:
        """
        update the bias-predictor with forecasts and observation
            After the update, the state is updated with new kalman estimates for the bias, .state.x
            
            
            Args:
                bias_predictor (KalmanBiasPredictor): The bias predictor object it self
            
                temperature_sources (TsVector): a set of forecasts, in the order oldest to the newest.With the periods covering parts of the observation_ts and time_axis supplied
            
                observation_ts (TimeSeries): the observation time-series
            
                time_axis (TimeAxis): covering the period/timesteps to be updated, e.g. yesterday, 3h resolution steps, according to the points in the filter
            
        """
        ...

    @staticmethod
    def update_with_geo_forecast(bias_predictor: KalmanBiasPredictor, temperature_sources: TemperatureSourceVector, observation_ts: TimeSeries, time_axis: TimeAxis) -> None:
        """
        update the bias-predictor with forecasts and observation
            After the update, the state is updated with new kalman estimates for the bias, .state.x
            
            
            Args:
                bias_predictor (KalmanBiasPredictor): The bias predictor object it self
            
                temperature_sources (TemperatureSourceVector): ta set of forecasts, in the order oldest to the newest.Note that the geo part of source is not used in this context, only the tswith periods covering parts of the observation_ts and time_axis supplied
            
                observation_ts (Timeseries): the observation time-series
            
                time_axis (TimeAxis): covering the period/timesteps to be updated,e.g. yesterday, 3h resolution steps, according to the points in the filter
            
        """
        ...



class KalmanFilter:
    """
    Specialized kalman filter for temperature (e.g.:solar-driven bias patterns)
    The observation point(t, v) is folded on to corresponding period of the
    day(number of periods in a day is parameterized, typically 8).
    A simplified kalman filter algorithm using the forecast bias as
    the state - variable.
    Observed bias(fc - obs) is feed into the filter and establishes the
    kalman best predicted estimates(x) for the bias.
    This bias can then be used as a correction to forecast in the future
    to compensate for systematic forecast errors.
    Credits: Thanks to met.no for providing the original source for this algorithm.
    
    
    See also:
        https://en.wikipedia.org/wiki/Kalman_filter
    """
    @property
    def parameter(self)->Any:
        """
        The KalmanParameter used by this filter
        """
        ...
    @parameter.setter
    def parameter(self, value:Any)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self) -> None:
        """
        Construct a filter with default KalmanParameter
        
        """
        ...
    @overload
    def __init__(self, p: KalmanParameter) -> None:
        """
        Construct a filter with the supplied parameter
        """
        ...

    def create_initial_state(self) -> KalmanState:
        """
        returns initial state, suitable for starting, using the filter parameters
        """
        ...

    def update(self, observed_bias: object, t: Union[time,float,int], state: KalmanState) -> None:
        """
        update the with the observed_bias for a specific period starting at utctime t
            
            Args:
                observed_bias (float): nan if no observation is available otherwise obs - fc
            
                t (time): utctime of the start of the observation period, this filter utilizes daily solar patterns, so time in day - cycle is the only important aspect.
            
                state (KalmanState): contains the kalman state, that is updated by the function upon return
            
        """
        ...



class KalmanParameter:
    """
    Defines the parameters that is used to tune the kalman-filter algorithm for temperature type of signals
    """
    @property
    def covariance_init(self)->float:
        """
          default=0.5,  for the error covariance P matrix start-values
        """
        ...
    @covariance_init.setter
    def covariance_init(self, value:float)->None:
        ...
    @property
    def hourly_correlation(self)->float:
        """
         default=0.93, correlation from one-hour to the next
        """
        ...
    @hourly_correlation.setter
    def hourly_correlation(self, value:float)->None:
        ...
    @property
    def n_daily_observations(self)->int:
        """
          default = 8 each 24hour, every 3 hour
        """
        ...
    @n_daily_observations.setter
    def n_daily_observations(self, value:int)->None:
        ...
    @property
    def ratio_std_w_over_v(self)->float:
        """
         default=0.06, st.dev W /st.dev V ratio
        """
        ...
    @ratio_std_w_over_v.setter
    def ratio_std_w_over_v(self, value:float)->None:
        ...
    @property
    def std_error_bias_measurements(self)->float:
        """
         default=2.0, st.dev for the bias measurements
        """
        ...
    @std_error_bias_measurements.setter
    def std_error_bias_measurements(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, n_daily_observations: object, hourly_correlation: object, covariance_init: object, std_error_bias_measurements: object, ratio_std_w_over_v: object) -> None:
        """
        Constructs KalmanParameter with default or supplied values
        
        """
        ...
    @overload
    def __init__(self, const_ref: KalmanParameter) -> None:
        """
        clone the supplied KalmanParameter
        """
        ...



class KalmanState:
    """
    keeps the state of the specialized kalman-filter:
    * x : vector[n=n_daily_observations] best estimate
    * k : vector[n], gain factors
    * P : matrix[nxn], covariance
    * W : noise[nxn]
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, n_daily_observations: object, covariance_init: object, hourly_correlation: object, process_noise_init: object) -> None:
        """
        create a state based on supplied parameters
        
        """
        ...
    @overload
    def __init__(self, clone: KalmanState) -> None:
        """
        clone the supplied state
        
        """
        ...
    @overload
    def __init__(self) -> None:
        """
        construct a default state
        """
        ...

    @staticmethod
    def get_P(state: KalmanState) -> DoubleVector:
        """
        returns a copy of current kalman covariance matrix P
        """
        ...

    @staticmethod
    def get_W(state: KalmanState) -> DoubleVector:
        """
        returns a copy of current kalman noise matrix W
        """
        ...

    @staticmethod
    def get_k(state: KalmanState) -> DoubleVector:
        """
        returns a copy of current kalman gain k
        """
        ...

    @staticmethod
    def get_x(state: KalmanState) -> DoubleVector:
        """
        returns a copy of current bias estimate x
        """
        ...

    def size(self) -> int:
        """
        returns the size of the state, corresponding to n_daily_observations
        """
        ...



class KirchnerCalculator:
    """
    Kirchner model for computing the discharge based on precipitation and evapotranspiration data.
    
    This algorithm is based on the log transform of the ode formulation of the time change in discharge as a function
    of measured precipitation, evapo-transpiration and discharge, i.e. equation 19 in the publication
    Catchments as simple dynamic systems: Catchment characterization, rainfall-runoff modeling, and doing
    'hydrology backward' by James W. Kirchner, published in Water Resources Research, vol. 45, W02429,
    doi: 10.1029/2008WR006912, 2009.
    """
    @overload
    def __init__(self, param: KirchnerParameter) -> None:
        """
        create a calculator using supplied parameter
        
        """
        ...
    @overload
    def __init__(self, abs_err: object, rel_err: object, param: KirchnerParameter) -> None:
        """
        create a calculator using supplied parameter, also specifying the ODE error parameters
        """
        ...

    def step(self, state: KirchnerState, response: KirchnerResponse, t0: Union[time,float,int], t1: Union[time,float,int], precipitation: object, evapotranspiration: object) -> None:
        """
        step Kirchner model forward from time t0 to time t1
            
            note: If the supplied q (state) is less than min_q(0.00001, it represents mm water..), it is forced to min_q to ensure numerical stability
            
            
            Args:
                state (KirchnerState):  current state, updated on return
            
                response (KirchnerResponse): the response , out parameter.
            
                t0 (time): start time
            
                t1 (time): end time
            
                precipitation (float): in mm/h
            
                evapotranspiration (float): in mm/h
            
        """
        ...



class KirchnerParameter:
    @property
    def c1(self)->float:
        """
         default =2.439
        """
        ...
    @c1.setter
    def c1(self, value:float)->None:
        ...
    @property
    def c2(self)->float:
        """
         default= 0.966
        """
        ...
    @c2.setter
    def c2(self, value:float)->None:
        ...
    @property
    def c3(self)->float:
        """
         default = -0.10
        """
        ...
    @c3.setter
    def c3(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, c1: object, c2: object, c3: object) -> None:
        """
        creates parameter object according to parameters
        """
        ...



class KirchnerResponse:
    @property
    def q_avg(self)->float:
        """
         average discharge over time-step in [mm/h]
        """
        ...
    @q_avg.setter
    def q_avg(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class KirchnerState:
    @property
    def q(self)->float:
        """
         state water 'content' in [mm/h], it defaults to 0.0001 mm, zero is not a reasonable valid value
        """
        ...
    @q.setter
    def q(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, q: object) -> None:
        """
        create a state specifying initial content q
        """
        ...



class LandTypeFractions:
    """
    LandTypeFractions are used to describe type of land,
    like glacier, lake, reservoir and forest.
    It is designed as a part of GeoCellData
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, glacier: object, lake: object, reservoir: object, forest: object, unspecified: object) -> None:
        """
        construct LandTypeFraction specifying the area of each type
        """
        ...

    def forest(self) -> float:
        """
        returns the forest part
        """
        ...

    def glacier(self) -> float:
        """
        returns the glacier part
        """
        ...

    def lake(self) -> float:
        """
        returns the lake part
        """
        ...

    def reservoir(self) -> float:
        """
        returns the reservoir part
        """
        ...

    def set_fractions(self, glacier: object, lake: object, reservoir: object, forest: object) -> None:
        """
        set the fractions explicit, each a value in range 0..1, sum should be 1.0
        """
        ...

    def snow_storage(self) -> float:
        """
        returns the area where snow can build up, 1.0-lake-reservoir
        """
        ...

    def unspecified(self) -> float:
        """
        returns the unspecified part
        """
        ...



class MethodStackParameter:
    """
    Contains the parameters for the method-stack,
    related to inter-method and routing behaviour
    """
    @property
    def reservoir_direct_response_fraction(self)->float:
        """
         range 0..1, default 1.0, e.g. all precipitation on a reservoir goes to direct response
        - set to 0.0, then all precipitation is routed as pr. standard for the stack
        """
        ...
    @reservoir_direct_response_fraction.setter
    def reservoir_direct_response_fraction(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: MethodStackParameter) -> None:
        """
        make a clone
        """
        ...



class OKParameter:
    """
    Ordinary Kriging Parameter, keeps parameters that controls the ordinary kriging calculation
    """
    @property
    def a(self)->float:
        """
         the a-constant, range or distance, value in the covariance formula
        """
        ...
    @a.setter
    def a(self, value:float)->None:
        ...
    @property
    def c(self)->float:
        """
         the c-constant, sill value in the covariance formula
        """
        ...
    @c.setter
    def c(self, value:float)->None:
        ...
    @property
    def cov_type(self)->OKCovarianceType:
        """
         covariance type EXPONENTIAL|GAUSSIAN to be used
        """
        ...
    @cov_type.setter
    def cov_type(self, value:OKCovarianceType)->None:
        ...
    @property
    def z_scale(self)->float:
        """
         z_scale to be used for range|distance calculations
        """
        ...
    @z_scale.setter
    def z_scale(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, c: object, a: object, cov_type: OKCovarianceType, z_scale: object) -> None:
        ...



class ParameterClient:
    """
    The client api for the shyft.hydrology parameter model service.
    """
    @property
    def host_port(self)->str:
        """
         Endpoint network address of the remote server.
        """
        ...
    @property
    def is_open(self)->bool:
        """
         If the connection to the remote server is (still) open.
        """
        ...
    @property
    def reconnect_count(self)->int:
        """
         Number of reconnects to the remote server that have been performed.
        """
        ...
    @property
    def timeout_ms(self)->int:
        """
         Timout for remote server operations, in number milliseconds.
        """
        ...
    def __init__(self, host_port: str, timeout_ms: object) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
        ...

    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
            
        """
        ...

    def get_model_infos(self, mids: Union[IntVector,list[int],range], created_in: UtcPeriod = "[not-valid-period>") -> ModelInfoVector:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
            
            Args:
                mids (IntVector): empty = all, or a list of known exisiting model-identifiers
            
                created_in (UtcPeriod): For which period you are interested in model-infos.
            
            Returns:
                ModelInfoVector: model_infos. Strongly typed list of ModelInfo
            
        """
        ...

    def read_model(self, mid: object) -> ParameterModel:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def read_models(self, mids: Union[IntVector,list[int],range]) -> ParameterModelVector:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mids (Int64Vector): A strongly typed list of ints, the model-identifers for the wanted models
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def remove_model(self, mid: object) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                int: ec. 0 or error-code?
            
        """
        ...

    def store_model(self, m: ParameterModel, mi: ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
            
            Args:
                m (Model): The model to store
            
                mi (ModelInfo): The model-info to store for the model
            
            Returns:
                int: mid. model-identifier for the stored model and model-info
            
        """
        ...

    def update_model_info(self, mid: object, mi: ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
            
            Args:
                mid (int): model-identifer
            
                mi (ModelInfo): The new updated model-info
            
            Returns:
                bool: ok. true if success
            
        """
        ...



class ParameterModel:
    """
    A model identifier and a strongly typed dict, cpp-map,  of stack parameters, suitable for use with a region-model matching the type.
    """
    @property
    def id(self)->int:
        """
         unique model id
        """
        ...
    @id.setter
    def id(self, value:int)->None:
        ...
    @property
    def parameters(self)->Any:
        """
         kind of parameter with id 
        """
        ...
    @parameters.setter
    def parameters(self, value:Any)->None:
        ...
    def __init__(self) -> None:
        ...



class ParameterModelVector:
    """
    A strongly typed list of ParameterModels
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: ParameterModelVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class ParameterOptimizer:
    """
    The ParameterOptimizer allows for testing and comparing optimizer methods for the same inital parameters and goal function.
    """
    @property
    def fx(self)->Any:
        """
        Callable[[],float]: goal function for which parameters are optimized
        """
        ...
    @fx.setter
    def fx(self, value:Any)->None:
        ...
    @property
    def p_max(self)->DoubleVector:
        """
         upper parameter bound in at least one-dimensional space
        """
        ...
    @property
    def p_min(self)->DoubleVector:
        """
         lower parameter bound in at least one-dimensional space
        """
        ...
    def __init__(self, p_min: Union[DoubleVector,list[float],np.ndarray], p_max: Union[DoubleVector,list[float],np.ndarray], fx: object) -> None:
        """
        Initialises a ParameterOptimizer instance.
            
                p_min (Union[DoubleVector[float], DoubleVector[int]]): the lower parameter bound in at least one-dimensional space.
            
                p_max (Union[DoubleVector[float], DoubleVector[int]]): the upper parameter bound in at least one-dimensional space.
            
                fx (Callable): the goal function to be used in the parameter optimization problem.
            
        """
        ...

    def goal_function(self, x: Union[DoubleVector,list[float],np.ndarray]) -> float:
        """
        The goal function to be used in the optimization problem.
            
                x (Union[DoubleVector[float], DoubleVector[int]]): the point in at least one-dimensional space at which the goal function is evaluated.
            
            Returns:
                DoubleVector[float]: y. The function value f(x) at point x.
            
        """
        ...

    def optimize(self, x: Union[DoubleVector,list[float],np.ndarray], calibration_option: CalibrationOption) -> DoubleVector:
        """
        Run the optimizer for given parameters and configuration.
            
                x (Union[DoubleVector[float], DoubleVector[int]]): the initial point in at least one-dimensional space for the optimization problem.
            
                calibration_option (CalibrationOption): the calibration for the various optimization methods.
            
            Returns:
                DoubleVector[float]: x_opt. The optimal parameters for which the optimum of f(x) is found.
            
            See also:
                CalibrationOption
            
        """
        ...



class ParameterServer:
    """
    The server-side component for the shyft.hydrology parameter model repository.
    """
    def __init__(self, root_dir: str) -> None:
        """
        Creates a server object that serves models from root_dir.
            The root_dir will be create if it does not exists.
            
            Args:
                root_dir (str): Path to the root-directory that keeps/will keep the model-files
            
        """
        ...

    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
        ...

    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
            
        """
        ...

    def is_running(self) -> bool:
        """
        true if server is listening and running
            
            See also:
                start_server()
            
        """
        ...

    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
            
            Args:
                ip (str): ip or host-name to start listening on
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_listening_port(self, port_no: object) -> None:
        """
        set the listening port for the service
            
            Args:
                port_no (int): a valid and available tcp-ip port number to listen on.
                typically it could be 20000 (avoid using official reserved numbers)
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_max_connections(self, max_connect: object) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
            
            Args:
                max_connect (int): maximum number of connections before denying more connections
            
            See also:
                get_max_connections()
            
        """
        ...

    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),is_running
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
        """
        ...

    def stop_server(self, timeout: object = 1000) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                start_server()
            
        """
        ...



class PenmanMonteithCalculator:
    """
    Evapotranspiration model, Penman-Monteith equation, PM
    reference ::
    
      ASCE-EWRI The ASCE Standardized Reference Evapotranspiration Equation Environmental and
         * Water Resources Institute (EWRI) of the American Society of Civil Engineers Task Com- mittee on Standardization of
         * Reference Evapotranspiration Calculation, ASCE, Washington, DC, Environmental and Water Resources Institute (EWRI) of
         * the American Society of Civil Engineers Task Com- mittee on Standardization of Reference Evapotranspiration Calculation,
         * ASCE, Washington, DC, 2005
    
    calculating reference evapotranspiration
    This function is plain and simple, taking albedo and turbidity
    into the constructor and provides function:
    reference_evapotranspiration;
    [mm/s] units.
    """
    def __init__(self, param: PenmanMonteithParameter) -> None:
        """
        create a calculator using supplied parameter
        """
        ...

    def reference_evapotranspiration(self, response: PenmanMonteithResponse, dt: Union[time,float,int], net_radiation: object, tempmax: object, tempmin: object, rhumidity: object, elevation: object, windspeed: object) -> None:
        """
        calculates reference evapotranspiration (standard or full based on full_model param), updating response
            
        """
        ...



class PenmanMonteithParameter:
    @property
    def full_model(self)->bool:
        """
         boolean to specify if full penman-monteith required, set to false by default, so it is asce-ewri standard
        """
        ...
    @full_model.setter
    def full_model(self, value:bool)->None:
        ...
    @property
    def height_t(self)->float:
        """
         typical value 2.0
        """
        ...
    @height_t.setter
    def height_t(self, value:float)->None:
        ...
    @property
    def height_veg(self)->float:
        """
         grass 0.15
        """
        ...
    @height_veg.setter
    def height_veg(self, value:float)->None:
        ...
    @property
    def height_ws(self)->float:
        """
         typical value 2.0
        """
        ...
    @height_ws.setter
    def height_ws(self, value:float)->None:
        ...
    @property
    def rl(self)->float:
        """
         stomatal resistance, for full PM only, 140
        """
        ...
    @rl.setter
    def rl(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, height_veg: object, height_ws: object, height_t: object, rl: object, full_model: object) -> None:
        """
        a new object with specified parameters
        """
        ...



class PenmanMonteithResponse:
    @property
    def et_ref(self)->float:
        """
         ..
        """
        ...
    @et_ref.setter
    def et_ref(self, value:float)->None:
        ...
    @property
    def soil_heat(self)->float:
        """
         ..
        """
        ...
    @soil_heat.setter
    def soil_heat(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class PrecipitationCorrectionCalculator:
    """
    Scales precipitation with the specified scale factor
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, scale_factor: object) -> None:
        """
        create a calculator using supplied parameter
        """
        ...

    def calc(self, precipitation: object) -> float:
        """
        returns scale_factor*precipitation
            
        """
        ...



class PrecipitationCorrectionParameter:
    @property
    def scale_factor(self)->float:
        """
         default =1.0
        """
        ...
    @scale_factor.setter
    def scale_factor(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, scale_factor: object) -> None:
        """
        creates parameter object according to parameters
        """
        ...



class PrecipitationSource(GeoPointSource):
    """
    geo located precipitation[mm/h]
    """
    @property
    def geo_ts(self)->GeoTimeSeries:
        """
         returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
        ...
    @property
    def mid_point_(self)->Any:
        """
        reference to internal mid_point
        """
        ...
    @mid_point_.setter
    def mid_point_(self, value:Any)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier, string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, midpoint: GeoPoint, ts: TimeSeries) -> None:
        ...
    @overload
    def __init__(self, geo_ts: GeoTimeSeries) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns a copy of mid_point
        """
        ...



class PrecipitationSourceVector:
    @property
    def geo_tsvector(self)->GeoTimeSeriesVector:
        """
         returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
        ...
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, geo_ts_list: list) -> Any:
        """
        Construct from list
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_vector: GeoTimeSeriesVector) -> Any:
        """
        Construct from a GeoTimeSeriesVector,
            Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
            
        
        """
        ...
    @overload
    def __init__(self, src: PrecipitationSourceVector) -> None:
        """
        clone src
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    @staticmethod
    def from_geo_and_ts_vector(geo_points: GeoPointVector, tsv: TsVector) -> PrecipitationSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
            
            Args:
                geo_points (GeoPointVector): the geo-points
            
            Returns:
                SourceVector: src_vector. a newly created geo-located vector of specified type
            
                tsv (TsVector): the corresponding time-series located at corresponding geo-point
            
        """
        ...



class PriestleyTaylorCalculator:
    """
    PriestleyTaylor,PT, (google for PriestleyTaylor)
    primitive implementation for calculating the potential evaporation.
    This function is plain and simple, taking land_albedo and PT.alpha
    into the constructor and provides a function that calculates potential evapotransporation
    [mm/s] units.
    """
    def __init__(self, land_albedo: object, alpha: object) -> None:
        ...

    def potential_evapotranspiration(self, temperature: object, global_radiation: object, rhumidity: object) -> float:
        """
        Calculate PotentialEvapotranspiration, given specified parameters
            
               * param temperature in [degC]
               * param global_radiation [W/m2]
               * param rhumidity in interval [0,1]
               * return PotentialEvapotranspiration in [mm/s] units
               *
            
        """
        ...



class PriestleyTaylorParameter:
    @property
    def albedo(self)->float:
        """
         typical value 0.2
        """
        ...
    @albedo.setter
    def albedo(self, value:float)->None:
        ...
    @property
    def alpha(self)->float:
        """
         typical value 1.26
        """
        ...
    @alpha.setter
    def alpha(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, albedo: object, alpha: object) -> None:
        """
        a new object with specified parameters
        """
        ...



class PriestleyTaylorResponse:
    @property
    def pot_evapotranspiration(self)->float:
        """
         response
        """
        ...
    @pot_evapotranspiration.setter
    def pot_evapotranspiration(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class RadiationCalculator:
    """
    Radiation,R,
    
    reference ::
    
      Allen, R. G.; Trezza, R. & Tasumi, M. Analytical integrated functions for daily solar radiation on slopes Agricultural and Forest Meteorology, 2006, 139, 55-73)
      primitive implementation for calculating predicted clear-sky short-wave solar radiation for inclined surfaces
    
    
    This function is plain and simple, taking albedo and turbidity
    into the constructor and provides 2 functions:
    
     * net_radiation calculates predicted solar radiation (if no measured data available) or translates measured data into the slope plus adds the longwave radiation: instantaneously;
     * net_radiation_step calculates predicted solar radiation or/and translates measured horizontal radiation into sloped surface for the time period between tstart and tend plus adds the lw radiation
    
    Recommended usage is the net_radiation_step for 24h-steps; it was also tested with 1h and 3h steps
    """
    def __init__(self, param: RadiationParameter) -> None:
        """
        create a calculator using supplied parameter
        """
        ...

    def net_radiation(self, response: RadiationResponse, latitude: object, t1: Union[time,float,int], dt: Union[time,float,int], slope: object, aspect: object, temperature: object, rhumidity: object, elevation: object, rsm: object) -> None:
        """
        calculates net radiation using either step or instantaneous method, updating response
            
        """
        ...

    def net_radiation_inst(self, response: RadiationResponse, latitude: object, t: Union[time,float,int], slope: object, aspect: object, temperature: object, rhumidity: object, elevation: object, rsm: object) -> None:
        """
        calculates net radiation using instantaneous method, updating response
            
        """
        ...

    def net_radiation_step(self, response: RadiationResponse, latitude: object, t1: Union[time,float,int], dt: Union[time,float,int], slope: object, aspect: object, temperature: object, rhumidity: object, elevation: object, rsm: object) -> None:
        """
        calculates net radiation, updating response
            
        """
        ...

    def net_radiation_step_asce_st(self, response: RadiationResponse, latitude: object, t1: Union[time,float,int], dt: Union[time,float,int], slope: object, aspect: object, temperature: object, rhumidity: object, elevation: object, rsm: object) -> None:
        """
        calculates net radiation, updating response
            
        """
        ...



class RadiationParameter:
    @property
    def albedo(self)->float:
        """
         typical value 0.2
        """
        ...
    @albedo.setter
    def albedo(self, value:float)->None:
        ...
    @property
    def turbidity(self)->float:
        """
         typical value 1.0
        """
        ...
    @turbidity.setter
    def turbidity(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, albedo: object, turbidity: object) -> None:
        """
        a new object with specified parameters
        """
        ...



class RadiationResponse:
    @property
    def net(self)->float:
        """
         ..
        """
        ...
    @net.setter
    def net(self, value:float)->None:
        ...
    @property
    def net_lw(self)->float:
        """
         ..
        """
        ...
    @net_lw.setter
    def net_lw(self, value:float)->None:
        ...
    @property
    def net_sw(self)->float:
        """
         ..
        """
        ...
    @net_sw.setter
    def net_sw(self, value:float)->None:
        ...
    @property
    def ra(self)->float:
        """
         ..
        """
        ...
    @ra.setter
    def ra(self, value:float)->None:
        ...
    @property
    def sw_cs_p(self)->float:
        """
         ..
        """
        ...
    @sw_cs_p.setter
    def sw_cs_p(self, value:float)->None:
        ...
    @property
    def sw_t(self)->float:
        """
         ..
        """
        ...
    @sw_t.setter
    def sw_t(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class RadiationSource(GeoPointSource):
    """
    geo located radiation[W/m2]
    """
    @property
    def geo_ts(self)->GeoTimeSeries:
        """
         returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
        ...
    @property
    def mid_point_(self)->Any:
        """
        reference to internal mid_point
        """
        ...
    @mid_point_.setter
    def mid_point_(self, value:Any)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier, string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, midpoint: GeoPoint, ts: TimeSeries) -> None:
        ...
    @overload
    def __init__(self, geo_ts: GeoTimeSeries) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns a copy of mid_point
        """
        ...



class RadiationSourceVector:
    @property
    def geo_tsvector(self)->GeoTimeSeriesVector:
        """
         returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
        ...
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, geo_ts_list: list) -> Any:
        """
        Construct from list
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_vector: GeoTimeSeriesVector) -> Any:
        """
        Construct from a GeoTimeSeriesVector,
            Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
            
        
        """
        ...
    @overload
    def __init__(self, src: RadiationSourceVector) -> None:
        """
        clone src
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    @staticmethod
    def from_geo_and_ts_vector(geo_points: GeoPointVector, tsv: TsVector) -> RadiationSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
            
            Args:
                geo_points (GeoPointVector): the geo-points
            
            Returns:
                SourceVector: src_vector. a newly created geo-located vector of specified type
            
                tsv (TsVector): the corresponding time-series located at corresponding geo-point
            
        """
        ...



class RelHumSource(GeoPointSource):
    """
    geo located relative humidity[%rh], range 0..1
    """
    @property
    def geo_ts(self)->GeoTimeSeries:
        """
         returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
        ...
    @property
    def mid_point_(self)->Any:
        """
        reference to internal mid_point
        """
        ...
    @mid_point_.setter
    def mid_point_(self, value:Any)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier, string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, midpoint: GeoPoint, ts: TimeSeries) -> None:
        ...
    @overload
    def __init__(self, geo_ts: GeoTimeSeries) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns a copy of mid_point
        """
        ...



class RelHumSourceVector:
    @property
    def geo_tsvector(self)->GeoTimeSeriesVector:
        """
         returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
        ...
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, geo_ts_list: list) -> Any:
        """
        Construct from list
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_vector: GeoTimeSeriesVector) -> Any:
        """
        Construct from a GeoTimeSeriesVector,
            Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
            
        
        """
        ...
    @overload
    def __init__(self, src: RelHumSourceVector) -> None:
        """
        clone src
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    @staticmethod
    def from_geo_and_ts_vector(geo_points: GeoPointVector, tsv: TsVector) -> RelHumSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
            
            Args:
                geo_points (GeoPointVector): the geo-points
            
            Returns:
                SourceVector: src_vector. a newly created geo-located vector of specified type
            
                tsv (TsVector): the corresponding time-series located at corresponding geo-point
            
        """
        ...



class River:
    """
    A river that we use for routing, its a single piece of a RiverNetwork
    
    The routing river have flow from:
    
      a) zero or more 'cell_nodes',  typically a cell_model type, lateral flow,like cell.rc.average_discharge [m3/s]
      b) zero or more upstream connected rivers, taking their .output_m3s()
    
    then a routing river can *optionally* be connected to a down-stream river
    providing a routing function (currently just a convolution of a uhg).
    
    This definition is recursive, and we use RiverNetwork to ensure the routing graph
    is directed and with no cycles.
    """
    @property
    def downstream(self)->RoutingInfo:
        """
         routing information for downstream, target-id, and hydrological distance
        """
        ...
    @downstream.setter
    def downstream(self, value:RoutingInfo)->None:
        ...
    @property
    def id(self)->int:
        """
         a valid identifier >0 for the river|routing element
        """
        ...
    @property
    def parameter(self)->UHGParameter:
        """
         describing the downstream propagation 
        """
        ...
    @parameter.setter
    def parameter(self, value:UHGParameter)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, id: object, downstream: RoutingInfo, parameter: UHGParameter) -> None:
        """
        a new object with specified parameters, notice that a valid river-id|routing-id must be >0
        """
        ...

    def uhg(self, dt: Union[time,float,int]) -> DoubleVector:
        """
        create the hydro-graph for this river, taking specified delta-t, dt,
            static hydrological distance as well as the shape parameters
            alpha and beta used to form the gamma-function.
            The length of the uhg (delay) is determined by the downstream-distance,
            and the velocity parameter. 
            The shape of the uhg is determined by alpha&beta parameters.
            
        """
        ...



class RiverNetwork:
    """
    A RiverNetwork takes care of the routing
    see also description of River
    The RiverNetwork provides all needed functions to build
    routing into the region model
    It ensures safe manipulation of rivers:
    
     * no cycles,
     * no duplicate object-id's etc.
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: RiverNetwork) -> None:
        """
        make a clone of river-network
        """
        ...

    def add(self, river: River) -> RiverNetwork:
        """
        add a river to the network, verifies river id, no cycles etc.
            
            raises exception on error
            tip: build your river-network from downstream to upstream order
            
        """
        ...

    def downstream_by_id(self, id: object) -> int:
        """
        return id of downstream river, 0 if none
        """
        ...

    def network_contains_directed_cycle(self) -> bool:
        """
        True if network have cycles detected
        """
        ...

    def remove_by_id(self, id: object) -> None:
        """
        disconnect and remove river for network
        """
        ...

    def river_by_id(self, id: object) -> River:
        """
        get river by id
        """
        ...

    def set_downstream_by_id(self, id: object, downstream_id: object) -> None:
        """
        set downstream target for specified river id
        """
        ...

    def upstreams_by_id(self, id: object) -> IntVector:
        """
        returns a list(vector) of id of upstream rivers from the specified one
            
        """
        ...



class RoutingInfo:
    """
    Describe the hydrological distance and the id of the target routing element (river)
    """
    @property
    def distance(self)->float:
        """
         the hydrological distance, in unit of [m]
        """
        ...
    @distance.setter
    def distance(self, value:float)->None:
        ...
    @property
    def id(self)->int:
        """
         id of the target,down-stream river
        """
        ...
    @id.setter
    def id(self, value:int)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, id: object, distance: object) -> None:
        """
        create an object with the supplied parameters
        """
        ...



class SkaugenCalculator:
    """
    Skaugen snow model method
    
    This algorithm uses theory from Skaugen 
    """
    def __init__(self) -> None:
        ...

    def step(self, delta_t: Union[time,float,int], parameter: SkaugenParameter, temperature: object, precipitation: object, radiation: object, wind_speed: object, state: SkaugenState, response: SkaugenResponse) -> None:
        """
        steps the model forward delta_t seconds, using specified input, updating state and response
        """
        ...



class SkaugenParameter:
    @property
    def alpha_0(self)->float:
        """
         default = 40.77
        """
        ...
    @alpha_0.setter
    def alpha_0(self, value:float)->None:
        ...
    @property
    def cfr(self)->float:
        """
         default = 0.01
        """
        ...
    @cfr.setter
    def cfr(self, value:float)->None:
        ...
    @property
    def cx(self)->float:
        """
         default = 2.5
        """
        ...
    @cx.setter
    def cx(self, value:float)->None:
        ...
    @property
    def d_range(self)->float:
        """
         default = 113.0
        """
        ...
    @d_range.setter
    def d_range(self, value:float)->None:
        ...
    @property
    def max_water_fraction(self)->float:
        """
         default = 0.1
        """
        ...
    @max_water_fraction.setter
    def max_water_fraction(self, value:float)->None:
        ...
    @property
    def ts(self)->float:
        """
         default = 0.14
        """
        ...
    @ts.setter
    def ts(self, value:float)->None:
        ...
    @property
    def tx(self)->float:
        """
         default = 0.16
        """
        ...
    @tx.setter
    def tx(self, value:float)->None:
        ...
    @property
    def unit_size(self)->float:
        """
         default = 0.1
        """
        ...
    @unit_size.setter
    def unit_size(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, alpha_0: object, d_range: object, unit_size: object, max_water_fraction: object, tx: object, cx: object, ts: object, cfr: object) -> None:
        """
        create parameter object with specifed values
        
        """
        ...
    @overload
    def __init__(self, p: SkaugenParameter) -> None:
        """
        create a clone of p
        """
        ...



class SkaugenResponse:
    @property
    def outflow(self)->float:
        """
         from snow-routine in [mm/h] over cell-area
        """
        ...
    @outflow.setter
    def outflow(self, value:float)->None:
        ...
    @property
    def sca(self)->float:
        """
         snow-covered area in fraction
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def swe(self)->float:
        """
         snow water equivalient [mm] over cell-area
        """
        ...
    @swe.setter
    def swe(self, value:float)->None:
        ...
    @property
    def total_stored_water(self)->float:
        """
         same as swe(deprecated)
        """
        ...
    @total_stored_water.setter
    def total_stored_water(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class SkaugenState:
    @property
    def alpha(self)->float:
        """
         ..
        """
        ...
    @alpha.setter
    def alpha(self, value:float)->None:
        ...
    @property
    def free_water(self)->float:
        """
         ..
        """
        ...
    @free_water.setter
    def free_water(self, value:float)->None:
        ...
    @property
    def nu(self)->float:
        """
         ..
        """
        ...
    @nu.setter
    def nu(self, value:float)->None:
        ...
    @property
    def num_units(self)->int:
        """
         ..
        """
        ...
    @num_units.setter
    def num_units(self, value:int)->None:
        ...
    @property
    def residual(self)->float:
        """
         ..
        """
        ...
    @residual.setter
    def residual(self, value:float)->None:
        ...
    @property
    def sca(self)->float:
        """
         ..
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def swe(self)->float:
        """
         ..
        """
        ...
    @swe.setter
    def swe(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, nu: object, alpha: object, sca: object, swe: object, free_water: object, residual: object, num_units: object) -> None:
        """
        create a state with specified values
        
        """
        ...
    @overload
    def __init__(self, s: SkaugenState) -> None:
        """
        create a clone of s
        """
        ...



class SnowTilesCalculator:
    """
    Tile based snow model method
    
    This algorithm uses ...
    """
    def __init__(self, parameter: SnowTilesParameter) -> None:
        """
        creates a calculator with given parameter
        """
        ...

    def step(self, state: SnowTilesState, response: SnowTilesResponse, t0: Union[time,float,int], t1: Union[time,float,int], precipitation: object, temperature: object) -> None:
        """
        steps the model forward from t0 to t1, updating state and response
            
        """
        ...



class SnowTilesParameter:
    @property
    def _multiply(self)->DoubleVector:
        """
         multiplication factors for distributing solid precipitation between the tiles (mean of elements = 1)
        """
        ...
    @property
    def area_fractions(self)->DoubleVector:
        """
         area fractions of the individual tiles, unit (-), range per tile [0.01, 1], the sum of the elements in the vector must equal 1
        """
        ...
    @area_fractions.setter
    def area_fractions(self, value:DoubleVector)->None:
        ...
    @property
    def cfr(self)->float:
        """
         refreeze coefficient, unit (-), range [0, 1]
        """
        ...
    @cfr.setter
    def cfr(self, value:float)->None:
        ...
    @property
    def cx(self)->float:
        """
         degree-day melt factor, unit (mm/C/day), range [0, 30]
        """
        ...
    @cx.setter
    def cx(self, value:float)->None:
        ...
    @property
    def lwmax(self)->float:
        """
         max liquid water content given as a fraction of ice in the snowpack, unit (-), range [0, 1]
        """
        ...
    @lwmax.setter
    def lwmax(self, value:float)->None:
        ...
    @property
    def shape(self)->float:
        """
         shape parameter of the Gamma distribution defining the multiplication factors for snowfall, unit (-), range [0.1, inf]
        """
        ...
    @shape.setter
    def shape(self, value:float)->None:
        ...
    @property
    def ts(self)->float:
        """
         threshold temperature for melt onset, unit (C), range [-4, 4]
        """
        ...
    @ts.setter
    def ts(self, value:float)->None:
        ...
    @property
    def tx(self)->float:
        """
         threshold temperature determining precipitation phase, unit (C), range [-4, 4]
        """
        ...
    @tx.setter
    def tx(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, shape: object, tx: object, cx: object, ts: object, lwmax: object, cfr: object, area_fractions: Union[DoubleVector,list[float],np.ndarray]) -> None:
        ...



class SnowTilesResponse:
    @property
    def outflow(self)->Any:
        """
        average outflow, rain and snowmelt, for the cell, unit (mm/hour)
        """
        ...
    @outflow.setter
    def outflow(self, value:Any)->None:
        ...
    @property
    def sca(self)->float:
        """
         average sca as a fraction of total cell area, unit (-)
        """
        ...
    @sca.setter
    def sca(self, value:float)->None:
        ...
    @property
    def swe(self)->float:
        """
         average swe for the cell, unit (mm)
        """
        ...
    @swe.setter
    def swe(self, value:float)->None:
        ...
    def __init__(self) -> None:
        ...



class SnowTilesState:
    @property
    def fw(self)->DoubleVector:
        """
         frozen water in the snowpack, unit (mm)
        """
        ...
    @fw.setter
    def fw(self, value:DoubleVector)->None:
        ...
    @property
    def lw(self)->DoubleVector:
        """
         liquid water in the snowpack, unit (mm)
        """
        ...
    @lw.setter
    def lw(self, value:DoubleVector)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, fw: Union[DoubleVector,list[float],np.ndarray], lw: Union[DoubleVector,list[float],np.ndarray]) -> None:
        ...



class StateClient:
    """
    The client api for the shyft.hydrology state model service.
    """
    @property
    def host_port(self)->str:
        """
         Endpoint network address of the remote server.
        """
        ...
    @property
    def is_open(self)->bool:
        """
         If the connection to the remote server is (still) open.
        """
        ...
    @property
    def reconnect_count(self)->int:
        """
         Number of reconnects to the remote server that have been performed.
        """
        ...
    @property
    def timeout_ms(self)->int:
        """
         Timout for remote server operations, in number milliseconds.
        """
        ...
    def __init__(self, host_port: str, timeout_ms: object) -> None:
        """
        Creates a python client that can communicate with the corresponding server
        """
        ...

    def close(self) -> None:
        """
        Close the connection. It will automatically reopen if needed.
            
        """
        ...

    def get_model_infos(self, mids: Union[IntVector,list[int],range], created_in: UtcPeriod = "[not-valid-period>") -> ModelInfoVector:
        """
        returns all or selected model-info objects based on model-identifiers(mids)
            
            Args:
                mids (IntVector): empty = all, or a list of known exisiting model-identifiers
            
                created_in (UtcPeriod): For which period you are interested in model-infos.
            
            Returns:
                ModelInfoVector: model_infos. Strongly typed list of ModelInfo
            
        """
        ...

    def read_model(self, mid: object) -> StateModel:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def read_models(self, mids: Union[IntVector,list[int],range]) -> StateModelVector:
        """
        Read and return the model for specified model-identifier (mid)
            
            Args:
                mids (Int64Vector): A strongly typed list of ints, the model-identifers for the wanted models
            
            Returns:
                Model: m. The resulting model from the server
            
        """
        ...

    def remove_model(self, mid: object) -> int:
        """
        Remove the specified model bymodel-identifier (mid)
            
            Args:
                mid (int): the model-identifer for the wanted model
            
            Returns:
                int: ec. 0 or error-code?
            
        """
        ...

    def store_model(self, m: StateModel, mi: ModelInfo) -> int:
        """
        Store the model to backend, if m.id==0 then a new unique model-info is created and used
            
            Args:
                m (Model): The model to store
            
                mi (ModelInfo): The model-info to store for the model
            
            Returns:
                int: mid. model-identifier for the stored model and model-info
            
        """
        ...

    def update_model_info(self, mid: object, mi: ModelInfo) -> bool:
        """
        Update the model-info for specified model-identifier(mid)
            
            Args:
                mid (int): model-identifer
            
                mi (ModelInfo): The new updated model-info
            
            Returns:
                bool: ok. true if success
            
        """
        ...



class StateModel:
    """
    A model identifier and a StateVector, where the vector can keep an uniform set of StateId of specified type
    """
    @property
    def id(self)->int:
        """
         unique model id
        """
        ...
    @id.setter
    def id(self, value:int)->None:
        ...
    @property
    def states(self)->Any:
        """
         kind of state vector with id 
        """
        ...
    @states.setter
    def states(self, value:Any)->None:
        ...
    def __init__(self) -> None:
        ...



class StateModelVector:
    """
    A strongly typed list of StateModel's
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: StateModelVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class StateServer:
    """
    The server-side component for the shyft.hydrology state model repository.
    """
    def __init__(self, root_dir: str) -> None:
        """
        Creates a server object that serves models from root_dir.
            The root_dir will be create if it does not exists.
            
            Args:
                root_dir (str): Path to the root-directory that keeps/will keep the model-files
            
        """
        ...

    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
        ...

    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
            
        """
        ...

    def is_running(self) -> bool:
        """
        true if server is listening and running
            
            See also:
                start_server()
            
        """
        ...

    def set_listening_ip(self, ip: str) -> None:
        """
        set the listening port for the service
            
            Args:
                ip (str): ip or host-name to start listening on
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_listening_port(self, port_no: object) -> None:
        """
        set the listening port for the service
            
            Args:
                port_no (int): a valid and available tcp-ip port number to listen on.
                typically it could be 20000 (avoid using official reserved numbers)
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_max_connections(self, max_connect: object) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
            
            Args:
                max_connect (int): maximum number of connections before denying more connections
            
            See also:
                get_max_connections()
            
        """
        ...

    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),is_running
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
        """
        ...

    def stop_server(self, timeout: object = 1000) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                start_server()
            
        """
        ...



class TargetSpecificationPts:
    """
    To guide the model calibration, we have a goal-function that we try to minimize
    This class contains the needed specification of this goal-function so that we can:
    
     1. from simulations, collect time-series at catchment level for (DISCHARGE|SNOW_COVERED_AREA|SNOW_WATER_EQUIVALENT)
     2. from observations, have a user-specified series the expression above should be equal to
     3. use user specified kling-gupta factors to evaluate kind-of-difference between target and simulated
     4. scale-factor to put a weight on this specific target-specification compared to other(we can have multiple)
     5. a user specified id, uid, a string to identify the external source for calibration
     6. a user specified time-series expression with symbolic ref to `sim`, like `TimeSeries('sim').log()`
    """
    @property
    def calc_mode(self)->TargetSpecCalcType:
        """
         NASH_SUTCLIFFE, KLING_GUPTA,ABS_DIFF,RMSE
        """
        ...
    @calc_mode.setter
    def calc_mode(self, value:TargetSpecCalcType)->None:
        ...
    @property
    def catchment_indexes(self)->IntVector:
        """
         catchment indexes, 'cids'
        """
        ...
    @catchment_indexes.setter
    def catchment_indexes(self, value:IntVector)->None:
        ...
    @property
    def catchment_property(self)->CatchmentPropertyType:
        """
         DISCHARGE,SNOW_COVERED_AREA, SNOW_WATER_EQUIVALENT
        """
        ...
    @catchment_property.setter
    def catchment_property(self, value:CatchmentPropertyType)->None:
        ...
    @property
    def fx(self)->TimeSeries:
        """
         an expression to be applied to the simulated ts, e.g.:`TimeSeries('sim').log()`
        """
        ...
    @fx.setter
    def fx(self, value:TimeSeries)->None:
        ...
    @property
    def river_id(self)->int:
        """
         river identifier for routed discharge calibration
        """
        ...
    @river_id.setter
    def river_id(self, value:int)->None:
        ...
    @property
    def s_a(self)->float:
        """
         KG-scalefactor for alpha (variance)
        """
        ...
    @s_a.setter
    def s_a(self, value:float)->None:
        ...
    @property
    def s_b(self)->float:
        """
         KG-scalefactor for beta (bias)
        """
        ...
    @s_b.setter
    def s_b(self, value:float)->None:
        ...
    @property
    def s_r(self)->float:
        """
         KG-scalefactor for correlation
        """
        ...
    @s_r.setter
    def s_r(self, value:float)->None:
        ...
    @property
    def scale_factor(self)->float:
        """
         the scale factor to be used when considering multiple target_specifications
        """
        ...
    @scale_factor.setter
    def scale_factor(self, value:float)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         target ts
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier:string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> Any:
        """
        Construct an empty class
            
        
        """
        ...
    @overload
    def __init__(self, ts: TsFixed, cids: Union[IntVector,list[int],range], scale_factor: object, calc_mode: TargetSpecCalcType, s_r: object, s_a: object, s_b: object, catchment_property: CatchmentPropertyType, uid: str) -> Any:
        """
        construct a target specification filled in with supplied parameters
            
            Args:
                ts (TsFixed): time-series containing the target time-series
            
                cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
            
                scale_factor (float): the weight of this target-specification
            
                calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs,RMSE method
            
                s_r (float): KG scalefactor for correlation
            
                s_a (float): KG scalefactor for alpha(variance)
            
                s_b (float): KG scalefactor for beta(bias)
            
                catchment_property (CatchmentPropertyType): what to extract from catchment(DISCHARGE|SNOW_COVERED_AREA|SNOW_WATER_EQUIVALENT|ROUTED_DISCHARGE|CELL_CHARGE)
            
                uid (str): user specified string/id to help integration efforts
            
        
        """
        ...
    @overload
    def __init__(self, ts: TimeSeries, cids: Union[IntVector,list[int],range], scale_factor: object, calc_mode: TargetSpecCalcType, s_r: object, s_a: object, s_b: object, catchment_property: CatchmentPropertyType, uid: str) -> Any:
        """
        construct a target specification filled in with supplied parameters
            
            Args:
                ts (TimeSeries): time-series containing the target time-series, note that the time-axis of this ts must be a fixed-interval type
            
                cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
            
                scale_factor (float): the weight of this target-specification
            
                calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
            
                s_r (float): KG scalefactor for correlation
            
                s_a (float): KG scalefactor for alpha(variance)
            
                s_b (float): KG scalefactor for beta(bias)
            
                catchment_property (CatchmentPropertyType): what to extract from catchment(DISCHARGE|SNOW_COVERED_AREA|SNOW_WATER_EQUIVALENT|ROUTED_DISCHARGE|CELL_CHARGE)
            
                uid (str): user specified string/id to help integration efforts
            
        
        """
        ...
    @overload
    def __init__(self, ts: TsFixed, cids: Union[IntVector,list[int],range], scale_factor: object, calc_mode: TargetSpecCalcType) -> Any:
        """
        construct a target specification filled in with supplied parameters
            
            Args:
                ts (TsFixed): time-series containing the target time-series
            
                cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
            
                scale_factor (float): the weight of this target-specification
            
                calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
            
        
        """
        ...
    @overload
    def __init__(self, ts: TimeSeries, cids: Union[IntVector,list[int],range], scale_factor: object, calc_mode: TargetSpecCalcType) -> Any:
        """
        construct a target specification filled in with supplied parameters
            
            Args:
                ts (TimeSeries): time-series containing the target time-series, note the time-axis needs to be fixed_dt!
            
                cids (IntVector): A list of catchment id's(cids) that together adds up into same as the target-ts
            
                scale_factor (float): the weight of this target-specification
            
                calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
            
        
        """
        ...
    @overload
    def __init__(self, ts: TsFixed, rid: object, scale_factor: object, calc_mode: TargetSpecCalcType, s_r: object, s_a: object, s_b: object, uid: str) -> Any:
        """
        construct a target specification filled in with supplied parameters
            
            Args:
                ts (TsFixed): time-series containing the target time-series
            
                rid (int): A river-id identifying the point of flow in the river-network
            
                scale_factor (float): the weight of this target-specification
            
                calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
            
                s_r (float): KG scalefactor for correlation
            
                s_a (float): KG scalefactor for alpha(variance)
            
                s_b (float): KG scalefactor for beta(bias)
            
                uid (str): user specified string/id to help integration efforts
            
        
        """
        ...
    @overload
    def __init__(self, ts: TimeSeries, rid: object, scale_factor: object, calc_mode: TargetSpecCalcType, s_r: object, s_a: object, s_b: object, uid: str) -> Any:
        """
        construct a target specification filled in with supplied parameters
            
            Args:
                ts (TimeSeries): time-series containing the target time-series, note time-axis required to be fixed-dt type
            
                rid (int): A river-id identifying the point of flow in the river-network
            
                scale_factor (float): the weight of this target-specification
            
                calc_mode (TargetSpecCalcType): specifies how to calculate the goal function, NS, KG, Abs method
            
                s_r (float): KG scalefactor for correlation
            
                s_a (float): KG scalefactor for alpha(variance)
            
                s_b (float): KG scalefactor for beta(bias)
            
                uid (str): user specified string/id to help integration efforts
            
        """
        ...

    def evaluate_fx(self, ts: TimeSeries) -> TimeSeries:
        """
        Compute the evaluated result using the `fx` expression.
            
            Args:
                ts (TimeSeries): 
            
            Returns:
                TimeSeries: result. The evaluated result using the passed time-series as substitute to symbolic reference(s).
            
        """
        ...



class TargetSpecificationVector:
    """
    A list of (weighted) target specifications to be used for model calibration
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: TargetSpecificationVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class TemperatureSource(GeoPointSource):
    """
    geo located temperatures[deg Celcius]
    """
    @property
    def geo_ts(self)->GeoTimeSeries:
        """
         returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
        ...
    @property
    def mid_point_(self)->Any:
        """
        reference to internal mid_point
        """
        ...
    @mid_point_.setter
    def mid_point_(self, value:Any)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier, string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, midpoint: GeoPoint, ts: TimeSeries) -> None:
        ...
    @overload
    def __init__(self, geo_ts: GeoTimeSeries) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns a copy of mid_point
        """
        ...



class TemperatureSourceVector:
    @property
    def geo_tsvector(self)->GeoTimeSeriesVector:
        """
         returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
        ...
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, geo_ts_list: list) -> Any:
        """
        Construct from list
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_vector: GeoTimeSeriesVector) -> Any:
        """
        Construct from a GeoTimeSeriesVector,
            Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
            
        
        """
        ...
    @overload
    def __init__(self, src: TemperatureSourceVector) -> None:
        """
        clone src
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    @staticmethod
    def from_geo_and_ts_vector(geo_points: GeoPointVector, tsv: TsVector) -> TemperatureSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
            
            Args:
                geo_points (GeoPointVector): the geo-points
            
            Returns:
                SourceVector: src_vector. a newly created geo-located vector of specified type
            
                tsv (TsVector): the corresponding time-series located at corresponding geo-point
            
        """
        ...



class TsTransform:
    """
    transform the supplied time-series, f(t) interpreted according to its point_interpretation() policy
    into a new shyft core TsFixed time-series,
    that represents the true average for each of the n intervals of length dt, starting at start.
    the result ts will have the policy is set to POINT_AVERAGE_VALUE
    note:  that the resulting ts is a fresh new ts, not connected to the source ts
    """
    def __init__(self) -> None:
        ...

    @overload
    def to_average(self, start: Union[time,float,int], dt: Union[time,float,int], n: object, src: TimeSeries) -> TsFixed:
        ...
    @overload
    def to_average(self, start: Union[time,float,int], dt: Union[time,float,int], n: object, src: TimeSeries) -> TsFixed:
        ...
    @overload
    def to_average(self, start: Union[time,float,int], dt: Union[time,float,int], n: object, src: TsFixed) -> TsFixed:
        ...
    @overload
    def to_average(self, start: Union[time,float,int], dt: Union[time,float,int], n: object, src: TsFixed) -> TsFixed:
        ...
    @overload
    def to_average(self, start: object, dt: object, n: object, src: TimeSeries) -> TsFixed:
        ...
    @overload
    def to_average(self, start: object, dt: object, n: object, src: TimeSeries) -> TsFixed:
        ...
    @overload
    def to_average(self, start: object, dt: object, n: object, src: TsFixed) -> TsFixed:
        ...
    @overload
    def to_average(self, start: object, dt: object, n: object, src: TsFixed) -> TsFixed:
        ...



class UHGParameter:
    """
    The Unit Hydro Graph Parameter contains sufficient
    description to create a unit hydro graph, that have a shape
    and a discretized 'time-length' according to the model time-step resolution.
    Currently we use a gamma function for the shape:
    <a ref='https://en.wikipedia.org/wiki/Gamma_distribution'>gamma distribution</a>
    """
    @property
    def alpha(self)->float:
        """
         default 3.0,unit-less, ref. shape of gamma-function
        """
        ...
    @alpha.setter
    def alpha(self, value:float)->None:
        ...
    @property
    def beta(self)->float:
        """
         default 0.0, unit-less, added to pdf of gamma(alpha,1.0)
        """
        ...
    @beta.setter
    def beta(self, value:float)->None:
        ...
    @property
    def velocity(self)->float:
        """
         default 1.0, unit [m/s]
        """
        ...
    @velocity.setter
    def velocity(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, velocity: object, alpha: object, beta: object) -> None:
        """
        a new object with specified parameters
        """
        ...



class WindSpeedSource(GeoPointSource):
    """
    geo located wind speeds[m/s]
    """
    @property
    def geo_ts(self)->GeoTimeSeries:
        """
         returns a GeoTimeSeries  where the geo-point are copied, but time-series are by-reference
        """
        ...
    @property
    def mid_point_(self)->Any:
        """
        reference to internal mid_point
        """
        ...
    @mid_point_.setter
    def mid_point_(self, value:Any)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @property
    def uid(self)->str:
        """
         user specified identifier, string
        """
        ...
    @uid.setter
    def uid(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, midpoint: GeoPoint, ts: TimeSeries) -> None:
        ...
    @overload
    def __init__(self, geo_ts: GeoTimeSeries) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        returns a copy of mid_point
        """
        ...



class WindSpeedSourceVector:
    @property
    def geo_tsvector(self)->GeoTimeSeriesVector:
        """
         returns a GeoTimeSeriesVector containing GeoTimeSeries where the geo-points are copied, but time-series are by-reference
        """
        ...
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, geo_ts_list: list) -> Any:
        """
        Construct from list
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_vector: GeoTimeSeriesVector) -> Any:
        """
        Construct from a GeoTimeSeriesVector,
            Reference to the shyft.time_series.GeoTsMatrix.extract_geo_ts_vector() to see how to create one from the dtss/geo extensions
            
        
        """
        ...
    @overload
    def __init__(self, src: WindSpeedSourceVector) -> None:
        """
        clone src
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    @staticmethod
    def from_geo_and_ts_vector(geo_points: GeoPointVector, tsv: TsVector) -> WindSpeedSourceVector:
        """
        Create from a geo_points and corresponding ts-vectors
            
            Args:
                geo_points (GeoPointVector): the geo-points
            
            Returns:
                SourceVector: src_vector. a newly created geo-located vector of specified type
            
                tsv (TsVector): the corresponding time-series located at corresponding geo-point
            
        """
        ...


ABS_DIFF: TargetSpecCalcType
BOBYQA: OptimizerMethod
CELL_CHARGE: CatchmentPropertyType
DISCHARGE: CatchmentPropertyType
DREAM: OptimizerMethod
EXPONENTIAL: OKCovarianceType
GAUSSIAN: OKCovarianceType
GLOBAL: OptimizerMethod
KLING_GUPTA: TargetSpecCalcType
NASH_SUTCLIFFE: TargetSpecCalcType
PT_GS_K: RegionModelType
PT_GS_K_OPT: RegionModelType
PT_HPS_K: RegionModelType
PT_HPS_K_OPT: RegionModelType
PT_HS_K: RegionModelType
PT_HS_K_OPT: RegionModelType
PT_SS_K: RegionModelType
PT_SS_K_OPT: RegionModelType
PT_ST_HBV: RegionModelType
PT_ST_K: RegionModelType
PT_ST_K_HBV: RegionModelType
PT_ST_K_OPT: RegionModelType
RMSE: TargetSpecCalcType
ROUTED_DISCHARGE: CatchmentPropertyType
R_PM_GS_K: RegionModelType
R_PM_GS_K_OPT: RegionModelType
R_PT_GS_K: RegionModelType
R_PT_GS_K_OPT: RegionModelType
SCEUA: OptimizerMethod
SNOW_COVERED_AREA: CatchmentPropertyType
SNOW_WATER_EQUIVALENT: CatchmentPropertyType
catchment: stat_scope
cell: stat_scope
def ActualEvapotranspirationCalculate_step(water_level: object, potential_evapotranspiration: object, scale_factor: object, snow_fraction: object, dt: Union[time,float,int]) -> float:
    """
    actual_evapotranspiration calculates actual evapotranspiration, returning same unit as input pot.evap
        based on supplied parameters
        
        Args:
            water_level (): [mm] water level eqvivalent in ground, ae goes to zero if ground is drying out
        
            potential_evapotranspiration (): [mm/x], x time-unit
        
            scale_factor (): typically 1.5
        
            snow_fraction ():  0..1 there is to ae over snow surface, so snow_fraction 1.0, yields 0.0
        
            dt (): [s] timestep length, currently not part of the formula
        
        Returns:
            : actual evapotranspiration. 
        
    """
    ...

def HbvActualEvapotranspirationCalculate_step(soil_moisture: object, potential_evapotranspiration: object, lp: object, snow_fraction: object, dt: Union[time,float,int]) -> float:
    """
    actual_evapotranspiration calculates actual evapotranspiration, returning same unit as input pot.evap
        based on supplied parameters, formula like (1.0 - snow_fraction)*(soil_moisture < lp ? pot_evapo*(soil_moisture / lp):pot_evapo)
        
        Args:
            soil_moisture (): [mm] water level eqvivalent in ground, ae goes to zero if ground is drying out
        
            potential_evapotranspiration (): [mm/x], x time-unit
        
            lp (): [mm] soil_moisture threshold, lp typically 150
        
            snow_fraction ():  0..1 there is to ae over snow surface, so snow_fraction 1.0, yields 0.0
        
            dt (): [s] timestep length, currently not part of the formula
        
        Returns:
            : actual evapotranspiration. 
        
    """
    ...

def bayesian_kriging_temperature(src: TemperatureSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, btk_parameter: BTKParameter) -> TemperatureSourceVector:
    """
    Runs kriging for temperature sources and project the temperatures out to the destination geo-timeseries
        
        Notice that bayesian kriging is currently not very efficient for large grid inputs,
        using only one thread, and considering all source-timeseries (entire grid) for all destinations
        For few sources, spread out on a grid, it's quite efficient should work well
        
        
        
        Args:
            src ( TemperatureSourceVector): input a geo-located list of temperature time-series with filled in values (some might be nan etc.)
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis ( TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis and they are transformed and interpolated into the destination-timeaxis
        
            btk_parameter (BTKParameter): the parameters to be used during interpolation
        
        Returns:
            TemperatureSourceVector: result. with filled in temperatures according to their position, the idw_parameters and time_axis
        
    """
    ...

@overload
def compute_geo_ts_values_at_time(geo_ts_vector: GeoPointSourceVector, t: Union[time,float,int]) -> DoubleVector:
    """
    compute the ts-values of the GeoPointSourceVector for the specified time t and return DoubleVector
        
        Args:
            geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
        
        Returns:
            DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
        
            t (int): timestamp in utc seconds since epoch
        
    
    """
    ...
@overload
def compute_geo_ts_values_at_time(geo_ts_vector: TemperatureSourceVector, t: Union[time,float,int]) -> DoubleVector:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
        
        Args:
            geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
        
        Returns:
            DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
        
            t (int): timestamp in utc seconds since epoch
        
    
    """
    ...
@overload
def compute_geo_ts_values_at_time(geo_ts_vector: PrecipitationSourceVector, t: Union[time,float,int]) -> DoubleVector:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
        
        Args:
            geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
        
        Returns:
            DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
        
            t (int): timestamp in utc seconds since epoch
        
    
    """
    ...
@overload
def compute_geo_ts_values_at_time(geo_ts_vector: WindSpeedSourceVector, t: Union[time,float,int]) -> DoubleVector:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
        
        Args:
            geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
        
        Returns:
            DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
        
            t (int): timestamp in utc seconds since epoch
        
    
    """
    ...
@overload
def compute_geo_ts_values_at_time(geo_ts_vector: RelHumSourceVector, t: Union[time,float,int]) -> DoubleVector:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
        
        Args:
            geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
        
        Returns:
            DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
        
            t (int): timestamp in utc seconds since epoch
        
    
    """
    ...
@overload
def compute_geo_ts_values_at_time(geo_ts_vector: RadiationSourceVector, t: Union[time,float,int]) -> DoubleVector:
    """
    compute the ts-values of the GeoPointSourceVector type for the specified time t and return DoubleVector
        
        Args:
            geo_ts_vector (GeoPointSourceVector): Any kind of GeoPointSource vector
        
        Returns:
            DoubleValue: values. List of extracted values at same size/position as the geo_ts_vector
        
            t (int): timestamp in utc seconds since epoch
        
    """
    ...

def create_precipitation_source_vector_from_np_array(time_axis: TimeAxis, geo_points: GeoPointVector, np_array: object, point_fx: point_interpretation_policy) -> PrecipitationSourceVector:
    """
    Create a PrecipitationSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
        
        Args:
            time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
        
            geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
        
            np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
        
            point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
        
        Returns:
            PrecipitationSourceVector: tsv. a PrecipitationSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
        
    """
    ...

def create_radiation_source_vector_from_np_array(time_axis: TimeAxis, geo_points: GeoPointVector, np_array: object, point_fx: point_interpretation_policy) -> RadiationSourceVector:
    """
    Create a RadiationSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
        
        Args:
            time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
        
            geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
        
            np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
        
            point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
        
        Returns:
            RadiationSourceVector: tsv. a RadiationSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
        
    """
    ...

def create_rel_hum_source_vector_from_np_array(time_axis: TimeAxis, geo_points: GeoPointVector, np_array: object, point_fx: point_interpretation_policy) -> RelHumSourceVector:
    """
    Create a RelHumSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
        
        Args:
            time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
        
            geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
        
            np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
        
            point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
        
        Returns:
            RelHumSourceVector: tsv. a RelHumSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
        
    """
    ...

def create_temperature_source_vector_from_np_array(time_axis: TimeAxis, geo_points: GeoPointVector, np_array: object, point_fx: point_interpretation_policy) -> TemperatureSourceVector:
    """
    Create a TemperatureSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
        
        Args:
            time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
        
            geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
        
            np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
        
            point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
        
        Returns:
            TemperatureSourceVector: tsv. a TemperatureSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
        
    """
    ...

def create_wind_speed_source_vector_from_np_array(time_axis: TimeAxis, geo_points: GeoPointVector, np_array: object, point_fx: point_interpretation_policy) -> WindSpeedSourceVector:
    """
    Create a WindSpeedSourceVector from specified time_axis,geo_points, 2-d np_array and point_fx.
        
        Args:
            time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
        
            geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
        
            np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
        
            point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
        
        Returns:
            WindSpeedSourceVector: tsv. a WindSpeedSourceVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
        
    """
    ...

def glacier_melt_step(dtf: object, temperature: object, sca: object, glacier_fraction: object) -> float:
    """
    Calculates outflow from glacier melt rate [mm/h].
        
        Args:
            dtf (float): degree timestep factor [mm/day/deg.C]; lit. values for Norway: 5.5 - 6.4 in Hock, R. (2003), J. Hydrol., 282, 104-115.
        
            temperature (float): degC, considered constant over timestep dt
        
            sca (float): fraction of snow cover in cell [0..1], glacier melt occurs only if glacier fraction > snow fraction
        
            glacier_fraction (float): glacier fraction [0..1] in the total area
        
        Returns:
            float: glacier_melt. output from glacier, melt rate [mm/h]
        
    """
    ...

def idw_precipitation(src: PrecipitationSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, idw_param: IDWPrecipitationParameter) -> PrecipitationSourceVector:
    """
    Runs inverse distance interpolation to project precipitation sources out to the destination geo-timeseries
        
        Args:
            src (PrecipitationSourceVector): a geo-located list of precipitation time-series with filled in values (some might be nan etc.)
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
        
            idw_param (IDWPrecipitationParameter): the parameters to be used during interpolation
        
        Returns:
            PrecipitationSourceVector: precipitation. with filled in precipitations according to their position, the idw_parameters and time_axis
        
    """
    ...

def idw_radiation(src: RadiationSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, idw_param: IDWParameter, slope_factors: Union[DoubleVector,list[float],np.ndarray]) -> RadiationSourceVector:
    """
    Runs inverse distance interpolation to project radiation sources out to the destination geo-timeseries
        
        Args:
            src (RadiationSourceVector): input a geo-located list of precipitation time-series with filled in values (some might be nan etc.)
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
        
            idw_param (IDWParameter): the parameters to be used during interpolation
        
            slope_factors (DoubleVector): the slope-factor corresponding to geopoints, typical 0.9
        
        Returns:
            RadiationSourceVector: radiation. with filled in radiation according to their position, the idw_parameters and time_axis
        
    """
    ...

def idw_relative_humidity(src: RelHumSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, idw_param: IDWParameter) -> RelHumSourceVector:
    """
    Runs inverse distance interpolation to project relative humidity sources out to the destination geo-timeseries
        
        Args:
            src (RelHumSourceVector): input a geo-located list of precipitation time-series with filled in values (some might be nan etc.)
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
        
            idw_param (IDWParameter): the parameters to be used during interpolation
        
        Returns:
            RelHumSourceVector: relhum. with filled in relative humidity according to their position, the idw_parameters and time_axis
        
    """
    ...

def idw_temperature(src: TemperatureSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, idw_param: IDWTemperatureParameter) -> TemperatureSourceVector:
    """
    Runs inverse distance interpolation to project temperature sources out to the destination geo-timeseries
        
        Args:
            src (TemperatureSourceVectorn): input a geo-located list of temperature time-series with filled in values (some might be nan etc.)
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
        
            idw_param (IDWTemperatureParameter): the parameters to be used during interpolation, defines the behaviour of the interpolation. Please use help on the IDWTemperatureParameter for details.
        
        Returns:
            TemperatureSourceVector: temperatures. with filled in temperatures according to their position, the idw_parameters and time_axis
        
    """
    ...

def idw_wind_speed(src: WindSpeedSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, idw_param: IDWParameter, shear_factor: object = nan, hub_height: object = nan, reference_height: object = nan) -> WindSpeedSourceVector:
    """
    Runs inverse distance interpolation to project geo located wind input sources out to the destination geo-timeseries
        
        
        Args:
            src (WindspeedSourceVector): input a geo-located list of wind speed time-series with filled in values (some might be nan etc.)
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, they are transformed and interpolated into the destination-timeaxis
        
            idw_param (IDWParametern): the parameters to be used during interpolation
        
            shear_factor (float): optional:wind shear coefficient to estimate the wind speed at higher elevations, typical 0.11 onshore and 0.15 offshore,>0.0
        
            hub_height (float): optional:distance from the ground to the middle of the turbine's rotor, >0.0 
        
            reference_height (float): optional: height above ground of measured wind speed, >0.0
        
        Returns:
            WindSpeedSourceVector: windspeed. with filled in wind speed according to their position, the idw_parameters and time_axis
        
    """
    ...

def make_uhg_from_gamma(n_steps: object, alpha: object, beta: object) -> DoubleVector:
    """
    make_uhg_from_gamma a simple function to create a uhg (unit hydro graph) weight vector
        containing n_steps, given the gamma shape factor alpha and beta.
        ensuring the sum of the weight vector is 1.0
        and that it has a min-size of one element (1.0)
        
        
        Args:
            n_steps (int): number of time-steps, elements, in the vectorn
        
            alpha (float): the gamma_distribution gamma-factor
        
            beta (float): the base-line, added to pdf(gamma(alpha,1))
        
        Returns:
            : unit hydro graph factors.  - normalized to sum 1.0
        
    """
    ...

def ordinary_kriging(src: GeoPointSourceVector, dst: GeoPointVector, time_axis: TimeAxisFixedDeltaT, parameter: OKParameter) -> GeoPointSourceVector:
    """
    Runs ordinary kriging for geo sources and project the source out to the destination geo-timeseries
        Notice that kriging is currently not very efficient for large grid inputs,
        using only one thread, and considering all source-timeseries (entire grid) for all destinations
        For few sources, spread out on a grid, it's quite efficient should work well
        Also note that this function currently does not elicite observations with nan-data
        most useful when you have control on the inputs, providing full set of data.
        
        
        Args:
            src (GeoSourceVector): input a geo-located list of time-series with filled in values
        
            dst (GeoPointVector): the GeoPoints,(x,y,z) locations to interpolate into
        
            time_axis (TimeAxis): the destination time-axis, recall that the inputs can be any-time-axis, and they are transformed and interpolated into the destination-timeaxis
        
            parameter (OKParameter): the parameters to be used during interpolation
        
        Returns:
            GeoSourceVector: result. with filled in ts-values according to their position, the parameters and time_axis
        
        
    """
    ...

def parse_cf_time(reference: str) -> UtcPeriod:
    """
    parses cf time unit string like hours since 1970-01-01 00:00:00
        and returs an UtcPeriod, where the .start is the reference time
        and the .timespan() is the delta time, e.g. 3600.0
        If the reference is illformed, then the returned period is not .valid()
        The intended usage is to get the reference time and time-delta of netcdf files,
        replacing the need of cftime python library.
        
        Args:
            reference (str): netcdf time-reference
        
        Returns:
            UtcPeriod: p. p.valid()==True if well formed,p.start equal to the reference time, and p.timespan() equal to the time-delta specified
        
    """
    ...

def version() -> str:
    """
    
        C++ signature :
            char const * __ptr64 version()
    """
    ...

