"""This file is auto-generated with stub_generation.py"""
from typing import List,Any,overload,Callable,Union
from enum import Enum
import numpy as np
from shyft.time_series._time_series import *
# import Boost.Python
nan = float('nan')

class TimeAxisType(Enum):
    """
    enumeration for the internal time-axis representations
    """
    FIXED: int
    CALENDAR: int
    POINT: int


class convolve_policy(Enum):
    """
    Ref TimeSeries.convolve_w function, this policy determine how to handle initial conditions
    `USE_NEAREST`: value(0) is used for all values before value(0), and value(n-1) is used for all values after value(n-1) == 'mass preserving'
    `USE_ZERO` : use zero for all values before value(0) or after value(n-1) == 'shape preserving'
    `USE_NAN`  : nan is used for all values outside the ts
    `BACKWARD` : filter is 'backward looking' == boundary handling in the beginning of ts
    `FORWARD`  : filter is 'forward looking' == boundary handling in the end of ts
    `CENTER`   : filter is centered == boundary handling in both ends
    """
    USE_NEAREST: int
    USE_ZERO: int
    USE_NAN: int
    BACKWARD: int
    FORWARD: int
    CENTER: int


class derivative_method(Enum):
    """
    Ref. the .derivative time-series function, this defines how to compute the
    derivative of a given time-series
    """
    DEFAULT: int
    FORWARD: int
    BACKWARD: int
    CENTER: int


class extend_fill_policy(Enum):
    """
    Ref TimeSeries.extend function, this policy determines how to represent values in a gap
    EPF_NAN : use nan values in the gap
    EPF_LAST: use the last value before the gap
    EPF_FILL: use a supplied value in the gap
    """
    FILL_NAN: int
    USE_LAST: int
    FILL_VALUE: int


class extend_split_policy(Enum):
    """
    Ref TimeSeries.extend function, this policy determines where to split/shift from one ts to the other
    EPS_LHS_LAST : use nan values in the gap
    EPS_RHS_FIRST: use the last value before the gap
    EPS_VALUE    : use a supplied value in the gap
    """
    LHS_LAST: int
    RHS_FIRST: int
    AT_VALUE: int


class ice_packing_temperature_policy(Enum):
    """
    Policy enum to specify how `TimeSeries.ice_packing` handles missing temperature values.
    
    The enum defines three values:
     * `DISALLOW_MISSING` disallows any missing values. With this policy whenever a NaN value is encountered,
       or the window of values to consider extends outside the range of the time series, a NaN value will be
       written to the result time-series.
     * `ALLOW_INITIAL_MISSING` disallows explicit NaN values, but allows the window of values to consider
       to expend past the range of the time-series for the initial values.
     * `ALLOW_ANY_MISSING` allow the window of values to contain NaN values, averaging what it can.
       Only if all the values in the window is NaN, the result wil be NaN.
    """
    DISALLOW_MISSING: int
    ALLOW_INITIAL_MISSING: int
    ALLOW_ANY_MISSING: int


class interpolation_scheme(Enum):
    """
    Interpolation methods used by TimeSeries.transform
    """
    SCHEME_POLYNOMIAL: int
    SCHEME_LINEAR: int
    SCHEME_CATMULL_ROM: int


class point_interpretation_policy(Enum):
    """
    Determines how to interpret the points in a timeseries when interpreted as a function of time, f(t)
    """
    POINT_INSTANT_VALUE: int
    POINT_AVERAGE_VALUE: int


class statistics_property(Enum):
    """
    special values for percentiles evaluation to extract non percentile values from a set
    """
    AVERAGE: int
    MIN_EXTREME: int
    MAX_EXTREME: int


class trim_policy(Enum):
    """
    Enum to decide if to trim inwards or outwards where TRIM_IN means inwards, TRIM_ROUND rounds halfway cases away from zero.
    """
    TRIM_IN: int
    TRIM_OUT: int
    TRIM_ROUND: int


class AverageAccessorTs:
    """
    Accessor to get out true average for the time-axis intervals for a point time-series
    """
    @overload
    def __init__(self, ts: TimeSeries, ta: TimeAxisFixedDeltaT) -> None:
        """
        construct accessor from ts and time-axis ta
        
        """
        ...
    @overload
    def __init__(self, ts: TimeSeries, ta: TimeAxisFixedDeltaT) -> None:
        """
        constructor from ref ts and time-axis ta
        """
        ...

    def size(self) -> int:
        """
        returns number of intervals in the time-axis for this accessor
        """
        ...

    def value(self, i: object) -> float:
        """
        returns the i'th true average value
        """
        ...



class ByteVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, const_ref_v: ByteVector) -> None:
        """
        copy construct a new strongly typed list
            
        """
        ...

    def FromNdArray(self) -> ByteVector:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    def from_numpy(self) -> ByteVector:
        ...

    def to_numpy(self) -> Any:
        """
        convert to numpy
        """
        ...



class CacheStats:
    """
    Cache statistics for the DtsServer.
    """
    @property
    def coverage_misses(self)->int:
        """
         number of misses where we did find the time-series id, but the period coverage was insufficient
        """
        ...
    @coverage_misses.setter
    def coverage_misses(self, value:int)->None:
        ...
    @property
    def fragment_count(self)->int:
        """
         number of time-series fragments in the cache, (greater or equal to id_count)
        """
        ...
    @fragment_count.setter
    def fragment_count(self, value:int)->None:
        ...
    @property
    def hits(self)->int:
        """
         number of hits by time-series id
        """
        ...
    @hits.setter
    def hits(self, value:int)->None:
        ...
    @property
    def id_count(self)->int:
        """
         number of unique time-series identities in cache
        """
        ...
    @id_count.setter
    def id_count(self, value:int)->None:
        ...
    @property
    def misses(self)->int:
        """
         number of misses by time-series id
        """
        ...
    @misses.setter
    def misses(self, value:int)->None:
        ...
    @property
    def point_count(self)->int:
        """
         total number of time-series points in the cache
        """
        ...
    @point_count.setter
    def point_count(self, value:int)->None:
        ...
    def __init__(self) -> None:
        ...



class Calendar:
    """
    Calendar deals with the concept of a human calendar
    
    In Shyft we practice the 'utc-time perimeter' principle,
    
      * the core is utc-time exclusively 
      * we deal with time-zone and calendars at the interfaces/perimeters
    
    In python, this corresponds to timestamp[64], or as the integer version of the time package representation
    e.g. the difference between time.time()- utctime_now() is in split-seconds
    
    
    Calendar functionality:
    
     * Conversion between the calendar coordinates YMDhms or iso week YWdhms and utctime, taking  any timezone and DST into account
    
     * Calendar constants, utctimespan like values for Year,Month,Week,Day,Hour,Minute,Second
    
     * Calendar arithmetic, like adding calendar units, e.g. day,month,year etc.
    
     * Calendar arithmetic, like trim/truncate a utctime down to nearest timespan/calendar unit. eg. day
    
     * Calendar arithmetic, like calculate difference in calendar units(e.g days) between two utctime points
    
     * Calendar Timezone and DST handling
    
     * Converting time to string and vice-versa
    
    
    
    
    Notes:
        Please notice that although the calendar concept is complete:
    
    
         * We only implement features as needed in the core and interfaces
    
           Currently this includes most options, including olson time-zone handling
    
           The time-zone support is currently a snapshot of rules ~2023
    
           but we plan to use standard packages like Howard Hinnant's online approach for this later.
    
         * Working range for DST is 1905..2105 (dst is considered 0 outside)
    
         * Working range,validity of the calendar computations is limited to gregorian as of boost::date.
    
         * Working range avoiding overflows is -4000 .. + 4000 Years
    """
    DAY: time
    HOUR: time
    MINUTE: time
    MONTH: time
    QUARTER: time
    RANGE: UtcPeriod
    SECOND: time
    TZ_RANGE: UtcPeriod
    WEEK: time
    YEAR: time
    @property
    def tz_info(self)->TzInfo:
        """
         keeping the time-zone name, utc-offset and DST rules (if any)
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, tz_offset: Union[time,float,int]) -> None:
        """
        creates a calendar with constant tz-offset
            
            Args:
                tz_offset (time): specifies utc offset, time(3600) gives UTC+01 zone
            
        
        """
        ...
    @overload
    def __init__(self, tz_offset: object) -> None:
        """
        creates a calendar with constant tz-offset
            
            Args:
                tz_offset (int): seconds utc offset, 3600 gives UTC+01 zone
            
        
        """
        ...
    @overload
    def __init__(self, olson_tz_id: str) -> None:
        """
        create a Calendar from Olson timezone id, eg. 'Europe/Oslo'
            
            Args:
                olson_tz_id (str): Olson time-zone id, e.g. 'Europe/Oslo'
            
        """
        ...

    def add(self, t: time, delta_t: time, n: int) -> time:
        """
            This function does a calendar semantic add.
            
            Conceptually this is similar to t + deltaT*n
            but with deltaT equal to calendar::DAY,WEEK,MONTH,YEAR
            and/or with dst enabled time-zone the variation of length due to dst
            as well as month/year length is taken into account.
            E.g. add one day, and calendar have dst, could give 23,24 or 25 hours due to dst.
            Similar for week or any other time steps.
            
            Args:
                t (time): timestamp utc seconds since epoch
            
                delta_t (time): timestep in seconds, with semantic interpretation of DAY,WEEK,MONTH,YEAR
            
                n (int): number of timesteps to add
            
            Returns:
                time: t. new timestamp with the added time-steps, seconds utc since epoch
            
            Notes:
                ref. to related functions .diff_units(...) and .trim(..)
            
        """
        ...

    @staticmethod
    def calendar_units(t: time) -> YMDhms:
        """
            returns YMDhms (.year,.month,.day,.hour,.minute..)  for specified t, in the time-zone as given by the calendar
            
            Args:
                t (time): timestamp utc seconds since epoch
            
            Returns:
                YMDhms: calendar_units. calendar units as in year-month-day hour-minute-second
            
        """
        ...

    @staticmethod
    def calendar_week_units(t: time) -> YWdhms:
        """
            returns iso YWdhms, with properties (.iso_year,.iso_week,.week_day,.hour,..)
            for specified t, in the time-zone as given by the calendar
            
            
            Args:
                t (time): timestamp utc seconds since epoch
            
            Returns:
                YWdms: calendar_week_units. calendar units as in iso year-week-week_day hour-minute-second
            
        """
        ...

    def day_of_year(self, t: Union[time,float,int]) -> int:
        """
        returns the day of year for the specified time
            
            Args:
                t (time): time to use for computation
            
            Returns:
                int: day_of_year. in range 1..366
            
        """
        ...

    def diff_units(self, t1: time, t2: time, delta_t: time, trim_policy:trim_policy = trim_policy.TRIM_IN) -> int:
        """
            calculate the distance t1..t2 in specified units, taking dst into account if observed
            The function takes calendar semantics when delta_t is calendar::DAY,WEEK,MONTH,YEAR,
            and in addition also dst if observed.
            e.g. the diff_units of calendar::DAY over summer->winter shift is 1,
            even if the number of hours during those days are 23 and 25 summer and winter transition respectively.
            It computes the calendar semantics of (t2-t1)/deltaT, where deltaT could be calendar units DAY,WEEK,MONTH,YEAR
            
            Args:
                t1 (time): timestamp utc seconds since epoch
            
                t2 (time): timestamp utc seconds since epoch
            
                delta_t (time): timestep in seconds, with semantic interpretation of DAY,WEEK,MONTH,YEAR
            
                trim_policy (trim_policy): Default TRIM_IN, could be TRIM_OUT or TRIM_ROUND.
            
            Returns:
                int: n_units. number of units, so that t2 = c.add(t1,delta_t,n) + remainder(discarded). Depending on the trim_policy, the remainder results will add/subtract one unit to the result.
            
            Notes:
                ref. to related functions .add(...) and .trim(...)
            
        """
        ...

    def quarter(self, t: Union[time,float,int]) -> int:
        """
        returns the quarter of the specified t, -1 if invalid t
            
            Args:
                t (int): timestamp utc seconds since epoch
            
            Returns:
                int: quarter. in range[1..4], -1 if invalid time
            
        """
        ...

    @staticmethod
    def region_id_list() -> StringVector:
        """
        Returns a list over predefined Olson time-zone identifiers
            
            Notes:
                the list is currently static and reflects tz-rules approx as of 2014
            
        """
        ...

    @overload
    def time(self, YMDhms: YMDhms) -> time:
        """
        convert calendar coordinates into time using the calendar time-zone
            
            Args:
                YMDhms (YMDhms): calendar cooordinate structure containg year,month,day, hour,minute,second
            
            Returns:
                int: timestamp. timestamp as in seconds utc since epoch
            
        
        """
        ...
    @overload
    def time(self, YWdhms: YWdhms) -> time:
        """
        convert calendar iso week coordinates structure into time using the calendar time-zone
            
            Args:
                YWdhms (YWdhms): structure containg iso specification calendar coordinates
            
            Returns:
                int: timestamp. timestamp as in seconds utc since epoch
            
        
        """
        ...
    @overload
    def time(self, Y: object, M: object = 1, D: object = 1, h: object = 0, m: object = 0, s: object = 0, us: object = 0) -> time:
        """
        convert calendar coordinates into time using the calendar time-zone
            
            Args:
                Y (int): Year
            
                M (int): Month  [1..12], default=1
            
                D (int): Day    [1..31], default=1
            
                h (int): hour   [0..23], default=0
            
                m (int): minute [0..59], default=0
            
                s (int): second [0..59], default=0
            
                us (int): micro second[0..999999], default=0
            
            Returns:
                time: timestamp. timestamp as in seconds utc since epoch
            
        """
        ...

    def time_from_week(self, Y: object, W: object = 1, wd: object = 1, h: object = 0, m: object = 0, s: object = 0, us: object = 0) -> time:
        """
        convert calendar iso week coordinates into time using the calendar time-zone
            
            Args:
                Y (int): ISO Year
            
                W (int): ISO Week  [1..54], default=1
            
                wd (int): week_day  [1..7]=[mo..su], default=1
            
                h (int): hour   [0..23], default=0
            
                m (int): minute [0..59], default=0
            
                s (int): second [0..59], default=0
            
                us (int): micro second[0..999999], default=0
            
            Returns:
                time: timestamp. timestamp as in seconds utc since epoch
            
        """
        ...

    @overload
    @staticmethod
    def to_string(*args, **kwargs):
        """
        convert time t to readable iso standard string taking 
             the current calendar properties, including timezone into account
            
            Args:
                utctime (time): seconds utc since epoch
            
            Returns:
                str: iso time string. iso standard formatted string,including tz info
            
        
        """
        ...
    @overload
    def to_string(self, utcperiod: UtcPeriod) -> str:
        """
        convert utcperiod p to readable string taking current calendar properties, including timezone into account
            
            Args:
                utcperiod (UtcPeriod): An UtcPeriod object
            
            Returns:
                str: period-string. [start..end>, iso standard formatted string,including tz info
            
        """
        ...

    @staticmethod
    def trim(t:time, delta_t:time) -> time:
        """
            Round time t down to the nearest calendar time-unit delta_t,
            taking the calendar time-zone and dst into account.
            
            
            Args:
                t (time): timestamp utc seconds since epoch
            
                delta_t (time): timestep in seconds, with semantic interpretation of Calendar.{DAY|WEEK|MONTH|YEAR}
            
            Returns:
                time: t. new trimmed timestamp, seconds utc since epoch
            
            Notes:
                ref to related functions .add(t,delta_t,n),.diff_units(t1,t2,delta_t)
            
        """
        ...



class CoreTsVector:
    """
    A raw vector of core time-series.
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class DoubleVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, const_ref_v: Union[DoubleVector,list[float],np.ndarray]) -> None:
        """
        copy construct a new strongly typed list
            
        """
        ...

    def FromNdArray(self) -> DoubleVector:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    def from_numpy(self) -> DoubleVector:
        ...

    def to_numpy(self) -> Any:
        """
        convert to numpy
        """
        ...



class DoubleVectorVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, const_ref_v: Union[DoubleVector,list[float],np.ndarray]Vector) -> None:
        """
        copy construct a new strongly typed list
            
        """
        ...

    def FromNdArray(self) -> DoubleVectorVector:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    def from_numpy(self) -> DoubleVectorVector:
        ...

    def to_numpy(self) -> Any:
        """
        convert to numpy
        """
        ...



class DtsClient:
    """
    The client side part of the distributed time series system(DTSS).
    
    The DtsClient communicate with the DtsServer using an efficient raw socket
    protocol using boost binary serialization. A typical operation would be that
    the DtsClient forwards TsVector that represents lists and structures of
    time-series expressions) to the DtsServer(s), that takes care of binding
    unbound symbolic time-series, evaluate and return the results back to the DtsClient.
    This class is closely related to the :class:`DtsServer` and useful reference
    is also :class:`TsVector` .
    
    
    Best practice for client/server is to use cache following two simple rules(the default):
        
        1. Always caching writes (because then consumers get it fresh and fast).
        2. Always use caching reads(utilize and maintain the adaptive cache).
    
    The only two known very rare and special scenarios where *uncached writes* can be useful
    are when loading large initial content of time-series db.
    Another special scenario, where caching reads should be turned off is when using the
    3rd party dtss backend extension, where the 3rd party db is written/modified
    *outside the control of dtss*
    Also note that the caching works with the ts-terminals, not the result of the expressions.
    When reading time-series expressions, such as `ts = ts1 - ts2`, implementation of the cache is such
    that it contains the ts-terminals (here, `ts1` and `ts2`), not the expression itself (`ts`).
    The `.cache_stats`, provides cache statistics for the server.
    The cache can be flushed, useful for some special cases of loading data outside cache.
    """
    total_clients: int
    @property
    def auto_connect(self)->bool:
        """
         If connections are made as needed, and kept short, otherwise externally managed.
        """
        ...
    @property
    def cache_stats(self)->CacheStats:
        """
         Get the cache_stats (including statistics) on the server.
        """
        ...
    @property
    def compress_expressions(self)->bool:
        """
         If True, the expressions are compressed before sending to the server.
        For expressions of any size, like 100 elements, with expression
        depth 100 (e.g. nested sums), this can speed up
        the transmission by a factor or 3.
        """
        ...
    @compress_expressions.setter
    def compress_expressions(self, value:bool)->None:
        ...
    @property
    def connections(self)->int:
        """
         Get remote server connections.
        """
        ...
    @overload
    def __init__(self, host_port: str, auto_connect: object = True, timeout_ms: object = 1000) -> None:
        """
        Constructs a dts-client with the specifed host_port parameter.
            A connection is immediately done to the server at specified port.
            If no such connection can be made, it raises a RuntimeError.
            
                host_port (string): a string of the format 'host:portnumber', e.g. 'localhost:20000'
            
                auto_connect (bool): default True, connection pr. call. if false, connection last lifetime of object unless explicitely closed/reopened
            
                timeout_ms (int): defalt 1000ms, used for timeout connect/reconnect/close operations
            
        
        """
        ...
    @overload
    def __init__(self, host_ports: Union[StringVector,list[str]], auto_connect: object, timeout_ms: object) -> None:
        """
        Constructs a dts-client with the specifed host_ports parameters.
            A connection is immediately done to the server at specified port.
            If no such connection can be made, it raises a RuntimeError.
            If several servers are passed, the .evaluate and .percentile function will partition the ts-vector between the
            provided servers and scale out the computation
            
                host_ports (StringVector): a a list of string of the format 'host:portnumber', e.g. 'localhost:20000'
            
                auto_connect (bool): default True, connection pr. call. if false, connection last lifetime of object unless explicitly closed/reopened
            
                timeout_ms (int): default 1000ms, used for timeout connect/reconnect/close operations
            
        """
        ...

    def add_geo_ts_db(self, geo_cfg: GeoTimeSeriesConfiguration) -> None:
        """
        Adds a new geo time-series database to the dtss-server with the given specifications
            
                geo_cfg (GeoTimeSeriesConfiguration): the configuration to be added to the server specifying the dimensionality etc.
            
            See also:
                .get_geo_db_ts_info()
            
        """
        ...

    def cache_flush(self) -> None:
        """
        Flush the cache (including statistics) on the server. This can be useful in scenario when cache_on_write=False in the store operations.
            
        """
        ...

    def close(self, timeout_ms: object = 1000) -> None:
        """
        Close the connection. If auto_connect is enabled it will automatically reopen if needed.
            
        """
        ...

    def evaluate(self, ts_vector: TsVector, utcperiod: UtcPeriod, use_ts_cached_read: object = True, update_ts_cache: object = True, clip_result: UtcPeriod = "[not-valid-period>") -> TsVector:
        """
        Evaluates the expressions in the ts_vector.
            If the expression includes unbound symbolic references to time-series,
            these time-series will be passed to the binding service callback
            on the serverside, passing on the specifed utcperiod.
            
            NOTE: That the ts-backing-store, either cached or by read, will return data for:
                * at least the period needed to evaluate the utcperiod
                * In case of cached result, this will currently involve the entire matching cached time-series segment.
            In particular, this means that the returned result **could be larger** than the specified utcperiod, unless you specify `clip_result`
                  Other available methods, such as the expression (x.average(ta)), including time-axis,
                  can be used to exactly control the returned result size.
                  Also note that the semantics of utcperiod is 
                  to ensure that enough data is read from the backend, so that it can evaluate the expressions.
                  Use clip_result argument to clip the time-range of the resulting time-series to fit your need if needed
                  - this will typically be in scenarios where you have not supplied time-axis operations (unbounded eval),
                  and you also are using caching.
                  
            
            See also
             :meth:`DtsClient.percentiles` if you want to evaluate percentiles of an expression.
            
            Args:
                ts_vector (TsVector): a list of time-series (expressions), including unresolved symbolic references
            
                utcperiod (UtcPeriod): the valid non-zero length period that the binding service should read from the backing ts-store/ts-service
            
                use_ts_cached_read (bool): use of server-side ts-cache
            
                update_ts_cache (bool): when reading time-series, also update the cache with the data
            
                clip_result (UtcPeriod): If supplied, clip the time-range of the resulting time-series to cover evaluation f(t) over this period only
            
            Returns:
                TsVector: tsvector. an evaluated list of point time-series in the same order as the input list
            
            See also:
                DtsServer
            
        """
        ...

    def find(self, search_expression: str) -> TsInfoVector:
        """
        Find ts information that fully matches the regular search-expression.
            For the shyft file based backend, take care to specify path elements precisely,
            so that the directories visited is minimised.
            e.g:`a\/.*\/my\.ts`
            Will prune out  any top level directory not starting with `a`,
            but will match any subdirectories below that level.
            Refer to python test-suites for a wide range of examples using find.
            Notice that the regexp search algoritm uses ignore case.
            Please be aware that custom backend by python extension might have different rules.
            
            Args:
                search_expression (str): regular search-expression, to be interpreted by the back-end tss server
            
            Returns:
                TsInfoVector: ts_info_vector. The search result, as vector of TsInfo objects
            
            See also:
                TsInfo,TsInfoVector
            
        """
        ...

    @overload
    def geo_evaluate(self, geo_ts_db_name: str, variables: Union[StringVector,list[str]], ensembles: Union[IntVector,list[int],range], time_axis: TimeAxis, ts_dt: Union[time,float,int], geo_range: GeoQuery, concat: object, cc_dt0: Union[time,float,int], use_cache: object = True, update_cache: object = True) -> GeoTsMatrix:
        """
        Evaluates a geo-temporal query on the server, and return the results
            
            Args:
                geo_ts_db_name (string): The name of the geo_ts_db, e.g. arome, ec, arome_cc ec_cc etc.
            
                variables (StringVector): list  of variables, like 'temperature','precipitation'. If empty, return data for all available variables
            
                ensembles (IntVector): list of ensembles to read, if empty return all available
            
                time_axis (TimeAxis): return geo_ts where t0 matches time-points of this time-axis. If concat, the ta.total_period().end determines how long to extend latest forecast
            
                ts_dt (time): specifies the length of the time-slice to read from each time-series
            
                geo_range (GeoQuery): Specify polygon to include, empty means all
            
                concat (bool): If true, the geo_ts for each ensemble/point is joined together to form one singe time-series, concatenating a slice from each of the forecasts
            
                cc_dt0 (time): concat delta time to skip from beginning of each geo_ts, so you can specify 3h, then select +3h.. slice-end from each forecast
            
                use_cache (bool): use cache if available(speedup)
            
                update_cache (bool): if reading data from backend, also stash it to the cache for faster evaluations
            
            Returns:
                GeoMatrix: r. A matrix where the elements are GeoTimeSeries, accessible using indicies time,variable, ensemble, t0
            
            See also:
                .get_geo_ts_db_info()
            
        
        """
        ...
    @overload
    def geo_evaluate(self, eval_args: GeoEvalArgs, use_cache: object = True, update_cache: object = True) -> GeoTsMatrix:
        """
        Evaluates a geo-temporal query on the server, and return the results
            
            Args:
                eval_args (GeoEvalArgs): complete set of arguments for geo-evaluation, including geo-db, scope for variables, ensembles, time and geo-range
            
                use_cache (bool): use cache if available(speedup)
            
                update_cache (bool): if reading data from backend, also stash it to the cache for faster evaluations
            
            Returns:
                GeoMatrix: r. A matrix where the elements are GeoTimeSeries, accessible using indicies time,variable, ensemble, t0
            
            See also:
                .get_geo_ts_db_info()
            
        """
        ...

    def geo_store(self, geo_ts_db_name: str, tsm: GeoMatrix, replace: object, cache: object = True) -> None:
        """
        Store a ts-matrix with needed dimensions and data to the specified geo-ts-db
            
            Args:
                geo_ts_db_name (string): The name of the geo_ts_db, e.g. arome, ec, arome_cc ec_cc etc.
            
                tsm (TsMatrix): A dense matrix with dimensionality complete for variables, ensembles and geo-points,flexible time-dimension 1..n
            
                replace (bool): Replace existing geo time-series with the new ones, does not extend existing ts, replaces them!
            
                cache (bool): Also put values to the cache
            
            See also:
                .get_geo_ts_db_info(),.geo_evaluate
            
        """
        ...

    def get_container_names(self) -> StringVector:
        """
        Return a list of the names of containers available on the server
            
        """
        ...

    def get_geo_db_ts_info(self) -> GeoTimeSeriesConfigurationVector:
        """
        Returns the configured geo-ts data-bases on the server, so queries can be specified and formulated
            
            Returns:
                GeoTimeseriesConfigurationVector: r. A strongly typed list of GeoTimeseriesConfiguration
            
            See also:
                .geo_evaluate()
            
        """
        ...

    def get_server_version(self) -> str:
        """
        Returns the server version major.minor.patch string, if multiple servers, the version of the first is returned
            
        """
        ...

    def get_ts_info(self, ts_url: str) -> TsInfo:
        """
        Get ts information for a time-series from the backend
            
            Args:
                ts_url (str): Time-series url to lookup ts info for
            
            Returns:
                TsInfo: ts_info. A TsInfo object
            
            See also:
                TsInfo
            
        """
        ...

    def merge_store_ts_points(self, tsv: TsVector, cache_on_write: object = True) -> None:
        """
        Merge the ts-points supplied in the tsv into the existing time-series on the server side.
            The effect of each ts is similar to **as if**:
            
                1. **read** ts.total_period() from ts point store
                2. **in memory appy** the TimeSeries.merge_points(ts) on the read-ts
                3. **write** the resulting merge-result back to the ts-store
            
            This function is suitable for typical data-collection tasks
            where the points collected is from an external source, appears as batches,
            that should just be added to the existing point-set
            
            Args:
                tsv (TsVector): ts-vector with time-series, url-reference and values to be stored at dtss server
            
                cache_on_write (bool): updates the cache with the result of the merge operation, if set to False, this is skipped, notice that this is only useful for very special use-cases.
            
            Returns:
                : None. 
            
            See also:
                TsVector
            
        """
        ...

    def percentiles(self, ts_vector: TsVector, utcperiod: UtcPeriod, time_axis: TimeAxis, percentile_list: Union[IntVector,list[int],range], use_ts_cached_read: object = True, update_ts_cache: object = True) -> TsVector:
        """
        Evaluates the expressions in the ts_vector for the specified utcperiod.
            If the expression includes unbound symbolic references to time-series,
            these time-series will be passed to the binding service callback
            on the serverside.
            
            Args:
                ts_vector (TsVector): a list of time-series (expressions), including unresolved symbolic references
            
                utcperiod (UtcPeriod): the valid non-zero length period that the binding service should read from the backing ts-store/ts-service
            
                time_axis (TimeAxis): the time_axis for the percentiles, e.g. a weekly time_axis
            
                percentile_list (IntVector): a list of percentiles, where -1 means true average, 25=25percentile etc
            
                use_ts_cached_read (bool): utilize server-side cached results
            
                update_ts_cache (bool): when reading time-series, also update the server-side cache
            
            Returns:
                TsVector: tsvector. an evaluated list of percentile time-series in the same order as the percentile input list
            
            See also:
                .evaluate(), DtsServer
            
        """
        ...

    def q_ack(self, name: str, msg_id: str, diagnostics: str) -> None:
        """
        After q_get, q_ack confirms that the message is ok/handled back to the process that called q_put.
            
            Args:
                name (): the name of the queue
            
                msg_id (): the msg_id, required to be unique within current messages keept by the queue
            
                diagnostics (): the freetext diagnostics to but along with the message, we recommend json formatted
            
        """
        ...

    def q_add(self, name: str) -> None:
        """
        Add a a named queue to the dtss server
            
            Args:
                name (): the name of the new queue, required to be unique
            
        """
        ...

    def q_get(self, name: str, max_wait: Union[time,float,int]) -> QueueMessage:
        """
        Get a message out from the named queue, waiting max_wait time for it if it's not already there.
            
            Args:
                name (): the name of the queue
            
                max_wait (): max_time to wait for message to arrive
            
            Returns:
                : q_msg. A queue message consisting of .info describing the message, and the time-series vector .tsv
            
        """
        ...

    def q_list(self) -> StringVector:
        """
        returns a list of defined queues on the dtss server
            
        """
        ...

    def q_maintain(self, name: str, keep_ttl_items: object, flush_all: object = False) -> None:
        """
        Maintains, removes items that has passed through the queue, and are marked as done.
            To flush absolutely all items, pass flush_all=True.
            
            Args:
                name (): the name of the queue
            
                keep_ttl_items (): If true, the ttl set for the done messages are respected, and they are not removed until the create+ttl has expired
            
                flush_all (): removes all items in the queue and kept by the queue, the queue is emptied
            
        """
        ...

    def q_msg_info(self, name: str, msg_id: str) -> QueueMessageInfo:
        """
        From the specified queue, fetch info about specified msg_id.
            By inspecting the provided information, one can see when the messaage is created, fetched, and done with.
            
            Args:
                name (): the name of the queue
            
                msg_id (): the msg_id
            
            Returns:
                : msg_info. the information/state of the identified message
            
        """
        ...

    def q_msg_infos(self, name: str) -> QueueMessageInfoVector:
        """
        Returns all message informations from a queue, including not yet pruned fetched/done messages
            
            Args:
                name (): the name of the queue
            
            Returns:
                : msg_infos. the list of information keept in the named queue
            
        """
        ...

    def q_put(self, name: str, msg_id: str, description: str, ttl: Union[time,float,int], tsv: TsVector) -> None:
        """
        Put a message, as specified with the supplied parameters, into the specified named queue.
            
            Args:
                name (): the name of the queue
            
                msg_id (): the msg_id, required to be unique within current messages keept by the queue
            
                description (): the freetext description to but along with the message, we recommend json formatted
            
                ttl (): time-to-live for the message after done, if specified, the q_maintain process can be asked to keep done messages that have ttl
            
                tsv (): time-series vector, with the wanted payload of time-series
            
        """
        ...

    def q_remove(self, name: str) -> None:
        """
        Removes a named queue from dtss server, including all data in flight on the queue
            
            Args:
                name (): the name of the queue
            
        """
        ...

    def q_size(self, name: str) -> int:
        """
        Returns number of queue messages waiting to be read by q_get.
            
            Args:
                name (): the name of the queue
            
            Returns:
                : unread count. number of elements queued up
            
        """
        ...

    def remove(self, ts_url: str) -> None:
        """
        Remove a time-series from the dtss backend
            The time-series referenced by ``ts_url`` is removed from the backend DtsServer.
            Note that the DtsServer may prohibit removing time-series.
            
            Args:
                ts_url (str): shyft url referencing a time series
            
        """
        ...

    def remove_container(self, container_url: str, delete_from_disk: object = False) -> None:
        """
        remove an internal shyft store container or an external container from the dtss-server.
            container_url on the form shyft://<container>/ will remove internal containers
            all other urls with be forwarded to the remove_external_cb callback on the server
            removal of containers can take a long time to finish
            
            Args:
                container_url (str): url of the container as pr. url definition above
            
                delete_from_disk (bool): Flag to indicate if the container should be deleted from disk
            
        """
        ...

    def remove_geo_ts_db(self, geo_ts_db_name: str) -> None:
        """
        Remove the specified geo time-series database from dtss-server
            
                geo_ts_db_name (string): the name of the geo-ts-database to be removed
            
            See also:
                .get_geo_db_ts_info(),add_geo_ts_db()
            
        """
        ...

    def reopen(self, timeout_ms: object = 1000) -> None:
        """
        (Re)open a connection after close or server restart.
            
        """
        ...

    def set_container(self, name: str, relative_path: str, container_type: str = 'ts_db', cfg: DtssCfg = DtssCfg) -> None:
        """
        create an internal shyft store container to the dtss-server with a root relative path.
            All ts-urls with shyft://<container>/ will resolve
            to this internal time-series storage for find/read/store operations
            will not replace existing containers that have the same name
            
            Args:
                name (str): Name of the container as pr. url definition above
            
                relative_path (str): A valid directory for the container relative to the root path of the server.
            
                container_type (str): one of ('ts_rdb', 'ts_ldb','ts_db'), container type to add.
            
        """
        ...

    def store(self, tsv: TsVector, store_policy: StorePolicy) -> None:
        """
        Store the time-series in the ts-vector in the dtss backend.
            Stores the time-series fragments data passed to the backend.
            If store_policy.strict == True:
            It is semantically stored as if 
            
            first erasing the existing stored points in the range of ts.time_axis().total_period()
            
            then inserting the points of the ts.
            
            Thus, only modifying the parts of time-series that are covered by the ts-fragment passed.
            If there is no previously existing time-series, its merely stores the ts-fragment as the initial
            content and definition of the time-series.
            
            When creating time-series 1st time, pay attention to the time-axis, and point-interpretation as this
            remains the properties of the newly created time-series.
            Storing 15min data to a time-series defined initially as hourly series will raise exception.
            On the otherhand, the variable interval time-series are generic and will accept any
            time-resolution to be stored
            
            If store_policy.strict ==False
            The passed time-series fragment is interpreted as a f(t), and projected to
            the time-axis time-points/intervals of the target time-series
            If the target time-series is a stair-case type (POINT_AVERAGE_VALUE),
            then the true average of the passed time-series fragment is used to align with the target.
            If the target time-series is a linear type (POINT_INSTANT_VALUE),
            then the f(t) of the passed time-series fragment for the time-points of
            the target-series is used.
            The `overwrite_on_write` = True,  is used to *replace* the entire definition of any previously stored time-series.
            This is semantically as if erasing the previously stored time-series and replacing its entire content
            and definition, starting fresh with the newly passed time-series.
            The time-series should be created like this, with url and a concrete point-ts:
            
            >>>   a=sa.TimeSeries(ts_url,ts_points)
            >>>   tsv.append(a)
            
            Args:
                tsv (TsVector): ts-vector with time-series, url-reference and values to be stored at dtss server
            
                store_policy (StorePolicy): Determines how to project the passed time-series fragments to the backend stored time-series
            
                cache_on_write (bool): defaults True, if set to False, the cache is not updated, and should only be considered used in very special use-cases.
            
            Returns:
                : None. 
            
            See also:
                TsVector
            
        """
        ...

    def store_ts(self, tsv: TsVector, overwrite_on_write: object = False, cache_on_write: object = True) -> None:
        """
        Store the time-series in the ts-vector in the dtss backend.
            Stores the time-series fragments data passed to the backend.
            It is semantically stored as if 
            
            first erasing the existing stored points in the range of ts.time_axis().total_period()
            
            then inserting the points of the ts.
            
            Thus, only modifying the parts of time-series that are covered by the ts-fragment passed.
            If there is no previously existing time-series, its merely stores the ts-fragment as the initial
            content and definition of the time-series.
            
            When creating time-series 1st time, pay attention to the time-axis, and point-interpretation as this
            remains the properties of the newly created time-series.
            Storing 15min data to a time-series defined initially as hourly series will raise exception.
            On the otherhand, the variable interval time-series are generic and will accept any
            time-resolution to be stored
            
            The `overwrite_on_write` = True,  is used to *replace* the entire definition of any previously stored time-series.
            This is semantically as if erasing the previously stored time-series and replacing its entire content
            and definition, starting fresh with the newly passed time-series.
            The time-series should be created like this, with url and a concrete point-ts:
            
            >>>   a=sa.TimeSeries(ts_url,ts_points)
            >>>   tsv.append(a)
            
            Args:
                tsv (TsVector): ts-vector with time-series, url-reference and values to be stored at dtss server
            
                overwrite_on_write (bool): When True the backend replaces the entire content and definition of any existing time-series with the passed time-series
            
                cache_on_write (bool): defaults True, if set to False, the cache is not updated, and should only be considered used in very special use-cases.
            
            Returns:
                : None. 
            
            See also:
                TsVector
            
        """
        ...

    def swap_container(self, container_name_a: str, container_name_b: str = False) -> None:
        """
        Swap the backend storage for container `a` and `b`.
            The content of a and b should be equal prior to the call to ensure wanted semantics,
            as well as cache correctness.
            This is the case if `a` is immutable, and copied to `b` prior to the operation.
            If `a` is not permanently immutable, it has to be ensured at least for
            the time where the copy/swap operation is done.
            The intended purpose is to support migration and moving ts-db backends.
            When swap is done, the remove_container can be used for the container that is redunant.
            A typical operation is copy `a`->`a_tmp`, then swap(`a`,`a_tmp`), then remove(`shyft://a_tmp`,True)
            
            Args:
                container_name_a (str): Name of container a
            
                container_name_b (str): Name of container b
            
        """
        ...

    def update_geo_ts_db_info(self, geo_ts_db_name: str, description: str, json: str, origin_proj4: str) -> None:
        """
        Update info fields of the geo ts db configuration to the supplied parameters
            
            Args:
                geo_ts_db_name (string): The name of the geo_ts_db, e.g. arome, ec, arome_cc ec_cc etc.
            
                description (str): The description field of the database
            
                json (str): The user specified json like string
            
                origin_proj4 (str): The origin proj4 field update
            
            See also:
                .get_geo_ts_db_info()
            
        """
        ...



class DtsConnection:
    @property
    def host_port(self)->str:
        """
         Endpoint network address of the remote server, like `host:port`.
        """
        ...
    @property
    def is_open(self)->bool:
        """
         If the connection to the remote server is (still) open.
        """
        ...
    @property
    def reconnect_count(self)->int:
        """
         Number of reconnects to the remote server that have been performed.
        """
        ...
    @property
    def timeout_ms(self)->int:
        """
         Timout for remote server operations, in number milliseconds.
        """
        ...
    def __init__(*args, **kwargs):
        """
        This class cannot be instantiated from Python
        """
        ...



class DtsConnectionVector:
    """
    A strongly typed list of DtsConnection
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class DtsServer:
    """
    A distributed time-series server.
    
    The server part of the Shyft Distributed TimeSeries System(DTSS).
    Capable of processing time-series messages and responding accordingly.
    
    It has dual service interfaces:
        
        1. raw-socket boost serialized binary, use  :class:`DtsClient` 
        2. web-api, web-socket(https/wss w. auth supported) using boost.beast, boost.spirit to process/emit messages. This also supports ts-change subscriptions.
    
    **python customization and extension capability**
    
    The user can setup callback to python to handle unbound symbolic time-series references, `ts-urls`.
    This means that you can use your own ts database backend if you have one that can beat the shyft-internal ts-db.
    
    The DtsServer then resolves symbolic references reading time-series from a service or storage for the specified period.
    The server object will then compute the resulting time-series vector,
    and respond back to clients with the results
    multi-node considerations:
        
        1. firewall/routing: ensure that the port you are using are open for ip-traffic(use ssh-tunnel if you need ssl/tls)
        2. we strongly recommend using linux for performance and longterm stability
    
    The Dts also support master-slave mode, that allows scaling out computations to several Dtss instances, see set_master_slave_mode
    
    **backend storage**
    
    There are 3 internal backends, and customization for external storage as well.
    Internal storage containers:
        
        1. **rocksdb** - by facebook, configurable specifying `ts_rdb` in the `set_container` method
        2. **leveldb** - (deprecated, replaced by rocksdb) by google, configurable specifying `ts_ldb` in the `set_container` method
        3. **filedb** - fast zero overhead, and simple internal binary formats, configurable specifying `ts_db` in the `set_container` method
    
    The kind of backend storage for the backing store ts-containers is specifed in the `set_container` method, for explicit creation of ts-containers.
    Notice that for remotely client created containers for geo time-series storage, the `default_geo_db_type` applies, set to `ts_rdb`.**External storage** can be setup by suppling python callbacks for the `find`,`read`,`store` and `remove_container` hooks.
    To ensure that containers are (remotely)  found and configured after reboot/restart, ensure to provide
    a dtss configuration file where this information is stored.
    Then specifying something else than the `shyft://` pre-fix for the ts-urls, allows any external storage to be used.
    **HPC setup, configure linux os user limits**
    For high performance envirments, the `ulimit`, especially memory, number of files open, needs to be set to
    higher values than the defaults, usually nofiles is 1024, which is to low for HPC apps.
    We recommend 4096, or 8192 or even higher
    for demanding databases.
    For tuning rocksdb, or leveldb, read tuning guides for those libraries, -we provide some basic parameters for tuning,
    but more can be added if needed.
    
    
    See also:
        DtsClient
    """
    @property
    def alive_connections(self)->int:
        """
         returns currently alive connections to the server
        """
        ...
    @property
    def auth_needed(self)->bool:
        """
         returns true if the server is setup with auth-tokens, requires web-api clients to pass a valid token
        """
        ...
    @property
    def cache_max_items(self)->int:
        """
         cache_max_items is the maximum number of time-series identities that are
        kept in memory. Elements exceeding this capacity is elided using the least-recently-used
        algorithm. Notice that assigning a lower value than the existing value will also flush out
        time-series from cache in the least recently used order.
        """
        ...
    @cache_max_items.setter
    def cache_max_items(self, value:int)->None:
        ...
    @property
    def cache_memory_target(self)->int:
        """
         The memory max target in number of bytes.
        If not set directly the following equation is  use:
        cache_memory_target = cache_ts_initial_size_estimate * cache_max_items
        When setting the target directly, number of items in the chache is 
        set so that real memory usage is less than the specified target.
        The setter could cause elements to be flushed out of cache.
        """
        ...
    @cache_memory_target.setter
    def cache_memory_target(self, value:int)->None:
        ...
    @property
    def cache_stats(self)->CacheStats:
        """
         the current cache statistics
        """
        ...
    @property
    def cache_ts_initial_size_estimate(self)->int:
        """
         The initial time-series size estimate in bytes for the cache mechanism.
        memory-target = cache_ts_initial_size_estimate * cache_max_items
        algorithm. Notice that assigning a lower value than the existing value will also flush out
        time-series from cache in the least recently used order.
        """
        ...
    @cache_ts_initial_size_estimate.setter
    def cache_ts_initial_size_estimate(self, value:int)->None:
        ...
    @property
    def cb(self)->Any:
        """
        Callable[[StringVector,UtcPeriod],TsVector]: callback for binding unresolved time-series references to concrete time-series.
        Called *if* the incoming messages contains unbound time-series.
        The signature of the callback function should be TsVector cb(StringVector,utcperiod)
        
        Examples:
        
        >>> from shyft import time_series as sa
        
        >>> def resolve_and_read_ts(ts_ids,read_period):
        >>>     print('ts_ids:', len(ts_ids), ', read period=', str(read_period))
        >>>     ta = sa.TimeAxis(read_period.start, sa.deltahours(1), read_period.timespan()//sa.deltahours(1))
        >>>     x_value = 1.0
        >>>     r = sa.TsVector()
        >>>     for ts_id in ts_ids :
        >>>         r.append(sa.TimeSeries(ta, fill_value = x_value))
        >>>         x_value = x_value + 1
        >>>     return r
        >>> # and then bind the function to the callback
        >>> dtss=sa.DtsServer()
        >>> dtss.cb=resolve_and_read_ts
        >>> dtss.set_listening_port(20000)
        >>> dtss.process_messages(60000)
        """
        ...
    @cb.setter
    def cb(self, value:Any)->None:
        ...
    @property
    def configuration_file(self)->str:
        """
         configuration file to enable persistent container configurations over coldstarts
        """
        ...
    @configuration_file.setter
    def configuration_file(self, value:str)->None:
        ...
    @property
    def default_geo_db_config(self)->GeoTimeSeriesConfiguration:
        """
         Default parameters for geo db created by clients
        """
        ...
    @default_geo_db_config.setter
    def default_geo_db_config(self, value:GeoTimeSeriesConfiguration)->None:
        ...
    @property
    def default_geo_db_type(self)->str:
        """
         default container type for geo db created by clients,(`ts_rdb`,`ts_ldb`,`ts_db`), defaults set to `ts_rdb`
        """
        ...
    @default_geo_db_type.setter
    def default_geo_db_type(self, value:str)->None:
        ...
    @property
    def find_cb(self)->Any:
        """
        Callable[[str],TsInfoVector]: callback for finding time-series using a search-expression.
        Called everytime the .find() method is called.
        The signature of the callback function should be fcb(search_expr: str)->TsInfoVector
        
        Examples:
        
        >>> from shyft import time_series as sa
        
        >>> def find_ts(search_expr: str)->sa.TsInfoVector:
        >>>     print('find:',search_expr)
        >>>     r = sa.TsInfoVector()
        >>>     tsi = sa.TsInfo()
        >>>     tsi.name = 'some_test'
        >>>     r.append(tsi)
        >>>     return r
        >>> # and then bind the function to the callback
        >>> dtss=sa.DtsServer()
        >>> dtss.find_cb=find_ts
        >>> dtss.set_listening_port(20000)
        >>> # more code to invoce .find etc.
        """
        ...
    @find_cb.setter
    def find_cb(self, value:Any)->None:
        ...
    @property
    def geo_ts_read_cb(self)->Any:
        """
        Callable[[GeoTimeSeriesConfiguration,GeoSlice],GeoMatrix]: Callback for reading  geo_ts db.
        Called everytime there is a need for geo_ts not stored in cached.
        The signature of the callback function should be grcb(cfg:GeoTimeSeriesConfiguration, slice:GeoSlice)->GeoMatrix
        """
        ...
    @geo_ts_read_cb.setter
    def geo_ts_read_cb(self, value:Any)->None:
        ...
    @property
    def geo_ts_store_cb(self)->Any:
        """
        Callable[[GeoTimeSeriesConfiguration,GeoMatrix,bool],None]: callback for storing to geo_ts db.
        Called everytime the client.store_geo_ts() method is called.
        The signature of the callback function should be gscb(cfg:GeoTimeSeriesConfiguration, tsm:GeoMatrix, replace:bool)->None
        """
        ...
    @geo_ts_store_cb.setter
    def geo_ts_store_cb(self, value:Any)->None:
        ...
    @property
    def graceful_close_timeout_ms(self)->int:
        """
         how long to let a connection linger after message is processed to allow for
        flushing out reply to client.
        Ref to dlib.net dlib.net/dlib/server/server_kernel_abstract.h.html 
        """
        ...
    @graceful_close_timeout_ms.setter
    def graceful_close_timeout_ms(self, value:int)->None:
        ...
    @property
    def remove_container_cb(self)->Any:
        """
        Callable[[str, bool],None]: callback for removing external containers.
        Called when the .remove_container() method is called with a non-shyft container url.
        The signature of the callback function should be rcb(container_url: string, remove_from_disk: bool)->None
        """
        ...
    @remove_container_cb.setter
    def remove_container_cb(self, value:Any)->None:
        ...
    @property
    def store_ts_cb(self)->Any:
        """
        Callable[[TsVector],None]: callback for storing time-series.
        Called everytime the .store_ts() method is called and non-shyft urls are passed.
        The signature of the callback function should be scb(tsv: TsVector)->None
        
        Examples:
        
        >>> from shyft import time_series as sa
        
        >>> def store_ts(tsv:sa.TsVector)->None:
        >>>     print('store:',len(tsv))
        >>>     # each member is a bound ref_ts with an url
        >>>     # extract the url, decode and store
        >>>     #
        >>>     #
        >>>     return
        >>> # and then bind the function to the callback
        >>> dtss=sa.DtsServer()
        >>> dtss.store_ts_cb=store_ts
        >>> dtss.set_listening_port(20000)
        >>> # more code to invoce .store_ts etc.
        """
        ...
    @store_ts_cb.setter
    def store_ts_cb(self, value:Any)->None:
        ...
    def __init__(self) -> None:
        ...

    def add_auth_tokens(self, tokens: Union[StringVector,list[str]]) -> None:
        """
        Adds auth tokens, and activate authentication.
            The tokens is compared exactly to the autorization token passed in the request.
            Authorization should onlye be used for the https/wss, unless other measures(vpn/ssh tunnels etc.) are used to protect auth tokens on the wire
            Important! Ensure to start_web_api with tls_only=True when using auth!
            
            Args:
                tokens (): list of tokens, where each token is like `Basic dXNlcjpwd2Q=`, e.g: base64 user:pwd
            
        """
        ...

    def auth_tokens(self) -> StringVector:
        """
        returns the registered authentication tokens.
            
        """
        ...

    def cache(self, ts_ids: Union[StringVector,list[str]], ts_vector: TsVector) -> None:
        """
        add/update specified ts_ids with corresponding ts to cache
            please notice that there is no validation of the tds_ids, they
            are threated identifiers,not verified against any existing containers etc.
            Requests that follows, will use the cached item as long as it satisfies
            the identifier and the coverage period requested
            
            Args:
                ts_ids (StringVector): a list of time-series ids
            
                ts_vector (TsVector): a list of corresponding time-series
            
        """
        ...

    def clear(self) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                cb, process_messages(msec),start_server()
            
        """
        ...

    def clear_cache_stats(self) -> None:
        """
        clear accumulated cache_stats
            
        """
        ...

    def close(self) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                cb, process_messages(msec),start_server()
            
        """
        ...

    def fire_cb(self, msg: Union[StringVector,list[str]], rp: UtcPeriod) -> TsVector:
        """
        testing fire cb from c++
        """
        ...

    def flush_cache(self, ts_ids: Union[StringVector,list[str]]) -> None:
        """
        flushes the *specified* ts_ids from cache
            Has only effect for ts-ids that are in cache, non-existing items are ignored
            
            Args:
                ts_ids (StringVector): a list of time-series ids to flush out
            
        """
        ...

    def flush_cache_all(self) -> None:
        """
        flushes all items out of cache (cache_stats remain un-touched)
            
        """
        ...

    def get_container_names(self) -> StringVector:
        """
        Return a list of the names of containers available on the server
            
        """
        ...

    def get_geo_db_ts_info(self) -> GeoTimeSeriesConfigurationVector:
        """
        Returns the configured geo-ts data-bases on the server, so queries can be specified and formulated
            
            Returns:
                GeoTimeseriesConfigurationVector: r. A strongly typed list of GeoTimeseriesConfiguration
            
            See also:
                .geo_evaluate()
            
        """
        ...

    def get_listening_ip(self) -> str:
        """
        Get the current ip listen address
            
            Returns:
                : listening ip. note that 0.0.0.0 means listening for all interfaces
            
        """
        ...

    def get_listening_port(self) -> int:
        """
        returns the port number it's listening at for serving incoming request
        """
        ...

    def get_max_connections(self) -> int:
        """
        returns the maximum number of connections to be served concurrently
            
        """
        ...

    def is_running(self) -> bool:
        """
        true if server is listening and running
            
            See also:
                start_server(),process_messages(msec)
            
        """
        ...

    def process_messages(self, msec: object) -> None:
        """
        wait and process messages for specified number of msec before returning
            the dtss-server is started if not already running
            
            Args:
                msec (int): number of millisecond to process messages
            
            Notes:
                this method releases GIL so that callbacks are not blocked when the
            dtss-threads perform the callback 
            
            See also:
                cb,start_server(),is_running,clear()
            
        """
        ...

    def remove_auth_tokens(self, tokens: Union[StringVector,list[str]]) -> None:
        """
        removes auth tokens, if it matches all available tokens, then deactivate auth requirement for clients
            
            Args:
                tokens (): list of tokens, where each token is like `Basic dXNlcjpwd2Q=`, e.g: base64 user:pwd
            
        """
        ...

    def remove_container(self, container_url: str, delete_from_disk: object = False) -> None:
        """
        remove an internal shyft store container or an external container from the dtss-server.
            container_url on the form shyft://<container>/ will remove internal containers
            all other urls with be forwarded to the remove_external_cb callback on the server
            removal of containers can take a long time to finish
            
            Args:
                container_url (str): url of the container as pr. url definition above
            
                delete_from_disk (bool): Flag to indicate if the container should be deleted from disk
            
        """
        ...

    def set_auto_cache(self, active: object) -> None:
        """
        set auto caching all reads active or passive.
            Default is off, and caching must be done through
            explicit calls to .cache(ts_ids,ts_vector)
            
            Args:
                active (bool): if set True, all reads will be put into cache
            
        """
        ...

    def set_can_remove(self, can_remove: object) -> None:
        """
        Set whether the DtsServer support removing time-series
            The default setting is false, su unless this method is called with true
            as argument the server will not allow removing data using `DtsClient.remove`.
            
            Args:
                can_remove (bool): ``true`` if the server should allow removing data. ``false`` otherwise
            
        """
        ...

    def set_container(self, name: str, root_dir: str, container_type: str = '', cfg: DtssCfg = DtssCfg) -> None:
        """
        set ( or replaces) an internal shyft store container to the dtss-server.
            All ts-urls with shyft://<container>/ will resolve
            to this internal time-series storage for find/read/store operations
            
            Args:
                name (str): Name of the container as pr. url definition above
            
                root_dir (str): A valid directory root for the container
            
                container_type (str): one of ('ts_rdb', 'ts_ldb','ts_db'), container type to add.
            
            Notes:
                currently this call should only be used when the server is not processing messages, - before starting, or after stopping listening operations
            
            
        """
        ...

    def set_geo_ts_db(self, geo_ts_cfg: GeoTimeSeriesConfiguration) -> None:
        """
        This add/replace a geo-ts database to the server, so that geo-related requests
            can be resolved by means of this configuation and the geo-related callbacks.
            
            Args:
                geo_ts_cfg (GeoTimeseriesConfiguration): The configuration for the new geo-ts data-base
            
        """
        ...

    def set_listening_ip(self, ip: str) -> None:
        """
        Set the ip address to specific interface ip. Must be called prior to the start server method
            
            Args:
                ip (): ip address, like 127.0.0.1 for local host only interface
            
        """
        ...

    def set_listening_port(self, port_no: object) -> None:
        """
        set the listening port for the service
            
            Args:
                port_no (int): a valid and available tcp-ip port number to listen on.
                typically it could be 20000 (avoid using official reserved numbers)
            
            Returns:
                None: nothing. 
            
        """
        ...

    def set_master_slave_mode(self, ip: str, port: object, master_poll_time: object, unsubscribe_threshold: object, unsubscribe_max_delay: object) -> None:
        """
        Set master-slave mode, redirecting all IO calls on this dtss to the master ip:port dtss.
            This instance of the dtss is kept in sync with changes done on the master using subscription to changes on the master
            Calculations, and caches are still done locally unloading the computational efforts from the master.
            
            Args:
                ip (str): The ip address where the master dtss is running
            
                port (int): The port number for the master dtss
            
                master_poll_time (time): [s] max time between each update from master, typicall 0.1 s is ok
            
                unsubscribe_threshold (int): minimum number of unsubscribed time-series before also unsubscribing from the master
            
                unsubscribe_max_delay (int): maximum time to delay unsubscriptions, regardless number
            
        """
        ...

    def set_max_connections(self, max_connect: object) -> None:
        """
        limits simultaneous connections to the server (it's multithreaded, and uses on thread pr. connect)
            
            Args:
                max_connect (int): maximum number of connections before denying more connections
            
            See also:
                get_max_connections()
            
        """
        ...

    def start_async(self) -> int:
        """
        (deprecated, use start_server) start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),set_listening_ip,is_running,cb,process_messages(msec)
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
            Notes:
                you should have setup up the callback, cb before calling start_async
            
                Also notice that processing will acquire the GIL
             -so you need to release the GIL to allow for processing messages
            
            See also:
                process_messages(msec)
            
        """
        ...

    def start_server(self) -> int:
        """
        start server listening in background, and processing messages
            
            See also:
                set_listening_port(port_no),set_listening_ip,is_running,cb,process_messages(msec)
            
            Returns:
                in: port_no. the port used for listening operations, either the value as by set_listening_port, or if it was unspecified, a new available port
            
            Notes:
                you should have setup up the callback, cb before calling start_server
            
                Also notice that processing will acquire the GIL
             -so you need to release the GIL to allow for processing messages
            
            See also:
                process_messages(msec)
            
        """
        ...

    def start_web_api(self, host_ip: str, port: object, doc_root: str, fg_threads: object = 2, bg_threads: object = 4, tls_only: object = False) -> int:
        """
        starts the dtss web-api on the specified host_ip, port, doc_root and number of threads
            
            Args:
                host_ip (str): 0.0.0.0 for any interface, 127.0.0.1 for local only etc.
            
                port (int): port number to serve the web_api on, ensure it's available!
            
                doc_root (str): directory from which we will serve http/https documents, like index.html etc.
            
                fg_threads (int): number of web-api foreground threads, typical 1-4 depending on load
            
                bg_threads (int): number of long running  background threads workers to serve dtss-request etc.
            
                tls_only (bool): default false, set to true to enforce tls sessions only.
            
            Returns:
                int: port. real port number used, if 0 is passed as `port` it is auto-allocated
            
        """
        ...

    def stop_server(self, timeout: object = 1000) -> None:
        """
        stop serving connections, gracefully.
            
            See also:
                start_server()
            
        """
        ...

    def stop_web_api(self) -> None:
        """
        Stops any ongoing web-api service
            
        """
        ...

    def swap_container(self, container_name_a: str, container_name_b: str = False) -> None:
        """
        Swap the backend storage for container `a` and `b`.
            The content of a and b should be equal prior to the call to ensure wanted semantics,
            as well as cache correctness.
            This is the case if `a` is immutable, and copied to `b` prior to the operation.
            If `a` is not permanently immutable, it has to be ensured at least for
            the time where the copy/swap operation is done.
            The intended purpose is to support migration and moving ts-db backends.
            When swap is done, the remove_container can be used for the container that is redunant.
            A typical operation is copy `a`->`a_tmp`, then swap(`a`,`a_tmp`), then remove(`shyft://a_tmp`,True)
            
            Args:
                container_name_a (str): Name of container a
            
                container_name_b (str): Name of container b
            
        """
        ...



class DtssCfg:
    """
    Configuration for google level db specific parameters.
    
    Each parameter have reasonable defaults, have a look at google level db documentation for the effect
    of max_file_size, write_buffer_size and compression.
    The ppf remains constant once db is created (any changes will be ignored).
    The other can be changed on persisted/existing databases.
    
    About compression:
    Turns out although very effective for a lot of time-series, it have a single thread performance cost of 2..3x
    native read/write performance due to compression/decompression.
    
    However, for geo dtss we are using multithreaded writes, so performance is limited to  the io-capacity,
    so it might be set to true for those kind of scenarios.
    """
    @property
    def compression(self)->bool:
        """
         (default False), using snappy compression, could reduce storage 1::3 at similar cost of performance
        """
        ...
    @compression.setter
    def compression(self, value:bool)->None:
        ...
    @property
    def ix_cache(self)->int:
        """
         low-level index-cache, could be useful when working with large compressed databases
        """
        ...
    @ix_cache.setter
    def ix_cache(self, value:int)->None:
        ...
    @property
    def log_level(self)->int:
        """
         default warn(200), trace(-1000),debug(0),info(100),error(300),fatal(400)
        """
        ...
    @log_level.setter
    def log_level(self, value:int)->None:
        ...
    @property
    def max_file_size(self)->int:
        """
         (default 100Mega), choose to make a reasonable number of files for storing time-series
        """
        ...
    @max_file_size.setter
    def max_file_size(self, value:int)->None:
        ...
    @property
    def ppf(self)->int:
        """
         (default 1024) ts-points per fragment(e.g.key/value), how large ts is chunked into fragments, read/write operations to key-value storage are in fragment sizes.
        """
        ...
    @ppf.setter
    def ppf(self, value:int)->None:
        ...
    @property
    def test_mode(self)->int:
        """
         for internal use only, should always be set to 0(the default)
        """
        ...
    @test_mode.setter
    def test_mode(self, value:int)->None:
        ...
    @property
    def ts_cache(self)->int:
        """
         low-level data-cache, could be useful in case of very large compressed databases
        """
        ...
    @ts_cache.setter
    def ts_cache(self, value:int)->None:
        ...
    @property
    def write_buffer_size(self)->int:
        """
         (default 10Mega), to balance write io-activity.
        """
        ...
    @write_buffer_size.setter
    def write_buffer_size(self, value:int)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, ppf: object, compress: object, max_file_size: object, write_buffer_size: object, log_level: object = 200, test_mode: object = 0, ix_cache: object = 0, ts_cache: object = 0) -> None:
        """
        construct a DtssCfg  with all values specified
            
        """
        ...



class GeoEvalArgs:
    """
    GeoEvalArgs is used for the geo-evaluate functions.
    
    It describes scope for the geo-evaluate function, in terms of:
      
       - the geo-ts database identifier
       - variables to extract, by names
       - ensemble members (list of ints)
       - temporal, using `t0` from specified time-axis, `+ ts_dt` for time-range
       - spatial, using `points` for a polygon
    
    and optionally:
      
       - the concat postprocessing with parameters
    """
    @property
    def cc_dt0(self)->time:
        """
         concat lead-time
        """
        ...
    @cc_dt0.setter
    def cc_dt0(self, value:time)->None:
        ...
    @property
    def concat(self)->bool:
        """
         postprocess using concatenated forecast, returns 'one' concatenated forecast from many
        """
        ...
    @concat.setter
    def concat(self, value:bool)->None:
        ...
    @property
    def ens(self)->IntVector:
        """
         list of ensembles to return, empty=all, if specified >0
        """
        ...
    @ens.setter
    def ens(self, value:IntVector)->None:
        ...
    @property
    def geo_range(self)->GeoQuery:
        """
         the spatial scope, as simple polygon
        """
        ...
    @geo_range.setter
    def geo_range(self, value:GeoQuery)->None:
        ...
    @property
    def geo_ts_db_id(self)->str:
        """
         the name for the config (keep it minimal)
        """
        ...
    @geo_ts_db_id.setter
    def geo_ts_db_id(self, value:str)->None:
        ...
    @property
    def t0_time_axis(self)->TimeAxis:
        """
         specifies the t0, and  .total_period().end is used as concatenation open-end fill-in length
        """
        ...
    @t0_time_axis.setter
    def t0_time_axis(self, value:TimeAxis)->None:
        ...
    @property
    def ts_dt(self)->time:
        """
         specifies the time-length to read from  each time-series,t0.. t0+ts_dt,
        """
        ...
    @ts_dt.setter
    def ts_dt(self, value:time)->None:
        ...
    @property
    def ts_expressions(self)->TsVector:
        """
         time series expressions to evaluate instead of variables
        """
        ...
    @ts_expressions.setter
    def ts_expressions(self, value:TsVector)->None:
        ...
    @property
    def variables(self)->StringVector:
        """
         the human readable description of this geo ts db
        """
        ...
    @variables.setter
    def variables(self, value:StringVector)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, geo_ts_db_id: str, variables: Union[StringVector,list[str]], ensembles: Union[IntVector,list[int],range], time_axis: TimeAxis, ts_dt: Union[time,float,int], geo_range: GeoQuery, concat: object, cc_dt0: Union[time,float,int]) -> None:
        """
        Construct GeoEvalArgs from specified parameters
            
            Args:
                geo_ts_db_id (str): identifies the geo-ts-db, short, as 'arome', 'ec', as specified with server.add_geo_ts_db(cfg)
            
                variables (StringVector): names of the wanted variables, if empty, return all variables configured
            
                ensembles (IntVector): List of ensembles, if empty, return all ensembles configured
            
                time_axis (TimeAxis): specifies the t0, and  .total_period().end is used as concatenation open-end fill-in length
            
                ts_dt (time): specifies the time-length to read from  each time-series,t0.. t0+ts_dt, and  .total_period().end is used as concatenation open-end fill-in length
            
                geo_range (GeoQuery): the spatial scope of the query, if empty, return all configured
            
                concat (bool): postprocess using concatenated forecast, returns 'one' concatenated forecast from many.
            
                cc_dt0 (time): concat lead-time, skip cc_dt0 of each forecast (offsets the slice you selects)
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_db_id: str, ensembles: Union[IntVector,list[int],range], time_axis: TimeAxis, ts_dt: Union[time,float,int], geo_range: GeoQuery, concat: object, cc_dt0: Union[time,float,int], ts_expressions: TsVector) -> None:
        """
        Construct GeoEvalArgs from specified parameters
            
            Args:
                geo_ts_db_id (str): identifies the geo-ts-db, short, as 'arome', 'ec', as specified with server.add_geo_ts_db(cfg)
            
                ensembles (IntVector): List of ensembles, if empty, return all ensembles configured
            
                time_axis (TimeAxis): specifies the t0, and  .total_period().end is used as concatenation open-end fill-in length
            
                ts_dt (time): specifies the time-length to read from  each time-series,t0.. t0+ts_dt, and  .total_period().end is used as concatenation open-end fill-in length
            
                geo_range (GeoQuery): the spatial scope of the query, if empty, return all configured
            
                concat (bool): postprocess using concatenated forecast, returns 'one' concatenated forecast from many.
            
                cc_dt0 (time): concat lead-time, skip cc_dt0 of each forecast (offsets the slice you selects)
            
                ts_expressions (TsVector): expressions to evaluate, where the existing variables are referred to by index number as a string, ex. TimeSeries('0') 
            
        """
        ...



class GeoGridSpec:
    """
    A point set for a geo-grid, but does not have to be a regular grid.
    It serves the role of defining the spatial representative mid-points for
    a typical spatial  grid, e.g as for arome, or ec forecasts, where the origin shape usually is a box.
    To support general grid-spec, the optional, then equally sized, shapes 
    provides the polygon shape for each individual mid-point.
    """
    @property
    def epsg(self)->int:
        """
         the epsg coordinate system
        """
        ...
    @property
    def points(self)->GeoPointVector:
        """
         the representative mid-points of the spatial grid
        """
        ...
    @property
    def polygons(self)->GeoPointVectorVector:
        """
         the polygons describing the grid, mid-points are centroids of the polygons
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, epsg: object, points: GeoPointVector) -> None:
        """
        Construct a GeoQuery from specified parameters
            
            Args:
                epsg (int): A valid epsg for the spatial points
            
                points (GeoPointVector): 0 or more representative points for the spatial properties of the grid
            
        
        """
        ...
    @overload
    def __init__(self, epsg: object, polygons: GeoPointVectorVector) -> None:
        """
        Construct a GeoQuery from specified parameters
            
            Args:
                epsg (int): A valid epsg for the spatial points
            
                polygons (GeoPointVectorVector): 0 or more representative shapes as polygons, the mid-points are computed based on shapes
            
        """
        ...

    def find_geo_match(self, geo_query: GeoQuery) -> IntVector:
        """
        finds the points int the grid that is covered by the polygon of the geo_query
            note: that currently we only consider the horizontal dimension when matching points
            
            Args:
                geo_query (GeoQuery): A polygon giving an area to capture
            
            Returns:
                IntVector: matches. a list of all points that is inside, or on the border of the specified polygon, in guaranteed ascending point index order
            
        """
        ...



class GeoMatrix:
    """
    GeoMatrix is 4d matrix,index dimensions (t0,variable,ensemble,geo_point)
    to be understood as a slice of a geo-ts-db (slice could be the entire db)
    The element type of the matrix is TimeSeries
    """
    @property
    def shape(self)->GeoMatrixShape:
        """
         the shape of the GeoMatrix
        """
        ...
    def __init__(self, n_t0: object, n_v: object, n_e: object, n_g: object) -> None:
        """
        create GeoMatrix with specified t0,variables,ensemble and geo-point dimensions
            
        """
        ...

    def concatenate(self, cc_dt0: Union[time,float,int], concat_interval: Union[time,float,int]) -> GeoMatrix:
        """
        Concatenate all the forecasts in the GeoMatrix using supplied parameters
            
            Args:
                cc_dt0 (time): skip first period of length cc_dt0 each forecast
            
                concat_interval (time): the nominal length between each ts.time(0) of all time-series
            
            Returns:
                GeoMatrix: tsm. A new concatenated geo-ts-matrix
            
        """
        ...

    def get_ts(self, t: object, v: object, e: object, g: object) -> TimeSeries:
        """
        return self[t,v,e,g] of type TimeSeries
            
        """
        ...

    def set_ts(self, t: object, v: object, e: object, g: object, ts: TimeSeries) -> None:
        """
        performs self[t,v,e,g]= ts
            
        """
        ...



class GeoMatrixShape:
    @property
    def n_e(self)->int:
        """
         number of ensembles
        """
        ...
    @property
    def n_g(self)->int:
        """
         number of geo points
        """
        ...
    @property
    def n_t0(self)->int:
        """
         number of t0, e.g forecasts
        """
        ...
    @property
    def n_v(self)->int:
        """
         number of variables
        """
        ...
    def __init__(self, n_t0: object, n_v: object, n_e: object, n_g: object) -> None:
        """
        Create with specified dimensionality
            
        """
        ...



class GeoPoint:
    """
    GeoPoint commonly used in the shyft::core for
    representing a 3D point in the terrain model.
    The primary usage is in geo-located time-series and the interpolation routines
    
    Absolutely a primitive point model aiming for efficiency and simplicity.
    
    Units of x,y,z are metric, z positive upwards to sky, represents elevation
    x is east-west axis
    y is south-north axis
    """
    @property
    def x(self)->float:
        """
         east->west
        """
        ...
    @x.setter
    def x(self, value:float)->None:
        ...
    @property
    def y(self)->float:
        """
         south->north
        """
        ...
    @y.setter
    def y(self, value:float)->None:
        ...
    @property
    def z(self)->float:
        """
         ground->upwards
        """
        ...
    @z.setter
    def z(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, x: object, y: object, z: object) -> None:
        """
        construct a geo_point with x,y,z 
            
            Args:
                x (float): meter units
            
                y (float): meter units
            
                z (float): meter units
            
        
        """
        ...
    @overload
    def __init__(self, clone: GeoPoint) -> None:
        """
        create a copy
            
            Args:
                clone (GeoPoint): the object to clone
            
        """
        ...

    @staticmethod
    def difference(a: GeoPoint, b: GeoPoint) -> GeoPoint:
        """
        returns GeoPoint(a.x - b.x, a.y - b.y, a.z - b.z)
        """
        ...

    @staticmethod
    def distance2(a: GeoPoint, b: GeoPoint) -> float:
        """
        returns the euclidian distance^2 
        """
        ...

    def distance_measure(self, a: GeoPoint, b: object, p: object) -> float:
        """
        return sum(a-b)^p
        """
        ...

    @overload
    def transform(self, from_epsg: object, to_epsg: object) -> GeoPoint:
        """
        compute transformed point `from_epsg` to `to_epsg` coordinate
            
            Args:
                from_epsg (int): interpret the current point as cartesian epsg coordinate
            
                to_epsg (int): the returned points cartesian epsg coordinate system
            
            Returns:
                GeoPoint: new point. The new point in the specified cartesian epsg coordinate system
            
        
        """
        ...
    @overload
    def transform(self, from_proj4: str, to_proj4: str) -> GeoPoint:
        """
        compute transformed point `from_epsg` to `to_epsg` coordinate
            
            Args:
                from_proj4 (str): interpret the current point as this proj4 specification
            
                to_proj4 (str): the returned points proj4 specified coordinate system
            
            Returns:
                GeoPoint: new point. The new point in the specified proj4 coordinate system
            
        """
        ...

    @staticmethod
    def xy_distance(a: GeoPoint, b: GeoPoint) -> float:
        """
        returns sqrt((a.x - b.x)*(a.x - b.x) + (a.y - b.y)*(a.y - b.y))
        """
        ...

    @staticmethod
    def zscaled_distance(a: GeoPoint, b: GeoPoint, zscale: object) -> float:
        """
        sqrt( (a.x - b.x)*(a.x - b.x) + (a.y - b.y)*(a.y - b.y) + (a.z - b.z)*(a.z - b.z)*zscale*zscale)
        """
        ...



class GeoPointVector:
    """
    A vector, list, of GeoPoints
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: GeoPointVector) -> None:
        """
        Create a clone.
        
        """
        ...
    @overload
    def __init__(self, clone: GeoPointVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def area(self) -> float:
        """
        compute polygon area, if less than 3 points NAN is returned, otherwise boost::geometry::area(polygon)
            
            Returns:
                float: area. the area in m2 (or according to coordinate projection)
            
        """
        ...

    @staticmethod
    def create_from_x_y_z(x: Union[DoubleVector,list[float],np.ndarray], y: Union[DoubleVector,list[float],np.ndarray], z: Union[DoubleVector,list[float],np.ndarray]) -> GeoPointVector:
        """
        Create a GeoPointVector from x,y and z DoubleVectors of equal length
        """
        ...

    def extend(self, arg2: object) -> None:
        ...

    def mid_point(self) -> GeoPoint:
        """
        compute polygon mid point x,y as boost::geometry::centroid, and z as average of points.z(leaving out the closing point).
            
            Returns:
                GeoPoint: mid_point. The computed mid point, (nan,nan,nan) if there are no points
            
        """
        ...

    def nearest_neighbours(self, p: GeoPoint, n: object, z_scale: object = 0.0) -> GeoPointVector:
        """
        Return the `n` nearest neighbours of point `p` using `z_scaled` distance  for `z` direction
            
            Args:
                p (GeoPoint): Finds nearest neighbours to this point
            
                n (int): Number of neighbours to find
            
                z_scale (float): Default 0.0, e.g. only x,y direction counts, if 1.0, z counts equal etc.
            
            Returns:
                GeoPointVector: neighbours. The neighbours found
            
        """
        ...

    def polygon_buffer(self, distance: object) -> GeoPointVector:
        """
        Computes the buffer polygon for the supplied points using the `distance` parameter.
            boost.geometry.buffer algorithm is used, and if it results in multiple polygons, then
            boost.convex_hull is used to reduce it from many to one polygon envelope.
            If number of points is large, currently > 50,  a reduction algorithm using convex_hull is applied as first step.
            On the return, the point `z` coordinate is set to the average of the polygon points `z`.
            
            Args:
                distance (float): The buffer distance to nearest point in self
            
            Returns:
                GeoPointVector: polygon. A closed buffer polygon surrounding the passed points
            
        """
        ...

    @overload
    def transform(self, from_epsg: object, to_epsg: object) -> GeoPointVector:
        """
        compute transformed point `from_epsg` to `to_epsg` coordinate
            
            Args:
                from_epsg (int): interpret the current point as cartesian epsg coordinate
            
                to_epsg (int): the returned points cartesian epsg coordinate system
            
            Returns:
                GeoPointVector: new point. The new points in the specified cartesian epsg coordinate system
            
        
        """
        ...
    @overload
    def transform(self, from_proj4: str, to_proj4: str) -> GeoPointVector:
        """
        compute transformed point `from_epsg` to `to_epsg` coordinate
            
            Args:
                from_proj4 (str): interpret the current point as this proj4 specification
            
                to_proj4 (str): the returned points proj4 specified coordinate system
            
            Returns:
                GeoPointVector: new GeoPointVector. The new points in the specified proj4 coordinate system
            
        """
        ...



class GeoPointVectorVector:
    """
    A strongly typed list, of GeoPointVectors
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: GeoPointVectorVector) -> None:
        """
        Create a clone.
        
        """
        ...
    @overload
    def __init__(self, clone: GeoPointVectorVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def areas(self) -> DoubleVector:
        """
        Returns the polygon area for each polygon in the list
            
            Args:
            Returns:
                DoubleVector: areas. The computed polygon areas
            
        """
        ...

    def extend(self, arg2: object) -> None:
        ...

    def intersection_areas(self, polygon: GeoPointVector, n_max_neighbours: object = 8) -> DoubleVector:
        """
        Returns the intersection area with `polygon`, each polygon in the list
            Notice that the algo first compute the 8 nearest neighbours, then computes intersections of those.
            
            Args:
                polygon (GeoPointVector): The polygon to compute the intersection with
            
                n_max_neighbours (int): Limits the intersection work considering only the nearest neighbours, default 8
            
            Returns:
                DoubleVector: areas. The computed intersection area for each polygon in polygons, -the same order
            
        """
        ...

    def mid_points(self) -> GeoPointVector:
        """
        Returns the polygon mid_point area for each polygon in the list
            
            Args:
            Returns:
                GeoPointVector: mid_points. The computed mid-points
            
        """
        ...



class GeoQuery:
    """
    A query as a polygon with specified geo epsg coordinate system
    """
    @property
    def epsg(self)->int:
        """
         the epsg coordinate system
        """
        ...
    @property
    def polygon(self)->GeoPointVector:
        """
         the polygon giving the spatial scope
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, epsg: object, points: GeoPointVector) -> None:
        """
        Construct a GeoQuery from specified parameters
            
            Args:
                epsg (int): A valid epsg for the polygon, and also wanted coordinate system
            
                points (GeoPointVector): 3 or more points forming a polygon that is the spatial scope
            
        """
        ...



class GeoSlice:
    """
    Keeps data that describes as slice into the t0-variable-ensemble-geo, (t,v,e,g), space.
    It is the result-type of GeoTimeSeriesConfiguration.compute(GeoEvalArgs)
    and is passed to the geo-db-read callback to specify wanted time-series to read.
    Note that the content of a GeoSlice can only be interpreted in terms of the
    GeoTimeSeriesConfiguration it is derived from.
    The indices and values of the slice, strongly relates to the definition of it's geo-tsdb.
    """
    @property
    def e(self)->IntVector:
        """
         list of ensembles, each in range 0..GeoTimeSeriesConfiguration.n_ensembles-1
        """
        ...
    @e.setter
    def e(self, value:IntVector)->None:
        ...
    @property
    def g(self)->IntVector:
        """
         list of geo-point idx, each defined by GeoTimeSeriesConfiguration.grid.points[i]
        """
        ...
    @g.setter
    def g(self, value:IntVector)->None:
        ...
    @property
    def period(self)->UtcPeriod:
        """
         slice period as [skip_dt..skip_dt+ts_dt) from each ts relative t0
        """
        ...
    @property
    def skip_dt(self)->time:
        """
         time length to skip from start of each time-series, [t0+skip_dt.. t0+skip_dt+ts_dt>
        """
        ...
    @skip_dt.setter
    def skip_dt(self, value:time)->None:
        ...
    @property
    def t(self)->UtcTimeVector:
        """
         list of t0-time points, each of them should exist in GeoTimeSeriesConfiguration.t0_times
        """
        ...
    @t.setter
    def t(self, value:UtcTimeVector)->None:
        ...
    @property
    def ts_dt(self)->time:
        """
         time length to read from each time-series, [t0+skip_dt.. t0+skip_dt+ts_dt>
        """
        ...
    @ts_dt.setter
    def ts_dt(self, value:time)->None:
        ...
    @property
    def v(self)->IntVector:
        """
         list of variables idx, each defined by GeoTimeSeriesConfiguration.variables[i]
        """
        ...
    @v.setter
    def v(self, value:IntVector)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, v: Union[IntVector,list[int],range], g: Union[IntVector,list[int],range], e: Union[IntVector,list[int],range], t: Union[UtcTimeVector,list[time],list[float],np.ndarray], ts_dt: Union[time,float,int], skip_dt: Union[time,float,int]) -> None:
        """
        Construct a GeoSlice from supplied vectors.
            
            Args:
                v (IntVector): list of variables idx, each defined by GeoTimeSeriesConfiguration.variables[i]
            
                e (IntVector): list of ensembles, each in range 0..GeoTimeSeriesConfiguration.n_ensembles-1
            
                g (IntVector): list of geo-point idx, each defined by GeoTimeSeriesConfiguration.grid.points[i]
            
                t (UtcTimeVector): list of t0-time points, each of them should exist in GeoTimeSeriesConfiguration.t0_times
            
                ts_dt (time): time-length to read from each time-series, we read from [t0+skip_dt.. t0+skip_dt+ts_dt>
            
                skip_dt (time): time-length to skip from start each time-series, we read from [t0+skip_dt.. t0+skip_dt+ts_dt>
            
        """
        ...



class GeoTimeSeries:
    """
    A minimal geo-located time-series, a time-series plus a representative 3d mid_point
    """
    @property
    def mid_point(self)->GeoPoint:
        """
         the mid-point(of an area) for which the assigned time-series is valid
        """
        ...
    @mid_point.setter
    def mid_point(self, value:GeoPoint)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         the assigned time-series
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, mid_point: GeoPoint, ts: TimeSeries) -> None:
        """
        Construct a GeoTimeSeries
            
            Args:
                mid_point (GeoPoint): The 3d location representative for ts
            
                ts (TimeSeries): Any kind of TimeSeries
            
        """
        ...



class GeoTimeSeriesConfiguration:
    """
    Contain minimal  description to efficiently work with arome/ec  forecast data
    It defines the spatial, temporal and ensemble dimensions available, and
    provides means of mapping a GeoQuery to a set of ts_urls that 
    serves as keys for manipulating and assembling forcing input data
    for example to the shyft hydrology region-models.
    """
    @property
    def description(self)->str:
        """
         the human readable description of this geo ts db
        """
        ...
    @description.setter
    def description(self, value:str)->None:
        ...
    @property
    def dt(self)->time:
        """
         the (max) length of each geo-ts, so geo_ts total_period is [t0..t0+dt>
        """
        ...
    @property
    def grid(self)->GeoGridSpec:
        """
         the spatial grid definition
        """
        ...
    @property
    def json(self)->str:
        """
         a json formatted string with custom data as needed
        """
        ...
    @json.setter
    def json(self, value:str)->None:
        ...
    @property
    def n_ensembles(self)->int:
        """
         number of ensembles available, range 1..n
        """
        ...
    @property
    def name(self)->str:
        """
         the name for the config (keep it minimal)
        """
        ...
    @name.setter
    def name(self, value:str)->None:
        ...
    @property
    def origin_proj4(self)->str:
        """
         informative only, if not empty, specifies the origin proj4 of this dataset
        """
        ...
    @origin_proj4.setter
    def origin_proj4(self, value:str)->None:
        ...
    @property
    def prefix(self)->str:
        """
         ts-url prefix, like shyft:// for internally stored ts, or geo:// for externally stored parts
        """
        ...
    @property
    def t0_time_axis(self)->TimeAxis:
        """
         t0 time-points as time-axis
        """
        ...
    @property
    def t0_times(self)->UtcTimeVector:
        """
         list of time-points, where there are registered/available time-series
        """
        ...
    @t0_times.setter
    def t0_times(self, value:UtcTimeVector)->None:
        ...
    @property
    def variables(self)->StringList:
        """
         the list of available properties, like short keys for precipitation,temperature etc
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, prefix: str, name: str, description: str, grid: GeoGridSpec, t0_times: Union[UtcTimeVector,list[time],list[float],np.ndarray], dt: Union[time,float,int], n_ensembles: object, variables: Union[StringVector,list[str]], json: str = '', origin_proj4: str = '') -> None:
        """
        Construct a GeoQuery from specified parameters
            
            Args:
                prefix (str): ts-url prefix, like shyft:// for internally stored ts, or geo:// for externally stored parts
            
                name (str): A shortest possible unique name of the configuration
            
                description (str): a human readable description of the configuration
            
                grid (GeoGridSpec): specification of the spatial grid
            
                t0_times (UtcTimeVector): List of time where we have register time-series,e.g forecast times, first time-point
            
                dt (time): the (max) length of each geo-ts, so geo_ts total_period is [t0..t0+dt>
            
                n_ensembles (int): number of ensembles available, must be >0, 1 if no ensembles
            
                variables (StringVector): list of minimal keys, representing temperature, precipitation etc
            
                json (str): A user specified json string
            
                origin_proj4 (str): The proj4 string for the origin transform of this dataset
            
        """
        ...

    def bounding_box(self, slice: GeoSlice) -> GeoPointVector:
        """
        Compute the 3D bounding_box, as two GeoPoints containing the min-max of x,y,z of points in the GeoSlice
            Could be handy when generating queries to externally  stored geo-ts databases like netcdf etc.
            See also convex_hull().
            
            Args:
                slice (GeoSlice): a geo-slice with specified dimensions in terms of t0, variables, ensembles,geo-points
            
            Returns:
                GeoPointVector: bbox. with two GeoPoints, [0] keeping the minimum x,y,z, and [1] the maximum x,y,z
            
        """
        ...

    def compute(self, eval_args: GeoEvalArgs) -> GeoSlice:
        """
        Compute the GeoSlice from evaluation arguments
            
            Args:
                eval_args (GeoEvalArgs): Specification to evaluate
            
            Returns:
                GeoSlice: geo_slice. A geo-slice describing (t0,v,e,g) computed
            
        """
        ...

    def convex_hull(self, slice: GeoSlice) -> GeoPointVector:
        """
        Compute the 2D convex hull, as a list of GeoPoints describing the smallest convex planar polygon 
            containing all points in the slice wrt. x,y. 
            The returned point sequence is 'closed', i.e the first and last point in the sequence are equal. 
            See also bounding_box().
            
            Args:
                slice (GeoSlice): a geo-slice with specified dimensions in terms of t0, variables, ensembles,geo-points
            
            Returns:
                GeoPointVector: hull. containing the sequence of points of the convex hull polygon.
            
        """
        ...

    def create_geo_ts_matrix(self, slice: GeoSlice) -> GeoTsMatrix:
        """
        Creates a GeoTsMatrix(element type is GeoTimeSeries) to hold the values according to dimensionality of GeoSlice
            
            Args:
                slice (GeoSlice): a geo-slice with specified dimensions in terms of t0, variables, ensembles,geo-points
            
            Returns:
                GeoTsMatrix: geo_ts_matrix. ready to be filled in with points and time-series
            
        """
        ...

    def create_ts_matrix(self, slice: GeoSlice) -> GeoMatrix:
        """
        Creates a GeoMatrix (element type is TimeSeries only) to hold the values according to dimensionality of GeoSlice
            
            Args:
                slice (GeoSlice): a geo-slice with specified dimensions in terms of t0, variables, ensembles,geo-points
            
            Returns:
                GeoMatrix: ts_matrix. ready to be filled in time-series(they are all empty/null)
            
        """
        ...

    def find_geo_match_ix(self, geo_query: GeoQuery) -> IntVector:
        """
        Returns the indices to the points that matches the geo_query (polygon)
            
            Args:
                geo_query (GeoQuery): The query, polygon that matches the spatial scope
            
            Returns:
                IntVector: point_indexes. The list of indices that  matches the geo_query
            
        """
        ...



class GeoTimeSeriesConfigurationVector:
    """
    A strongly typed list of GeoTimeSeriesConfigurations
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: GeoTimeSeriesConfigurationVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class GeoTimeSeriesVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> Any:
        """
        Create an empty TsVector
            
        
        """
        ...
    @overload
    def __init__(self, geo_ts_list: list) -> Any:
        """
        Create a GeoTimeSeriesVector from a python list of GeoTimeSeries
            
            Args:
                geo_ts_list (List[GeoTimeSeries]): A list of GeoTimeSeries
            
        
        """
        ...
    @overload
    def __init__(self, time_axis: TimeAxis, geo_points: GeoPointVector, np_array: object, point_fx: point_interpretation_policy) -> Any:
        """
        Create a GeoTimeSeriesVector from time-axis,geo-points,2d-numpy-array and point-interpretation
            
            Args:
                time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
            
                geo_points (GeoPointVector): the geo-positions for the time-series, should be of length n_ts
            
                np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
            
                point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
            
            Returns:
                GeoTimeSeriesVector: GeoTimeSeriesVector. a GeoTimeSeriesVector of length first np_array dim, n_ts, each with geo-point and time-series with time-axis, values and point_fx
            
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    def extract_ts_vector(self) -> TsVector:
        """
        Provides a TsVector of the time-series part of GeoTimeSeries
            
            Returns:
                TsVector: ts-vector. A TsVector(shallow copy) of the time-series part of GeoTsVector
            
        """
        ...

    def values_at_time(self, t: Union[time,float,int]) -> DoubleVector:
        """
        The values at specified time as a DoubleVector, that you can use .to_numpy() to get np array from
            This function can be suitable if you are doing area-animated (birds-view) presentations
            
            Args:
                t (time): the time that should be used for getting each value
            
            Returns:
                DoubleVector: values. The evaluated geo.ts(t) for all items in the vector
            
        """
        ...



class GeoTsMatrix:
    """
    GeoTsMatrix is 4d matrix,index dimensions `(t0,variable,ensemble,geo_point)`
    to be understood as a slice of a geo-ts-db (slice could be the entire db)
    The element types of the matrix is  :class:`GeoTimeSeries` 
    """
    @property
    def shape(self)->GeoMatrixShape:
        """
         The shape of a GeoMatrix in terms of forecasts(n_t0),variables(n_v),ensembles(n_e) and geopoints(n_g)
        """
        ...
    def __init__(self, n_t0: object, n_v: object, n_e: object, n_g: object) -> None:
        """
        create GeoTsMatrix with specified t0,variables,ensemble and geo-point dimensions
            
        """
        ...

    def concatenate(self, cc_dt0: Union[time,float,int], concat_interval: Union[time,float,int]) -> GeoTsMatrix:
        """
        Concatenate all the forecasts in the GeoTsMatrix using supplied parameters
            
            Args:
                cc_dt0 (time): skip first period of length cc_dt0 each forecast
            
                concat_interval (time): the nominal length between each ts.time(0) of all time-series
            
            Returns:
                GeoTsMatrix: tsm. A new concatenated geo-ts-matrix
            
        """
        ...

    def evaluate(self) -> GeoTsMatrix:
        """
        Apply the expression to each time-series of the specified variable.
            
            Args:
            Returns:
                GeoTsMatrix: GeoTsMatrix. A new GeoTsMatrix, where all time-series is evaluated
            
        """
        ...

    def extract_geo_ts_vector(self, t: object, v: object, e: object) -> GeoTimeSeriesVector:
        """
        Given given arguments, return the GeoTimeSeriesVector suitable for constructing GeoPointSource for hydrology region-environments forcing data
            
            Args:
                t (int): the forecast index, e.g. selects specific forecast, in case of several (t0)
            
                v (int): the variable index, e.g. selects temperature,precipitation etc.
            
                e (int): the ensemble index, in case of many ensembles, select specific ensemble
            
            Returns:
                GeoTimeSeriesVector: GeoTimeSeriesVector. The GeoTsVector for selected forcast time t, variable and ensemble
            
        """
        ...

    def get_geo_point(self, t: object, v: object, e: object, g: object) -> GeoPoint:
        """
        return self[t,v,e,g].mid_point of type GeoPoint
            
        """
        ...

    def get_ts(self, t: object, v: object, e: object, g: object) -> TimeSeries:
        """
        return self[t,v,e,g] of type TimeSeries
            
        """
        ...

    def set_geo_point(self, t: object, v: object, e: object, g: object, point: GeoPoint) -> None:
        """
        performs self[t,v,e,g].mid_point= point
            
        """
        ...

    def set_ts(self, t: object, v: object, e: object, g: object, ts: TimeSeries) -> None:
        """
        performs self[t,v,e,g].ts= ts
            
        """
        ...

    @overload
    def transform(self, variable: object, expression: TimeSeries) -> GeoTsMatrix:
        """
        Apply the expression to each time-series of the specified variable.
            
            Args:
                variable (): the variable index to select the specific variable, use -1 to apply to all
            
                expr (): ts expression, like 2.0*TimeSeries('x'), where x will be substituted with the variable, notice that its a required to be just one unbound time-series with the reference name 'x'.
            
            Returns:
                GeoTsMatrix: GeoTsMatrix. A new GeoTsMatrix, where the time-series for the specified variable is transformed
            
        
        """
        ...
    @overload
    def transform(self, expr_vector: TsVector) -> GeoTsMatrix:
        """
        Apply the expression to each time-series of the specified variable.
            
            Args:
                expr_vector (): ts expressions, like 2.0*TimeSeries('0'), where 0 will be substituted with the corresponding variable, notice that its a required to be just one unbound time-series with the reference name '0'.
            
            Returns:
                GeoTsMatrix: GeoTsMatrix. A new GeoTsMatrix, where the time-series for the specified variable is transformed
            
        """
        ...



class IcePackingParameters:
    """
    Parameter pack controlling ice packing computations.
    See `TimeSeries.ice_packing` for usage.
    """
    @property
    def threshold_temperature(self)->float:
        """
         The threshold temperature for ice packing to occur. Ice packing will occur
        when the average temperature in the `window` period is less than the threshold.
        """
        ...
    @threshold_temperature.setter
    def threshold_temperature(self, value:float)->None:
        ...
    @property
    def threshold_window(self)->time:
        """
         The period back in seconds for which the average temperature is computed when
        looking for ice packing.
        """
        ...
    @threshold_window.setter
    def threshold_window(self, value:time)->None:
        ...
    @overload
    def __init__(self, threshold_window: Union[time,float,int], threshold_temperature: object) -> None:
        """
        Defines a paramter pack for ice packing detection.
            
            
            Args:
                threshold_window (utctime): Positive,  seconds for the lookback window.
            
                threshold_temperature (float): Floating point threshold temperature.
            
        
        """
        ...
    @overload
    def __init__(self, threshold_window: object, threshold_temperature: object) -> None:
        """
        Defines a paramter pack for ice packing detection.
            
            
            Args:
                threshold_window (int): Positive integer seconds for the lookback window.
            
                threshold_temperature (float): Floating point threshold temperature.
            
        """
        ...



class IcePackingRecessionParameters:
    """
    Parameter pack controlling ice packing recession computations.
    See `TimeSeries.ice_packing_recession` for usage.
    """
    @property
    def alpha(self)->float:
        """
         Parameter controlling the curving of the recession curve.
        """
        ...
    @alpha.setter
    def alpha(self, value:float)->None:
        ...
    @property
    def recession_minimum(self)->float:
        """
         The minimum value of the recession curve.
        """
        ...
    @recession_minimum.setter
    def recession_minimum(self, value:float)->None:
        ...
    def __init__(self, alpha: object, recession_minimum: object) -> None:
        """
        Defines a parameter pack for ice packing reduction using a simple recession for the water-flow.
            
            
            Args:
                alpha (float): Recession curve curving parameter.
            
                recession_minimum (float): Minimum value for the recession.
            
        """
        ...



class IntVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, const_ref_v: Union[IntVector,list[int],range]) -> None:
        """
        copy construct a new strongly typed list
            
        """
        ...

    def FromNdArray(self) -> IntVector:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    def from_numpy(self) -> IntVector:
        ...

    def to_numpy(self) -> Any:
        """
        convert to numpy
        """
        ...



class KrlsRbfPredictor:
    """
    Time-series predictor using the KRLS algorithm with radial basis functions.
    
    The KRLS (Kernel Recursive Least-Squares) algorithm is a kernel regression
    algorithm for aproximating data, the implementation used here is from:
    
    .. _DLib:
        http://dlib.net/ml.html#krls
    
    This predictor uses KRLS with radial basis functions (RBF).
    Other related  :meth:`shyft.time_series.TimeSeries.krls_interpolation`  :meth:`shyft.time_series.TimeSeries.TimeSeries.get_krls_predictor` 
    
    
    Examples:
    
    >>>
    >>> import numpy as np
    >>> import matplotlib.pyplot as plt
    >>> from shyft.time_series import (
    ...     Calendar, utctime_now, deltahours,
    ...     TimeAxis, TimeSeries,
    ...     KrlsRbfPredictor
    ... )
    >>>
    >>> # setup
    >>> cal = Calendar()
    >>> t0 = utctime_now()
    >>> dt = deltahours(3)
    >>> n = 365*8  # one year
    >>>
    >>> # ready plot
    >>> fig, ax = plt.subplots()
    >>> 
    >>> # shyft objects
    >>> ta = TimeAxis(t0, dt, n)
    >>> pred = KrlsRbfPredictor(
    ...     dt=deltahours(8),
    ...     gamma=1e-5,  # NOTE: this should be 1e-3 for real data
    ...     tolerance=0.001
    ... )
    >>>
    >>> # generate data
    >>> total_series = 4
    >>> data_range = np.linspace(0, 2*np.pi, n)
    >>> ts = None  # to store the final data-ts
    >>> # -----
    >>> for i in range(total_series):
    >>>     data = np.sin(data_range) + (np.random.random(data_range.shape) - 0.5)/5
    >>>     ts = TimeSeries(ta, data)
    >>>     # -----
    >>>     training_mse = pred.train(ts)  # train the predictor
    >>>     # -----
    >>>     print(f'training step {i+1:2d}: mse={training_mse}')
    >>>     ax.plot(ta.time_points[:-1], ts.values, 'bx')  # plot data
    >>>
    >>> # prediction
    >>> ts_pred = pred.predict(ta)
    >>> ts_mse = pred.mse_ts(ts, points=3)  # mse using 7 point wide filter
    >>>                                     # (3 points before/after)
    >>>
    >>> # plot interpolation/predicton on top of results
    >>> ax.plot(ta.time_points[:-1], ts_mse.values, '0.6', label='mse')
    >>> ax.plot(ta.time_points[:-1], ts_pred.values, 'r-', label='prediction')
    >>> ax.legend()
    >>> plt.show()
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, dt: Union[time,float,int], gamma: object = 0.001, tolerance: object = 0.01, size: object = 1000000) -> None:
        """
        Construct a new predictor.
            
            Args:
                dt (float): The time-step in seconds the predictor is specified for.
                Note that this does not put a limit on time-axes used, but for best results it should be
                approximatly equal to the time-step of time-axes used with the predictor. In addition it
                should not be to long, else you will get poor results. Try to keep the dt less than a day,
                3-8 hours is usually fine.
            
                gamma (float (optional)): Determines the width of the radial basis functions for
                the KRLS algorithm. Lower values mean wider basis functions, wider basis functions means faster
                computation but lower accuracy. Note that the tolerance parameter also affects speed and accurcy.
                A large value is around `1E-2`, and a small value depends on the time step. By using values larger
                than `1E-2` the computation will probably take to long. Testing have reveled that `1E-3` works great
                for a time-step of 3 hours, while a gamma of `1E-2` takes a few minutes to compute. Use `1E-4` for a
                fast and tolerably accurate prediction.
                Defaults to `1E-3`
            
                tolerance (float (optional)): The krls training tolerance. Lower values makes the prediction more accurate,
                but slower. This typically have less effect than gamma, but is usefull for tuning. Usually it should be
                either `0.01` or `0.001`.
                Defaults to `0.01`
            
                size (int (optional)): The size of the "memory" of the predictor. The default value is
                usually enough. Defaults to `1000000`.
            
        """
        ...

    def clear(self) -> None:
        """
        Clear all training data from the predictor.
            
        """
        ...

    def mse_ts(self, ts: TimeSeries, points: object = 0) -> TimeSeries:
        """
        Compute a mean-squared error time-series of the predictor relative to the supplied ts.
            
            Args:
                ts (TimeSeries): Time-series to compute mse against.
            
                points (int (optional)): Positive number of extra points around each point to use for mse.
                Defaults to 0.
            
            Returns:
                TimeSeries: mse_ts. Time-series with mean-squared error values.
            
            See also:
                KrlsRbfPredictor.predictor_mse, KrlsRbfPredictor.predict
            
        """
        ...

    def predict(self, ta: TimeAxis) -> TimeSeries:
        """
        Predict a time-series for for time-axis.
            
            Notes:
                The predictor will predict values outside the range of the values it is trained on, but these
            
                values will often be zero. This may also happen if there are long gaps in the training data
            
                and you try to predict values for the gap. Using wider basis functions partly remedies this,
            
                but makes the prediction overall less accurate.
            
            Args:
                ta (TimeAxis): Time-axis to predict values for.
            
            Returns:
                TimeSeries: ts. Predicted time-series.
            
            See also:
                KrlsRbfPredictor.mse_ts, KrlsRbfPredictor.predictor_mse
            
        """
        ...

    def predictor_mse(self, ts: TimeSeries, offset: object = 0, count: object = npos, stride: object = 1) -> float:
        """
        Compute the predictor mean-squared prediction error for count first from ts.
            
            Args:
                ts (TimeSeries): Time-series to compute mse against.
            
                offset (int (optional)): Positive offset from the start of the time-series. Default to 0.
            
                count (int (optional)): Positive number of samples from the time-series to to use.
                Default to the maximum value.
            
                stride (int (optional)): Positive stride between samples from the time-series. Defaults to 1.
            
            See also:
                KrlsRbfPredictor.predict, KrlsRbfPredictor.mse_ts
            
        """
        ...

    def train(self, ts: TimeSeries, offset: object = 0, count: object = npos, stride: object = 1, iterations: object = 1, mse_tol: object = 0.001) -> float:
        """
        Train the predictor using samples from ts.
            
            Args:
                ts (TimeSeries): Time-series to train on.
            
                offset (int (optional)): Positive offset from the start of the time-series. Default to 0.
            
                count (int (optional)): Positive number of samples to to use. Default to the maximum value.
            
                stride (int (optional)): Positive stride between samples from the time-series. Defaults to 1.
            
                iterations (int (optional)): Positive maximum number of times to train on the samples. Defaults to 1.
            
                mse_tol (float (optional)): Positive tolerance for the mean-squared error over the training data.
                If the mse after a training session is less than this skip training further. Defaults to `1E-9`.
            
            Returns:
                float (optional): mse. Mean squared error of the predictor relative to the time-series trained on.
            
        """
        ...



class ModelInfo:
    """
    Provides model-information useful for selection and filtering
    """
    @property
    def created(self)->time:
        """
         the time of creation, or last modification of the model
        """
        ...
    @created.setter
    def created(self, value:time)->None:
        ...
    @property
    def id(self)->int:
        """
         the unique model id, can be used to retrieve the real model
        """
        ...
    @id.setter
    def id(self, value:int)->None:
        ...
    @property
    def json(self)->str:
        """
         a json formatted string to enable scripting and python to store more information
        """
        ...
    @json.setter
    def json(self, value:str)->None:
        ...
    @property
    def name(self)->str:
        """
         any useful name or description
        """
        ...
    @name.setter
    def name(self, value:str)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, id: object, name: str, created: Union[time,float,int], json: str = '') -> None:
        """
        Constructs ModelInfo with the supplied parameters
            
            Args:
                id (int): the unique model id
            
                name (str): any useful name or description
            
                created (time): time of creation
            
                json (str): extra information, preferably a valid json
            
        """
        ...



class ModelInfoVector:
    """
    A strongly typed list of ModelInfo
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: ModelInfoVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class Point:
    """
    A timeseries point specifying utctime t and value v
    """
    @property
    def t(self)->time:
        """
         utctime of the point
        """
        ...
    @t.setter
    def t(self, value:time)->None:
        ...
    @property
    def v(self)->float:
        """
         the value of the point
        """
        ...
    @v.setter
    def v(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, t: Union[time,float,int], v: object) -> None:
        ...



class QacParameter:
    """
    The qac parameter controls how quailty checks are done, providing min-max range, plus repeated values checks
    It also provides parameters that controls how the replacement/correction values are filled in,
    like maximum time-span between two valid neighbour points that allows for linear/extension filling
    """
    @property
    def constant_filler(self)->float:
        """
         this is applied to values that fails quality checks, if no correction ts, and no interpolation/extension is active
        """
        ...
    @constant_filler.setter
    def constant_filler(self, value:float)->None:
        ...
    @property
    def max_timespan(self)->time:
        """
         maximum timespan between two ok values that allow interpolation, or extension of values.If zero, no linear/extend correction
        """
        ...
    @max_timespan.setter
    def max_timespan(self, value:time)->None:
        ...
    @property
    def max_v(self)->float:
        """
         maximum value or nan for no maximum value limit
        """
        ...
    @max_v.setter
    def max_v(self, value:float)->None:
        ...
    @property
    def min_v(self)->float:
        """
         minimum value or nan for no minimum value limit
        """
        ...
    @min_v.setter
    def min_v(self, value:float)->None:
        ...
    @property
    def repeat_allowed(self)->bool:
        """
         values that are allowed to repeat, within repeat-tolerance
        """
        ...
    @repeat_allowed.setter
    def repeat_allowed(self, value:bool)->None:
        ...
    @property
    def repeat_timespan(self)->time:
        """
         maximum timespan the same value can be repeated (within repeat_tolerance).If zero, no repeat validation done
        """
        ...
    @repeat_timespan.setter
    def repeat_timespan(self, value:time)->None:
        ...
    @property
    def repeat_tolerance(self)->float:
        """
         values are considered repeated if they differ by less than repeat_tolerance
        """
        ...
    @repeat_tolerance.setter
    def repeat_tolerance(self, value:float)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, max_timespan: Union[time,float,int], min_x: object, max_x: object, repeat_timespan: Union[time,float,int], repeat_tolerance: object, constant_filler: object = nan) -> None:
        """
        a quite complete qac, only lacks repeat_allowed value(s)
            
        
        """
        ...
    @overload
    def __init__(self, max_timespan: Union[time,float,int], min_x: object, max_x: object, repeat_timespan: Union[time,float,int], repeat_tolerance: object, repeat_allowed: object, constant_filler: object = nan) -> None:
        """
        a quite complete qac, including one repeat_allowed value
            
        """
        ...



class QueueMessage:
    """
    A QueueMessage as returned from the DtsClient.q_get(..) consist of the .info part and the payload time-series vector .tsv
    """
    @property
    def info(self)->Any:
        """
        The information about the message
        """
        ...
    @property
    def tsv(self)->Any:
        """
        The time-series vector payload part of the message
        """
        ...
    def __init__(self) -> None:
        ...



class QueueMessageInfo:
    """
    Information about the queue item,
    such as the state of the item, in-queue,fetched, done.
    This element is never to be created by the python user, but is a return type
    from the dtss queue message info related calls.
    """
    @property
    def created(self)->time:
        """
         Time when the message was put into the queue
        """
        ...
    @property
    def description(self)->str:
        """
         A user specified description, we recommend json format
        """
        ...
    @property
    def diagnostics(self)->str:
        """
         Time when the message acknowledged done from the receiver(end-to-end ack)
        """
        ...
    @property
    def done(self)->time:
        """
         Time when the message acknowledged done from the receiver(end-to-end ack)
        """
        ...
    @property
    def fetched(self)->time:
        """
         Time when the message was fetched from the queue
        """
        ...
    @property
    def msg_id(self)->str:
        """
         The unique id for this message in the live-queue
        """
        ...
    @property
    def ttl(self)->time:
        """
         Time to live set for this message, used to prune out old messages
        """
        ...
    def __init__(self) -> None:
        ...



class QueueMessageInfoVector:
    """
    A strongly typed list of QueueMessageInfo, as returned by the dtss queue info calls
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone_me: QueueMessageInfoVector) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class RatingCurveFunction:
    """
    Combine multiple RatingCurveSegments into a rating function.
    
    RatingCurveFunction aggregates multiple RatingCurveSegments and routes.
    computation calls to the correct segment based on the water level to compute for.
    
    See also:
        RatingCurveSegment, RatingCurveParameters
    """
    @overload
    def __init__(self) -> None:
        """
        Defines a new empty rating curve function.
            
        
        """
        ...
    @overload
    def __init__(self, segments: RatingCurveSegments, is_sorted: object = True) -> None:
        """
        constructs a function from a segment-list
            
        """
        ...

    def __iter__(self) -> Any:
        """
        Constant iterator. Invalidated on calls to .add_segment
        """
        ...

    @overload
    def add_segment(self, lower: object, a: object, b: object, c: object) -> None:
        """
        Add a new curve segment with the given parameters.
            
            See also:
                RatingCurveSegment
            
        
        """
        ...
    @overload
    def add_segment(self, segment: RatingCurveSegment) -> None:
        """
        Add a new curve segment as a copy of an exting.
            
            See also:
                RatingCurveSegment
            
        """
        ...

    @overload
    def flow(self, levels: Union[DoubleVector,list[float],np.ndarray]) -> DoubleVector:
        """
        Compute flow for a range of water levels.
            
            Args:
                levels (DoubleVector): Range of water levels to compute flow for.
            
        
        """
        ...
    @overload
    def flow(self, level: object) -> float:
        """
        Compute flow for the given level.
            
            Args:
                level (float): Water level to compute flow for.
            
        """
        ...

    def size(self) -> int:
        """
        Get the number of RatingCurveSegments composing the function.
        """
        ...



class RatingCurveParameters:
    """
    Parameter pack controlling rating level computations.
    
    A parameter pack encapsulates multiple RatingCurveFunction's with time-points.
    When used with a TimeSeries representing level values it maps computations for
    each level value onto the correct RatingCurveFunction, which again maps onto the
    correct RatingCurveSegment for the level value.
    
    See also:
        RatingCurveSegment, RatingCurveFunction, TimeSeries.rating_curve
    """
    @overload
    def __init__(self) -> Any:
        """
        Defines a empty RatingCurveParameter instance
        
        """
        ...
    @overload
    def __init__(self, t_f_list: RatingCurveTimeFunctions) -> Any:
        """
        create parameters in one go from list of RatingCurveTimeFunction elements
            
            Args:
                t_f_list (RatingCurveTimeFunctions): a list of RatingCurveTimeFunction elements
            
        """
        ...

    def __iter__(self) -> Any:
        """
        Constant iterator. Invalidated on calls to .add_curve
        """
        ...

    def add_curve(self, t: Union[time,float,int], curve: RatingCurveFunction) -> RatingCurveParameters:
        """
        Add a curve to the parameter pack.
            
            Args:
                t (RatingCurveFunction): First time-point the curve is valid for.
            
                curve (RatingCurveFunction): RatingCurveFunction to add at t.
            
            Returns:
                RatingCurveParameters: self.  to allow chaining building functions
            
        """
        ...

    @overload
    def flow(self, t: Union[time,float,int], level: object) -> float:
        """
        Compute the flow at a specific time point.
            
            Args:
                t (utctime): Time-point of the level value.
            
                level (float): Level value at t.
            
            Returns:
                float: flow. Flow correcponding to input level at t, `nan` if level is less than the least water level of the first segment or before the time of the first rating curve function.
            
        
        """
        ...
    @overload
    def flow(self, ts: TimeSeries) -> DoubleVector:
        """
        Compute the flow at a specific time point.
            
            Args:
                ts (TimeSeries): Time series of level values.
            
            Returns:
                DoubleVector: flow. Flow correcponding to the input levels of the time-series, `nan` where the level is less than the least water level of the first segment and for time-points before the first rating curve function.
            
        """
        ...



class RatingCurveSegment:
    """
    Represent a single rating-curve equation.
    
    The rating curve function is `a*(h - b)^c` where `a`, `b`, and `c` are parameters
    for the segment and `h` is the water level to compute flow for. Additionally there
    is a `lower` parameter for the least water level the segment is valid for. Seen
    separatly a segment is considered valid for any level greater than `lower`.
    
    The function segments are gathered into many `RatingCurveFunction` to represent a
    set of different rating functions for different levels.
    Related classes are RatingCurveFunction, RatingCurveParameters
    """
    @property
    def a(self)->float:
        """
         Parameter a
        """
        ...
    @a.setter
    def a(self, value:float)->None:
        ...
    @property
    def b(self)->float:
        """
         Parameter b
        """
        ...
    @b.setter
    def b(self, value:float)->None:
        ...
    @property
    def c(self)->float:
        """
         Parameter c
        """
        ...
    @c.setter
    def c(self, value:float)->None:
        ...
    @property
    def lower(self)->float:
        """
        Least valid water level. Not mutable after constructing a segment.
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, lower: object, a: object, b: object, c: object) -> None:
        """
        Defines a new RatingCurveSegment with the specified parameters
        """
        ...

    @overload
    def flow(self, levels: Union[DoubleVector,list[float],np.ndarray], i0: object = 0, iN: object = npos) -> DoubleVector:
        """
        Compute the flow for a range of water levels
            
            Args:
                levels (DoubleVector): Vector of water levels
            
                i0 (int): first index to use from levels, defaults to 0
            
                iN (int): first index _not_ to use from levels, defaults to std::size_t maximum.
            
            Returns:
                DoubleVector: flow. Vector of flow values.
            
        
        """
        ...
    @overload
    def flow(self, level: object) -> float:
        """
        Compute the flow for the given water level.
            
            Notes:
                There is _no_ check to see if level is valid. It's up to the user to call
            
                with a correct level.
            
            Args:
                level (float): water level
            
            Returns:
                double: flow. the flow for the given water level
            
        """
        ...

    def valid(self, level: object) -> bool:
        """
        Check if a water level is valid for the curve segment
            
                level (float): water level
            
            Returns:
                bool: valid. True if level is greater or equal to lower
            
        """
        ...



class RatingCurveSegments:
    """
    A typed list of RatingCurveSegment, used to construct RatingCurveParameters.
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone_me: RatingCurveSegments) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class RatingCurveTimeFunction:
    """
    Composed of time t and RatingCurveFunction
    """
    @property
    def f(self)->RatingCurveFunction:
        """
         the rating curve function
        """
        ...
    @f.setter
    def f(self, value:RatingCurveFunction)->None:
        ...
    @property
    def t(self)->time:
        """
         .f is valid from t, the epoch 1970[s] time 
        """
        ...
    @t.setter
    def t(self, value:time)->None:
        ...
    @overload
    def __init__(self) -> None:
        """
        Defines empty pair t,f
            
        
        """
        ...
    @overload
    def __init__(self, t: Union[time,float,int], f: RatingCurveFunction) -> None:
        """
        Construct an object with function f valid from time t
            
            Args:
                t (int): epoch time in 1970 utc [s]
            
                f (RatingCurveFunction): the function
            
        """
        ...



class RatingCurveTimeFunctions:
    """
    A typed list of RatingCurveTimeFunction elements
    """
    @overload
    def __init__(self) -> None:
        """
        Defines empty list pair t,f
            
        
        """
        ...
    @overload
    def __init__(self, clone_me: RatingCurveTimeFunctions) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class StorePolicy:
    """
    Determine how DTSS stores time-series, used in context of the DtsClient.
    """
    @property
    def cache(self)->bool:
        """
         update cache with new values
        """
        ...
    @cache.setter
    def cache(self, value:bool)->None:
        ...
    @property
    def recreate(self)->bool:
        """
         recreate time-series if it existed, with entirely new definition
        """
        ...
    @recreate.setter
    def recreate(self, value:bool)->None:
        ...
    @property
    def strict(self)->bool:
        """
         use strict requirement for aligment of timepoints.
        If True(default), then require perfect matching time-axis on the 
        incoming ts fragment, and transfer those points to the target.
        Notice that if the target is a break-point/flexible interval time-series,
        then all time-axis passed are a perfect match.
        If False, use a functional mapping approach, resampling or averaging
        the passed time-series fragment to align with the target time-series.
        If target is linear time-series, the new ts fragment is evaluated f(t) for the
        covered time-points in the target time-series.
        If target is stair-case time-series, then the true average of the new ts fragment,
        evaluated over the covering/touched period of the target time-series.
        """
        ...
    @strict.setter
    def strict(self, value:bool)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, recreate: object, strict: object, cache: object) -> None:
        """
        construct object with specified parameters
            
        """
        ...



class StringVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: Union[StringVector,list[str]]) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class TimeAxis:
    """
    A time-axis is a set of ordered non-overlapping periods,
    and TimeAxis provides the most generic implementation of this.
    The internal representation is selected based on provided parameters
    to the constructor.
    The internal representation is one of TimeAxis FixedDeltaT CalendarDelataT or ByPoints.
    The internal representation type and corresponding realizations are available as properties.
    
    Notes:
        The internal representation can be one of TimeAxisCalendarDeltaT,TimeAxisFixedDeltaT,TimeAxisByPoints
    """
    @property
    def calendar_dt(self)->TimeAxisCalendarDeltaT:
        """
         The calendar dt representation(if active)
        """
        ...
    @property
    def fixed_dt(self)->TimeAxisFixedDeltaT:
        """
         The fixed dt representation (if active)
        """
        ...
    @property
    def point_dt(self)->TimeAxisByPoints:
        """
          point_dt representation(if active)
        """
        ...
    @property
    def timeaxis_type(self)->TimeAxisType:
        """
         describes what time-axis representation type this is,e.g (fixed|calendar|point)_dt 
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, start: Union[time,float,int], delta_t: Union[time,float,int], n: object) -> None:
        """
        creates a time-axis with n intervals, fixed delta_t, starting at start
            
            Args:
                start (utctime): utc-time 1970 utc based
            
                delta_t (utctime): number of seconds delta-t, length of periods in the time-axis
            
                n (int): number of periods in the time-axis
            
        
        """
        ...
    @overload
    def __init__(self, calendar: Calendar, start: Union[time,float,int], delta_t: Union[time,float,int], n: object) -> None:
        """
        creates a calendar time-axis
            
            Args:
                calendar (Calendar): specifies the calendar to be used, keeps the time-zone and dst-arithmetic rules
            
                start (utctime): utc-time 1970 utc based
            
                delta_t (utctime): number of seconds delta-t, length of periods in the time-axis
            
                n (int): number of periods in the time-axis
            
        
        """
        ...
    @overload
    def __init__(self, time_points: Union[UtcTimeVector,list[time],list[float],np.ndarray], t_end: Union[time,float,int]) -> None:
        """
        creates a time-axis by specifying the time_points and t-end of the last interval
            
            Args:
                time_points (UtcTimeVector): ordered set of unique utc-time points, the start of each consecutive period
            
                t_end (time): the end of the last period in time-axis, utc-time 1970 utc based, must be > time_points[-1]
            
        
        """
        ...
    @overload
    def __init__(self, time_points: Union[UtcTimeVector,list[time],list[float],np.ndarray]) -> None:
        """
        create a time-axis supplying n+1 points to define n intervals
            
            Args:
                time_points (UtcTimeVector): ordered set of unique utc-time points, 0..n-2:the start of each consecutive period,n-1: end of last period
            
        
        """
        ...
    @overload
    def __init__(self, calendar_dt: TimeAxisCalendarDeltaT) -> None:
        """
        create a time-axis from a calendar time-axis
            
            Args:
                calendar_dt (TimeAxisCalendarDeltaT): existing calendar time-axis
            
        
        """
        ...
    @overload
    def __init__(self, fixed_dt: TimeAxisFixedDeltaT) -> None:
        """
        create a time-axis from a a fixed delta-t time-axis
            
            Args:
                fixed_dt (TimeAxisFixedDeltaT): existing fixed delta-t time-axis
            
        
        """
        ...
    @overload
    def __init__(self, point_dt: TimeAxisByPoints) -> None:
        """
        create a time-axis from a a by points  time-axis
            
            Args:
                point_dt (TimeAxisByPoints): existing by points time-axis
            
        """
        ...

    def __call__(self, i: object) -> UtcPeriod:
        """
        Returns the i-th period of the time-axis
            
            Args:
                i (int): index to lookup
            
            Returns:
                UtcPeriod: period. The period for the supplied index
            
        """
        ...

    def __len__(self) -> int:
        """
        Returns the number of periods in the time-axis
            
        """
        ...

    def empty(self) -> bool:
        """
        true if empty time-axis
            
            Returns:
                bool: empty. true if empty time-axis
            
        """
        ...

    def index_of(self, t: Union[time,float,int], ix_hint: object = npos) -> int:
        """
        
            Args:
                t (int): utctime in seconds 1970.01.01
            
                ix_hint (int): index-hint to make search in point-time-axis faster
            
            Returns:
                int: index. the index of the time-axis period that contains t, npos if outside range
            
        """
        ...

    def merge(self, other: TimeAxis) -> TimeAxis:
        """
        Returns a new time-axis that contains the union of time-points/periods of the two time-axis.
            If there is a gap between, it is filled
            merge with empty time-axis results into the other time-axis
            
            Args:
                other (TimeAxis): The other time-axis to merge with
            
            Returns:
                TimeAxis: merge_result. the resulting merged time-axis
            
        """
        ...

    def open_range_index_of(self, t: Union[time,float,int], ix_hint: object = npos) -> int:
        """
        returns the index that contains t, or is before t
            
            Args:
                t (int): utctime in seconds 1970.01.01
            
                ix_hint (int): index-hint to make search in point-time-axis faster
            
            Returns:
                int: index. the index the time-axis period that contains t, npos if before first period n-1, if t is after last period
            
        """
        ...

    def period(self, i: object) -> UtcPeriod:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                UtcPeriod: period. the i'th period of the time-axis
            
        """
        ...

    def size(self) -> int:
        """
        
            Returns:
                int: n. number of periods in time-axis
            
        """
        ...

    def slice(self, start: object, n: object) -> TimeAxis:
        """
        returns slice of time-axis as a new time-axis
            
            Args:
                start (int): first interval to include
            
                n (int): number of intervals to include
            
            Returns:
                TimeAxis: time-axis. A new time-axis with the specified slice
            
        """
        ...

    def time(self, i: object) -> time:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                int: utctime. the start(utctime) of the i'th period of the time-axis
            
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        
            Returns:
                UtcPeriod: total_period. the period that covers the entire time-axis
            
        """
        ...



class TimeAxisByPoints:
    """
    A time-axis is a set of ordered non-overlapping periods,
    and this class implements this by a set of 
    ordered unique time-points. This is the most flexible time-axis representation,
    that allows every period in the time-axis to have different length.
    It comes at the cost of space&performance in certain cases, so 
    avoid use in scenarios where high-performance is important.
    
    See also:
        TimeAxisCalendarDeltaT,TimeAxisFixedDeltaT,TimeAxis
    """
    @property
    def t(self)->UtcTimeVector:
        """
         time_points except end of last period, see t_end
        """
        ...
    @property
    def t_end(self)->time:
        """
         end of time-axis
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, time_points: Union[UtcTimeVector,list[time],list[float],np.ndarray], t_end: Union[time,float,int]) -> None:
        """
        creates a time-axis by specifying the time_points and t-end of the last interval
            
            Args:
                time_points (UtcTimeVector): ordered set of unique utc-time points, the start of each consecutive period
            
                t_end (int): the end of the last period in time-axis, utc-time 1970 utc based, must be > time_points[-1]
            
        
        """
        ...
    @overload
    def __init__(self, time_points: Union[UtcTimeVector,list[time],list[float],np.ndarray]) -> None:
        """
        create a time-axis supplying n+1 points to define n intervals
            
            Args:
                time_points (UtcTimeVector): ordered set of unique utc-time points, 0..n-2:the start of each consecutive period,n-1: end of last period
            
        """
        ...

    def empty(self) -> bool:
        """
        true if empty time-axis
            
            Returns:
                bool: empty. true if empty time-axis
            
        """
        ...

    def index_of(self, t: Union[time,float,int]) -> int:
        """
        
            Args:
                t (utcime): utctime in seconds 1970.01.01
            
            Returns:
                int: index. the index of the time-axis period that contains t, npos if outside range
            
        """
        ...

    def open_range_index_of(self, t: Union[time,float,int], ix_hint: object = npos) -> int:
        """
        returns the index that contains t, or is before t
            
            Args:
                t (int): utctime in seconds 1970.01.01
            
                ix_hint (int): index-hint to make search in point-time-axis faster
            
            Returns:
                int: index. the index the time-axis period that contains t, npos if before first period n-1, if t is after last period
            
        """
        ...

    def period(self, i: object) -> UtcPeriod:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                UtcPeriod: period. the i'th period of the time-axis
            
        """
        ...

    def size(self) -> int:
        """
        
            Returns:
                int: n. number of periods in time-axis
            
        """
        ...

    def slice(self, start: object, n: object) -> TimeAxisByPoints:
        """
        returns slice of time-axis as a new time-axis
            
            Args:
                start (int): first interval to include
            
                n (int): number of intervals to include
            
            Returns:
                TimeAxis: time-axis. A new time-axis with the specified slice
            
        """
        ...

    def time(self, i: object) -> time:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                int: utctime. the start(utctime) of the i'th period of the time-axis
            
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        
            Returns:
                UtcPeriod: total_period. the period that covers the entire time-axis
            
        """
        ...



class TimeAxisCalendarDeltaT:
    """
    A time-axis is a set of ordered non-overlapping periods,
    and this class implements a calendar-unit fixed delta-t time-axis by
    specifying the minimal calendar, t-start, delta-t and number of consecutive periods.
    This class is particularly useful if you need to work with calendar and daylight-saving time,
    or calendar-specific periods like day,week,month,quarters or years.
    
    
    
    Notes:
        For performance reasons, dt < DAY results in a TimeAxisFixedDeltaT.
    
    See also:
        TimeAxisFixedDeltaT,TimeAxisByPoints,TimeAxis
    """
    @property
    def calendar(self)->Calendar:
        """
         the calendar of the time-axis
        """
        ...
    @property
    def delta_t(self)->time:
        """
         timespan of each interval,use Calendar.DAY|.WEEK|.MONTH|.QUARTER|.YEAR, or seconds
        """
        ...
    @property
    def n(self)->int:
        """
         number of periods
        """
        ...
    @property
    def start(self)->time:
        """
         start of the time-axis, in seconds since 1970.01.01 UTC
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, calendar: Calendar, start: Union[time,float,int], delta_t: Union[time,float,int], n: object) -> None:
        """
        creates a calendar time-axis
            
            Args:
                calendar (Calendar): specifies the calendar to be used, keeps the time-zone and dst-arithmetic rules
            
                start (int): utc-time 1970 utc based
            
                delta_t (int): number of seconds delta-t, length of periods in the time-axis.For performance reasons, dt < DAY results in a TimeAxisFixedDeltaT.
            
                n (int): number of periods in the time-axis
            
        """
        ...

    def empty(self) -> bool:
        """
        true if empty time-axis
            
            Returns:
                bool: empty. true if empty time-axis
            
        """
        ...

    def index_of(self, t: Union[time,float,int]) -> int:
        """
        
            Args:
                t (utcime): utctime in seconds 1970.01.01
            
            Returns:
                int: index. the index of the time-axis period that contains t, npos if outside range
            
        """
        ...

    def open_range_index_of(self, t: Union[time,float,int], ix_hint: object = npos) -> int:
        """
        returns the index that contains t, or is before t
            
            Args:
                t (int): utctime in seconds 1970.01.01
            
                ix_hint (int): index-hint to make search in point-time-axis faster
            
            Returns:
                int: index. the index the time-axis period that contains t, npos if before first period n-1, if t is after last period
            
        """
        ...

    def period(self, i: object) -> UtcPeriod:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                UtcPeriod: period. the i'th period of the time-axis
            
        """
        ...

    def size(self) -> int:
        """
        
            Returns:
                int: n. number of periods in time-axis
            
        """
        ...

    def slice(self, start: object, n: object) -> TimeAxisCalendarDeltaT:
        """
        returns slice of time-axis as a new time-axis
            
            Args:
                start (int): first interval to include
            
                n (int): number of intervals to include
            
            Returns:
                TimeAxis: time-axis. A new time-axis with the specified slice
            
        """
        ...

    def time(self, i: object) -> time:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                int: utctime. the start(utctime) of the i'th period of the time-axis
            
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        
            Returns:
                UtcPeriod: total_period. the period that covers the entire time-axis
            
        """
        ...



class TimeAxisFixedDeltaT:
    """
    A time-axis is a set of ordered non-overlapping periods,
    and this class implements a fixed delta-t time-axis by
    specifying the minimal t-start, delta-t and number of consecutive periods.
    This class is a wrapper of the Shyft core-library high performance time-axis
    
    See also:
        TimeAxisCalendarDeltaT,TimeAxisByPoints,TimeAxis
    """
    @property
    def delta_t(self)->time:
        """
         time-span of each interval in seconds
        """
        ...
    @property
    def n(self)->int:
        """
         number of periods
        """
        ...
    @property
    def start(self)->time:
        """
         start of the time-axis, in seconds since 1970.01.01 UTC
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, start: Union[time,float,int], delta_t: Union[time,float,int], n: object) -> None:
        """
        creates a time-axis with n intervals, fixed delta_t, starting at start
            
            Args:
                start (int): utc-time 1970 utc based
            
                delta_t (int): number of seconds delta-t, length of periods in the time-axis
            
                n (int): number of periods in the time-axis
            
        """
        ...

    def empty(self) -> bool:
        """
        true if empty time-axis
            
            Returns:
                bool: empty. true if empty time-axis
            
        """
        ...

    @staticmethod
    def full_range() -> TimeAxisFixedDeltaT:
        """
        returns a timeaxis that covers [-oo..+oo> 
        """
        ...

    def index_of(self, t: Union[time,float,int]) -> int:
        """
        
            Args:
                t (utcime): utctime in seconds 1970.01.01
            
            Returns:
                int: index. the index of the time-axis period that contains t, npos if outside range
            
        """
        ...

    @staticmethod
    def null_range() -> TimeAxisFixedDeltaT:
        """
        returns a null timeaxis
        """
        ...

    def open_range_index_of(self, t: Union[time,float,int], ix_hint: object = npos) -> int:
        """
        returns the index that contains t, or is before t
            
            Args:
                t (int): utctime in seconds 1970.01.01
            
                ix_hint (int): index-hint to make search in point-time-axis faster
            
            Returns:
                int: index. the index the time-axis period that contains t, npos if before first period n-1, if t is after last period
            
        """
        ...

    def period(self, i: object) -> UtcPeriod:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                UtcPeriod: period. the i'th period of the time-axis
            
        """
        ...

    def size(self) -> int:
        """
        
            Returns:
                int: n. number of periods in time-axis
            
        """
        ...

    def slice(self, start: object, n: object) -> TimeAxisFixedDeltaT:
        """
        returns slice of time-axis as a new time-axis
            
            Args:
                start (int): first interval to include
            
                n (int): number of intervals to include
            
            Returns:
                TimeAxis: time-axis. A new time-axis with the specified slice
            
        """
        ...

    def time(self, i: object) -> time:
        """
        
            Args:
                i (int): the i'th period, 0..n-1
            
            Returns:
                int: utctime. the start(utctime) of the i'th period of the time-axis
            
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        
            Returns:
                UtcPeriod: total_period. the period that covers the entire time-axis
            
        """
        ...



class TimeSeries:
    """
    A time-series providing mathematical and statistical operations and functionality.
    
    A time-series can be an expression, or a concrete point time-series.
    All time-series do have a time-axis, values, and a point fx policy.
    The value, f(t) outside the time-axis is nan
    Operations between time-series, e.g. `a+b`, respects the mathematical `nan op something equals nan`
    
    The time-series can provide a value for all the intervals, and the point_fx policy
    defines how the values should be interpreted:
    
    POINT_INSTANT_VALUE(linear):
    
        the point value is valid at the start of the period, linear between points
        -or flat extended value if the next point is nan.
        typical for state-variables, like water-level, temperature measured at 12:00 etc.
    
    POINT_AVERAGE_VALUE(stair-case):
    
        the point represents an average or constant value over the period
        typical for model-input and results, precipitation mm/h, discharge m^3/s
    
    
    Examples:
    
    >>> import numpy as np
    >>> from shyft.time_series import Calendar,deltahours,TimeAxis,TimeSeries,POINT_AVERAGE_VALUE as fx_avg
    >>>
    >>> utc = Calendar()  # ensure easy consistent explicit handling of calendar and time
    >>> ta = TimeAxis(utc.time(2016, 9, 1, 8, 0, 0), deltahours(1), 10)  # create a time-axis to use
    >>> a = TimeSeries(ta, np.linspace(0, 10, num=len(ta)), fx_avg)
    >>> b = TimeSeries(ta, np.linspace(0,  1, num=len(ta)), fx_avg)
    >>> c = a + b*3.0  # c is now an expression, time-axis is the overlap of a and b, lazy evaluation
    >>> c_values = c.values.to_numpy()  # compute and extract the values, as numpy array
    >>> c_evaluated=c.evaluate() # computes the expression, return a new concrete point-ts equal to the expression
    >>>
    >>> # Calculate data for new time-points
    >>> value_1 = a(utc.time(2016, 9, 1, 8, 30)) # calculates value at a given time
    >>> ta_target = TimeAxis(utc.time(2016, 9, 1, 7, 30), deltahours(1), 12)  # create a target time_axis
    >>> ts_new = a.average(ta_target) # new time-series with values on ta_target
    >>>
    
    
    TimeSeries can also be symbolic, that is, have urls, that is resolved later, serverside using the  :class:`DtsServer` 
    The TimeSeries functionality includes:
    
    
        * **construction**: TimeSeries(time-axis,values,point_interpretation), TimeSeries(ts_url), TimeSeries(ts_url,ts_fragment)  :func:`create_from_pattern` 
        * **mutating points**: :meth:`set`  :meth:`fill`  :meth:`scale_by`  :meth:`merge_points` 
        * **combining/extending**: :meth:`extend`  :meth:`use_time_axis_from` 
        * **resampling**: :meth:`average`  :meth:`accumulate`  :meth:`time_shift`  :meth:`use_time_axis_from`  :meth:`use_time_axis` 
        * **f(x)**: :meth:`transform`  :meth:`abs`  :meth:`derivative`  :meth:`integral`  :meth:`min`  :meth:`max`  :meth:`pow`  :meth:`log` 
        * **boolean f(x)**: :meth:`inside` , creates mask(1.0,0.0 series) you can use for math/filtering expressions
        * **statistics**: :meth:`statistics`  :meth:`kling_gupta`  :meth:`nash_sutcliffe` 
        * **filtering**: :meth:`convolve_w`  :meth:`krls_interpolation` 
        * **quality and correction**: :meth:`quality_and_self_correction`  :meth:`quality_and_ts_correction` , min-max limits, replace by surrounding points or replacement ts
        * **bit-encoded**: :meth:`decode` 
        * **stacking and percentiles**: :meth:`stack` 
        * **n-ary operations**: :meth:`TsVector.sum`  :meth:`TsVector.forecast_merge` , operations on TsVector that results in a TimeSeries
        * **hydrology domain**: :meth:`rating_curve`  :meth:`bucket_to_hourly`  :meth:`ice_packing`  :meth:`ice_packing_recession`  :func:`shyft.time_series.create_glacier_melt_ts_m3s` 
    
    Other useful classes to look at: :class:`TimeAxis` , :class:`Calendar` , :class:`TsVector` , :class:`point_interpretation_policy` 
    
    
    
    Please check the extensive test suite, notebooks, examples and time_series for usage.
    """
    @property
    def values(self)->DoubleVector:
        """
         the values values (possibly calculated on the fly)
        """
        ...
    @overload
    def __init__(self) -> None:
        """
        constructs and empty time-series
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxis, values: Union[DoubleVector,list[float],np.ndarray], point_fx: point_interpretation_policy) -> None:
        """
        construct a timeseries time-axis ta, corresponding values and point interpretation policy point_fx
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxis, fill_value: object, point_fx: point_interpretation_policy) -> None:
        """
        construct a time-series with time-axis ta, specified fill-value, and point interpretation policy point_fx
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxisFixedDeltaT, values: Union[DoubleVector,list[float],np.ndarray], point_fx: point_interpretation_policy) -> None:
        """
        construct a timeseries timeaxis ta with corresponding values, and point interpretation policy point_fx
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxisFixedDeltaT, fill_value: object, point_fx: point_interpretation_policy) -> None:
        """
        construct a timeseries with fixed-delta-t time-axis ta, specified fill-value, and point interpretation policy point_fx
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxisByPoints, values: Union[DoubleVector,list[float],np.ndarray], point_fx: point_interpretation_policy) -> None:
        """
        construct a time-series with a point-type time-axis ta, corresponding values, and point-interpretation point_fx
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxisByPoints, fill_value: object, point_fx: point_interpretation_policy) -> None:
        """
        construct a time-series with a point-type time-axis ta, specified fill-value, and point-interpretation point_fx
            
        
        """
        ...
    @overload
    def __init__(self, core_result_ts: TsFixed) -> None:
        """
        construct a time-series from a shyft core time-series, to ease working with core-time-series in user-interface/scripting
            
        
        """
        ...
    @overload
    def __init__(self, clone: TimeSeries) -> None:
        """
        creates a shallow copy of the clone time-series
            
        
        """
        ...
    @overload
    def __init__(self, pattern: Union[DoubleVector,list[float],np.ndarray], dt: Union[time,float,int], ta: TimeAxis) -> None:
        """
        construct a repeated pattern time-series given a equally spaced dt pattern and a time-axis ta
            
            Args:
                pattern (DoubleVector): a list of numbers giving the pattern
            
                dt (int): number of seconds between each of the pattern-values
            
                ta (TimeAxis): time-axis that forms the resulting time-series time-axis
            
        
        """
        ...
    @overload
    def __init__(self, pattern: Union[DoubleVector,list[float],np.ndarray], dt: Union[time,float,int], t0: Union[time,float,int], ta: TimeAxis) -> None:
        """
        construct a time-series given a equally spaced dt pattern, starting at t0, and a time-axis ta
            
        
        """
        ...
    @overload
    def __init__(self, ts_id: str) -> None:
        """
        constructs a bind-able ts,
            providing a symbolic possibly unique id that at a later time
            can be bound, using the .bind(ts) method to concrete values
            if the ts is used as ts, like size(),.value(),time() before it
            is bound, then a runtime-exception is raised
            
            Args:
                ts_id (str): url-like identifier for the time-series,notice that shyft://<container>/<path> is for shyft-internal store
            
        
        """
        ...
    @overload
    def __init__(self, ts_id: str, bts: TimeSeries) -> None:
        """
        constructs a ready bound ts,
            providing a symbolic possibly unique id that at a later time
            can be used to correlate with back-end store
            
            
            Args:
                ts_id (str): url-type of id, notice that shyft://<container>/<path> is for shyft-internal store
            
                bts (TimeSeries): A time-series, that is either a concrete ts, or an expression that can be evaluated to form a concrete ts
            
        """
        ...

    @overload
    def __add__(self, arg2: TimeSeries) -> TimeSeries:
        ...
    @overload
    def __add__(self, arg2: object) -> TimeSeries:
        ...

    def __call__(self, t: Union[time,float,int]) -> float:
        """
        return the f(t) value for the time-series
        """
        ...

    @overload
    def __mul__(self, arg2: TimeSeries) -> TimeSeries:
        ...
    @overload
    def __mul__(self, arg2: object) -> TimeSeries:
        ...

    def __neg__(self) -> TimeSeries:
        ...

    def __radd__(self, arg2: object) -> TimeSeries:
        ...

    def __rmul__(self, arg2: object) -> TimeSeries:
        ...

    def __rsub__(self, arg2: object) -> TimeSeries:
        ...

    def __rtruediv__(self, arg2: object) -> TimeSeries:
        ...

    @overload
    def __sub__(self, arg2: TimeSeries) -> TimeSeries:
        ...
    @overload
    def __sub__(self, arg2: object) -> TimeSeries:
        ...

    @overload
    def __truediv__(self, arg2: TimeSeries) -> TimeSeries:
        ...
    @overload
    def __truediv__(self, arg2: object) -> TimeSeries:
        ...

    def abs(self) -> TimeSeries:
        """
        create a new ts, abs(py::self
            
            Returns:
                TimeSeries: ts. a new time-series expression, that will provide the abs-values of self.values
            
        """
        ...

    def accumulate(self, ta: TimeAxis) -> TimeSeries:
        """
        create a new ts where each i-th value ::
            | integral f(t)*dt, from t0..ti
            
            given the specified time-axis ta, and the point interpretation.
            
            Args:
                ta (TimeAxis): time-axis that specifies the periods where accumulated integral is applied
            
            Returns:
                TimeSeries: ts. a new time-series expression, that will provide the accumulated values when requested
            
            Notes:
            In contrast to  :meth:`integral` , `accumulate` has a point-instant interpretation. As  :meth:`values` gives the start values of each interval, see  :class:`TimeSeries` , `accumulate(ta).values` provides the accumulation over the intervals `[t0..t0, t0..t1, t0..t2, ...]`, thus `values[0]` is always 0.)
            
        """
        ...

    def average(self, ta: TimeAxis) -> TimeSeries:
        """
        create a new ts that is the true average of self
            over the specified time-axis ta.
            Notice that same definition as for integral applies; non-nan parts goes into the average
            
            Args:
                ta (TimeAxis): time-axis that specifies the periods where true-average is applied
            
            Returns:
                TimeSeries: ts. a new time-series expression, that will provide the true-average when requested
            
            Notes:
                the self point interpretation policy is used when calculating the true average
            
        """
        ...

    def bind(self, bts: TimeSeries) -> None:
        """
        given that this ts,self, is a bind-able ts (aref_ts)
            and that bts is a concrete point TimeSeries, or something that can be evaluated to one,
            use it as representation
            for the values of this ts.
            Other related functions are find_ts_bind_info,TimeSeries('a-ref-string')
            
            Args:
                bts (TimeSeries): a concrete point ts, or ready-to-evaluate expression, with time-axis, values and fx_policy
            
            Notes:
                raises runtime_error if any of preconditions is not true
            
        """
        ...

    def bind_done(self, skip_check: object = False) -> None:
        """
        after bind operations on unbound time-series of an expression is done, call bind_done()
            to prepare the expression for use
            Other related methods are .bind(), .find_ts_bind_info() and needs_bind().
            
            Args:
                skip_check (bool): If set true this function assumes all siblings are bound, as pr. standard usage pattern for the mentioned functions
            
            Notes:
                Usually this is done automatically by the dtss framework, but if not using dtss
            
                this function is needed *after* the symbolic ts's are bound
            
            
        """
        ...

    def bucket_to_hourly(self, start_hour_utc: object, bucket_emptying_limit: object) -> TimeSeries:
        """
        Precipitation bucket measurements have a lot of tweaks that needs to be resolved,
            including negative variations over the day due to faulty temperature-dependent
            volume/weight sensors attached.
            
            A precipitation bucket accumulates precipitation, so the readings should be strictly
            increasing by time, until the bucket is emptied (full, or as part of maintenance).
            
            The goal for the bucket_to_hourly algorithm is to provide *hourly* precipitation, based on some input signal
            that usually is hourly(averaging is used if not hourly).
            
            The main strategy is to use 24 hour differences (typically at hours in a day where the
            temperature is low, like early in the morning.), to adjust the hourly volume.
            
            Differences in periods of 24hour are distributed on all positive hourly evenets, the 
            negative derivatives are zeroed out, so that the hourly result for each 24 hour
            is steady increasing, and equal to the difference of the 24hour area.
            
            The derivative is then used to compute the hourly precipitation rate in mm/h
            
            
            Args:
                start_hour_utc (int): valid range [0..24], usually set to early morning(low-stable temperature)
            
                bucket_emptying_limit (float): a negative number, range[-oo..0>, limit of when to detect an emptying of a bucket in the unit of the measurements series
            
            Returns:
                TimeSeries: ts. a new hourly rate ts, that transforms the accumulated series, compensated for the described defects
            
        """
        ...

    def clone_expression(self) -> TimeSeries:
        """
        create a copy of the ts-expressions, except for the bound payload of the reference ts.
            For the reference terminals, those with `ts_id`, only the `ts_id` is copied.
            Thus, to re-evaluate the expression, those have to be bound.
            
            Notes:
                this function is only useful in context where multiple bind/rebind while keeping the expression is needed.
            
            Returns:
                TsVector: semantic_clone. returns a copy of the ts, except for the payload at reference/symbolic terminals, where only `ts_id`is copied
            
        """
        ...

    def compress(self, accuracy: object) -> TimeSeries:
        """
        Compress by reducing number of points sufficient to represent the same f(t) within accuracy.
            The returned ts is a new ts with break-point/variable interval representation.
            note: lazy binding expressions(server-side eval) is not yet supported.
            
            Args:
                accuracy (): if v[i]-v[i+1] <accuracy the v[i+1] is dropped
            
            Returns:
                TimeSeries: compressed_ts. a new compressed within accuracy time-series
            
        """
        ...

    def compress_size(self, accuracy: object) -> int:
        """
        Compute number of points this time-series could be reduced to if calling ts.compress(accuracy).
            note: lazy binding expressions(server-side eval) is not yet supported.
            
            Args:
                accuracy (): if v[i]-v[i+1] <accuracy the v[i+1] is dropped
            
            Returns:
                int: compressed_size. number of distinct point needed to represent the time-series
            
        """
        ...

    def convolve_w(self, weights: Union[DoubleVector,list[float],np.ndarray], policy: convolve_policy) -> TimeSeries:
        """
        create a new ts that is the convolved ts with the given weights list
            
            Args:
                weights (DoubleVector): the weights profile, use DoubleVector.from_numpy(...) to create these. It's the callers responsibility to ensure the sum of weights are 1.0
            
                policy (convolve_policy): (USE_NEAREST|USE_ZERO|USE_NAN + BACKWARD|FORWARD|CENTER). Specifies how to handle boundary values
            
            Returns:
                TimeSeries: ts. a new time-series that is evaluated on request to the convolution of self
            
        """
        ...

    def decode(self, start_bit: object, n_bits: object) -> TimeSeries:
        """
        Create an time-series that decodes the source using provided
            specification start_bit and n_bits.
            This function can typically be used to decode status-signals from sensors stored as 
            binary encoded bits, using integer representation
            The floating point format allows up to 52 bits to be precisely stored as integer
            - thus there are restrictions to start_bit and n_bits accordingly.
            Practical sensors quality signals have like 32 bits of status information encoded
            If the value in source time-series is:
            
                 * negative
                 * nan
                 * larger than 52 bits
            
            Then nan is returned for those values
            
            ts.decode(start_bit=1,n_bits=1) will return values [0,1,nan]
            similar:
            ts.decode(start_bit=1,n_bits=2) will return values [0,1,2,3,nan]
            etc..
            
            
            Args:
                start_bit (int): where in the n-bits integer the value is stored, range[0..51]
            
                n_bits (int): how many bits are encoded, range[0..51], but start_bit +n_bits < 51
            
            Returns:
                TimeSeries: decode_ts. Evaluated on demand decoded time-series
            
        """
        ...

    def derivative(self, method: derivative_method = derivative_method.DEFAULT) -> TimeSeries:
        """
        Compute the derivative of the ts, according to the method specified.
            For linear(POINT_INSTANT_VALUE), it is always the derivative of the straight line between points,
            - using nan for the interval starting at the last point until end of time-axis.
            Default for stair-case(POINT_AVERAGE_VALUE) is the average derivative over each time-step,
            - using 0 as rise for the first/last half of the intervals at the boundaries.
            here you can influence the method used, selecting .forward_diff, .backward_diff
            
            Args:
                method (derivative_method): default value gives center/average derivative .(DEFAULT|FORWARD|BACKWARD|CENTER)
            
            Returns:
                TimeSeries: derivative. The derivative ts
            
        """
        ...

    @staticmethod
    def deserialize(blob: ByteVector) -> TimeSeries:
        """
        convert a blob, as returned by .serialize() into a Timeseries
        """
        ...

    def evaluate(self) -> TimeSeries:
        """
        Forces evaluation of the expression, returns a new concrete time-series
            that is detached from the expression.
            
            Returns:
                TimeSeries: ts. the evaluated copy of the expression that self represents
            
        """
        ...

    def extend(self, ts: TimeSeries, split_policy: extend_split_policy = extend_split_policy.LHS_LAST, fill_policy: extend_fill_policy = extend_fill_policy.FILL_NAN, split_at: Union[time,float,int] = "time(0)", fill_value: object = nan) -> TimeSeries:
        """
        create a new time-series that is self extended with ts
            
            Args:
                ts (TimeSeries): time-series to extend self with, only values after both the start of self, and split_at is used
            
                split_policy (extend_split_policy): policy determining where to split between self and ts
            
                fill_policy (extend_fill_policy): policy determining how to fill any gap between self and ts
            
                split_at (utctime): time at which to split if split_policy == EPS_VALUE
            
                fill_value (float): value to fill any gap with if fill_policy == EPF_FILL
            
            Returns:
                TimeSeries: extended_ts. a new time-series that is the extension of self with ts
            
        """
        ...

    def fill(self, v: object) -> None:
        """
        fill all values with v
        """
        ...

    def find_ts_bind_info(self) -> TsBindInfoVector:
        """
        recursive search through the expression that this ts represents,
            and return a list of TsBindInfo that can be used to
            inspect and possibly 'bind' to ts-values.
            see also related function bind()
            
            Returns:
                TsBindInfoVector: bind_info. A list of BindInfo where each entry contains a symbolic-ref and a ts that needs binding
            
        """
        ...

    def get(self, i: object) -> Point:
        """
        returns i'th point(t,v)
        """
        ...

    def get_krls_predictor(self, dt: Union[time,float,int], gamma: object = 0.001, tolerance: object = 0.01, size: object = 1000000) -> KrlsRbfPredictor:
        """
        Get a KRLS predictor trained on this time-series.
            
            If you only want a interpolation of self use krls_interpolation instead, this method
            return the underlying predictor instance that can be used to generate mean-squared error
            estimates, or can be further trained on more data.
            
            Notes:
                A predictor can only be generated for a bound time-series.
            
            Args:
                dt (float): The time-step in seconds the underlying predictor is specified for. Note that this does not put a limit on time-axes used, but for best results it should be approximatly equal to the time-step of time-axes used with the predictor. In addition it should not be to long, else you will get poor results. Try to keep the dt less than a day, 3-8 hours is usually fine.
            
                gamma (float (optional)): Determines the width of the radial basis functions for the KRLS algorithm. Lower values mean wider basis functions, wider basis functions means faster computation but lower accuracy. Note that the tolerance parameter also affects speed and accurcy. A large value is around `1E-2`, and a small value depends on the time step. By using values larger than `1E-2` the computation will probably take to long. Testing have reveled that `1E-3` works great for a time-step of 3 hours, while a gamma of `1E-2` takes a few minutes to compute. Use `1E-4` for a fast and tolerably accurate prediction. Defaults to `1E-3`
            
                tolerance (float (optional)): The krls training tolerance. Lower values makes the prediction more accurate, but slower. This typically have less effect than gamma, but is usefull for tuning. Usually it should be either `0.01` or `0.001`. Defaults to `0.01`
            
                size (int (optional)): The size of the "memory" of the underlying predictor. The default value is usually enough. Defaults to `1000000`.
            
            Examples:
            
            >>> import numpy as np
            >>> import scipy.stats as stat
            >>> from shyft.time_series import (
            ...     Calendar, utctime_now, deltahours,
            ...     TimeAxis, TimeSeries
            ... )
            >>>
            >>> cal = Calendar()
            >>> t0 = utctime_now()
            >>> dt = deltahours(1)
            >>> n = 365*24  # one year
            >>>
            >>> # generate random bell-shaped data
            >>> norm = stat.norm()
            >>> data = np.linspace(0, 20, n)
            >>> data = stat.norm(10).pdf(data) + norm.pdf(np.random.rand(*data.shape))
            >>> # -----
            >>> ta = TimeAxis(cal, t0, dt, n)
            >>> ts = TimeSeries(ta, data)
            >>>
            >>> # create a predictor
            >>> pred = ts.get_krls_predictor()
            >>> total_mse = pred.predictor_mse(ts)  # compute mse relative to ts
            >>> krls_ts = pred.predict(ta)  # generate a prediction, this is the result from ts.krls_interpolation
            >>> krls_mse_ts = pred.mse_ts(ts, points=6)  # compute a mse time-series using 6 points around each sample
            
            Returns:
                KrlsRbfPredictor: krls_predictor. A KRLS predictor pre-trained once on self.
            Other related methods are:
             :meth:`shyft.time_series.TimeSeries.krls_interpolation` 
            
        """
        ...

    def get_time_axis(self) -> TimeAxis:
        """
        TimeAxis: the time-axis
        """
        ...

    def ice_packing(self, ip_params: IcePackingParameters, ipt_policy: ice_packing_temperature_policy) -> TimeSeries:
        """
        Create a binary time-series indicating whether ice-packing is occuring or not.
            
            Note:
            
                `self` is interpreted and assumed to be a temperature time-series.
            
            The ice packing detection is based on the mean temperature in a predetermined time
            window before the time-point of interrest (see `IcePackingParameters.window`.
            The algorithm determines there to be ice packing when the mean temperature is below
            a given threshold temperature (see `IcePackingParameters.threshold_temp`).
            
            
            Args:
                ip_param (IcePackingParameters): Parameter container controlling the ice packing detection.
            
                ipt_policy (ice_packing_temperature_policy): Policy flags for determining how to deal with missing temperature values.
            
            
            Returns:
                TimeSeries: ice_packing_ts. A time-series indicating wheter ice packing occurs or not
            
            Example:
            
            >>> import numpy as np
            >>> from shyft.time_series import (
            ...     IcePackingParameters, ice_packing_temperature_policy,
            ...     TimeAxis, TimeSeries, point_interpretation_policy, DoubleVector,
            ...     utctime_now, deltahours, deltaminutes,
            ... )
            >>> 
            >>> t0 = utctime_now()
            >>> dt = deltaminutes(15)
            >>> n = 100
            >>> 
            >>> # generate jittery data
            >>> # - first descending from +5 to -5 then ascending back to +5
            >>> # - include a NaN hole at the bottom of the V
            >>> n_ = n if (n//2)*2 == n else n+1  # assure even
            >>> data = np.concatenate((
            ...     np.linspace(5, -5, n_//2), np.linspace(-5, 5, n_//2)
            ... )) + np.random.uniform(-0.75, 0.75, n_)  # add uniform noise
            >>> data[n_//2 - 1:n_//2 + 2] = float('nan')  # add some missing data
            >>> 
            >>> # create Shyft data structures
            >>> ta = TimeAxis(t0, dt, n_)
            >>> temperature_ts = TimeSeries(ta, DoubleVector.from_numpy(data),
            ...                             point_interpretation_policy.POINT_AVERAGE_VALUE)
            >>> 
            >>> # do the ice packing detection
            >>> ip_param = IcePackingParameters(
            ...     threshold_window=deltahours(5),
            ...     threshold_temperature=-1.0)
            >>> # try all the different temperature policies
            >>> ice_packing_ts_disallow = temperature_ts.ice_packing(ip_param, ice_packing_temperature_policy.DISALLOW_MISSING)
            >>> ice_packing_ts_initial = temperature_ts.ice_packing(ip_param, ice_packing_temperature_policy.ALLOW_INITIAL_MISSING)
            >>> ice_packing_ts_any = temperature_ts.ice_packing(ip_param, ice_packing_temperature_policy.ALLOW_ANY_MISSING)
            >>> 
            >>> # plotting
            >>> from matplotlib import pyplot as plt
            >>> from shyft.time_series import time_axis_extract_time_points
            >>> 
            >>> # NOTE: The offsets below are added solely to be able to distinguish between the different time-axes
            >>> 
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], temperature_ts.values, label='Temperature')
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], ice_packing_ts_disallow.values.to_numpy() + 1,
            ...          label='Ice packing? [DISALLOW_MISSING]')
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], ice_packing_ts_initial.values.to_numpy() - 1,
            ...          label='Ice packing? [ALLOW_INITIAL_MISSING]')
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], ice_packing_ts_any.values.to_numpy() - 3,
            ...          label='Ice packing? [ALLOW_ANY_MISSING]')
            >>> plt.legend()
            >>> plt.show()
            
        """
        ...

    def ice_packing_recession(self, ip_ts: TimeSeries, ipr_params: IcePackingRecessionParameters) -> TimeSeries:
        """
        Create a new time series where segments are replaced by recession curves.
            
            Note:
            
                The total period (`TimeSeries.total_period`) of `self` needs to be equal to,
                or contained in the total period of `ip_ts`.
            
            
            Args:
                ip_ts (TimeSeries): A binary time-series indicating if ice packing occurring. See `TimeSeries.ice_packing`.
            
                ip_param (IcePackingParameters): Parameter container controlling the ice packing recession curve.
            
            
            Returns:
                TimeSeries: ice_packing_recession_ts. A time-series where sections in `self` is replaced by recession curves as indicated by `ip_ts`.
            
            Example:
            
            >>> import numpy as np
            >>> from shyft.time_series import (
            ...     IcePackingParameters, IcePackingRecessionParameters, ice_packing_temperature_policy,
            ...     TimeAxis, TimeSeries, point_interpretation_policy, DoubleVector,
            ...     utctime_now, deltahours, deltaminutes,
            ... )
            >>> 
            >>> t0 = utctime_now()
            >>> dt = deltaminutes(15)
            >>> n = 100
            >>> 
            >>> # generate jittery temperature data
            >>> # - first descending from +5 to -5 then ascending back to +5
            >>> # - include a NaN hole at the bottom of the V
            >>> n_ = n if (n//2)*2 == n else n+1  # assure even
            >>> temperature_data = np.concatenate((
            ...     np.linspace(5, -5, n_//2), np.linspace(-5, 5, n_//2)
            ... )) + np.random.uniform(-0.75, 0.75, n_)  # add uniform noise
            >>> temperature_data[n_ // 2 - 1:n_ // 2 + 2] = float('nan')  # add some missing data
            >>> 
            >>> # create Shyft data structures for temperature
            >>> ta = TimeAxis(t0, dt, n_)
            >>> temperature_ts = TimeSeries(ta, DoubleVector.from_numpy(temperature_data),
            ...                             point_interpretation_policy.POINT_AVERAGE_VALUE)
            >>> 
            >>> # generate jittery waterflow data
            >>> # - an upwards curving parabola
            >>> x0 = ta.total_period().start
            >>> x1 = ta.total_period().end
            >>> x = np.linspace(x0, x1, n_)
            >>> flow_data = -0.0000000015*(x - x0)*(x - x1) + 1 + np.random.uniform(-0.5, 0.5, n_)
            >>> del x0, x1, x
            >>> 
            >>> # create Shyft data structures for temperature
            >>> flow_ts = TimeSeries(ta, DoubleVector.from_numpy(flow_data),
            ...                      point_interpretation_policy.POINT_AVERAGE_VALUE)
            >>> 
            >>> # do the ice packing detection
            >>> ip_param = IcePackingParameters(
            ...     threshold_window=deltahours(5),
            ...     threshold_temperature=-1.0)
            >>> # compute the detection time-series
            >>> # ice_packing_ts = temperature_ts.ice_packing(ip_param, ice_packing_temperature_policy.DISALLOW_MISSING)
            >>> # ice_packing_ts = temperature_ts.ice_packing(ip_param, ice_packing_temperature_policy.ALLOW_INITIAL_MISSING)
            >>> ice_packing_ts = temperature_ts.ice_packing(ip_param, ice_packing_temperature_policy.ALLOW_ANY_MISSING)
            >>> 
            >>> # setup for the recession curve
            >>> ipr_param = IcePackingRecessionParameters(
            ...     alpha=0.00009,
            ...     recession_minimum=2.)
            >>> # compute a recession curve based on the ice packing ts
            >>> ice_packing_recession_ts_initial = flow_ts.ice_packing_recession(ice_packing_ts, ipr_param)
            >>> 
            >>> # plotting
            >>> from matplotlib import pyplot as plt
            >>> from shyft.time_series import time_axis_extract_time_points
            >>> 
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], temperature_ts.values, label='Temperature')
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], flow_ts.values, label='Flow')
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], ice_packing_ts.values.to_numpy(),
            ...          label='Ice packing?')
            >>> plt.plot(time_axis_extract_time_points(ta)[:-1], ice_packing_recession_ts_initial.values.to_numpy(),
            ...          label='Recession curve')
            >>> plt.legend()
            >>> plt.show()
            
        """
        ...

    def index_of(self, t: Union[time,float,int]) -> int:
        """
        return the index of the intervall that contains t, or npos if not found
        """
        ...

    def inside(self, min_v: object, max_v: object, nan_v: object = nan, inside_v: object = 1.0, outside_v: object = 0.0) -> TimeSeries:
        """
        Create an inside min-max range ts, that transforms the point-values
            that falls into the half open range [min_v .. max_v > to 
            the value of inside_v(default=1.0), or outside_v(default=0.0),
            and if the value considered is nan, then that value is represented as nan_v(default=nan)
            You would typically use this function to form a true/false series (inside=true, outside=false)
            
            
            Args:
                min_v (float): minimum range, values <  min_v are not inside min_v==NaN means no lower limit
            
                max_v (float): maximum range, values >= max_v are not inside. max_v==NaN means no upper limit
            
                nan_v (float): value to return if the value is nan
            
                inside_v (float): value to return if the ts value is inside the specified range
            
                outside_v (float): value to return if the ts value is outside the specified range
            
            Returns:
                TimeSeries: inside_ts. Evaluated on demand inside time-series
            
        """
        ...

    def integral(self, ta: TimeAxis) -> TimeSeries:
        """
        create a new ts that is the true integral of self
            over the specified time-axis ta.
            defined as integral of the non-nan part of each time-axis interval
            
            Args:
                ta (TimeAxis): time-axis that specifies the periods where true-integral is applied
            
            Returns:
                TimeSeries: ts. a new time-series expression, that will provide the true-integral when requested
            
            Notes:
                the self point interpretation policy is used when calculating the true average
            
        """
        ...

    def krls_interpolation(self, dt: Union[time,float,int], gamma: object = 0.001, tolerance: object = 0.01, size: object = 1000000) -> TimeSeries:
        """
        Compute a new TS that is a krls interpolation of self.
            
            The KRLS algorithm is a kernel regression algorithm for aproximating data, the implementation
            used here is from DLib: http://dlib.net/ml.html#krls
            The new time-series has the same time-axis as self, and the values vector contain no `nan` entries.
            
            If you also want the mean-squared error of the interpolation use get_krls_predictor instead, and
            use the predictor api to generate a interpolation and a mse time-series.
            Other related functions are TimeSeries.get_krls_predictor, KrlsRbfPredictor
            
            Args:
                dt (float): The time-step in seconds the underlying predictor is specified for. Note that this does not put a limit on time-axes used, but for best results it should be approximatly equal to the time-step of time-axes used with the predictor. In addition it should not be to long, else you will get poor results. Try to keep the dt less than a day, 3-8 hours is usually fine.
            
                gamma (float (optional)): Determines the width of the radial basis functions for the KRLS algorithm. Lower values mean wider basis functions, wider basis functions means faster computation but lower accuracy. Note that the tolerance parameter also affects speed and accurcy. A large value is around `1E-2`, and a small value depends on the time step. By using values larger than `1E-2` the computation will probably take to long. Testing have reveled that `1E-3` works great for a time-step of 3 hours, while a gamma of `1E-2` takes a few minutes to compute. Use `1E-4` for a fast and tolerably accurate prediction. Defaults to `1E-3`
            
                tolerance (float (optional)): The krls training tolerance. Lower values makes the prediction more accurate, but slower. This typically have less effect than gamma, but is usefull for tuning. Usually it should be either `0.01` or `0.001`. Defaults to `0.01`
            
                size (int (optional)): The size of the "memory" of the underlying predictor. The default value is usually enough. Defaults to `1000000`.
            
            Examples:
            
            >>> import numpy as np
            >>> import scipy.stats as stat
            >>> from shyft.time_series import (
            ...     Calendar, utctime_now, deltahours,
            ...     TimeAxis, TimeSeries
            ... )
            >>>
            >>> cal = Calendar()
            >>> t0 = utctime_now()
            >>> dt = deltahours(1)
            >>> n = 365*24  # one year
            >>>
            >>> # generate random bell-shaped data
            >>> norm = stat.norm()
            >>> data = np.linspace(0, 20, n)
            >>> data = stat.norm(10).pdf(data) + norm.pdf(np.random.rand(*data.shape))
            >>> # -----
            >>> ta = TimeAxis(cal, t0, dt, n)
            >>> ts = TimeSeries(ta, data)
            >>>
            >>> # compute the interpolation
            >>> ts_ipol = ts.krls_interpolation(deltahours(3))
            
            
            Returns:
                TimeSeries: krls_ts. A new time series being the KRLS interpolation of self.
            
        """
        ...

    def log(self) -> TimeSeries:
        """
        create a new ts that contains log(py::self)
        """
        ...

    def lower_half(self) -> TimeSeries:
        """
        Create a ts that contains non-negative values only.
            
            Returns:
                TimeSeries: lower_half_ts. Evaluated on demand inside time-series
            
        """
        ...

    def lower_half_mask(self) -> TimeSeries:
        """
        Create a ts that contains 1.0 in place of non-positive values, and 0.0 in case of positive values.
            
            Returns:
                TimeSeries: lower_half_mask_ts. Evaluated on demand inside time-series
            
        """
        ...

    @overload
    def max(self, number: object) -> TimeSeries:
        """
        create a new ts that contains the max of self and number for each time-step
        
        """
        ...
    @overload
    def max(self, ts_other: TimeSeries) -> TimeSeries:
        """
        create a new ts that contains the max of self and ts_other
        """
        ...

    def merge_points(self, ts: TimeSeries) -> TimeSeries:
        """
        Given that self is a concrete point-ts(not an expression), or empty ts,
            this function modifies the point-set of self, with points, (time,value) from other ts
            The result of the merge operation is the distinct set of time-points from self and other ts
            where values from other ts overwrites values of self if they happen
            to be at the same time-point
            
            Args:
                ts (TimeSeries): time-series to merge the time,value points from
            
            Returns:
                TimeSeries: self. self modified with the merged points from other ts
            
        """
        ...

    @overload
    def min(self, number: object) -> TimeSeries:
        """
        create a new ts that contains the min of self and number for each time-step
        
        """
        ...
    @overload
    def min(self, ts_other: TimeSeries) -> TimeSeries:
        """
        create a new ts that contains the min of self and ts_other
        """
        ...

    @overload
    def min_max_check_linear_fill(self, v_min: object, v_max: object, dt_max: object = time.max) -> TimeSeries:
        """
        Create a min-max range checked ts with fill-values if value is NaN or outside range
            If the underlying time-series is point-instant, then fill-values are linear-interpolation,
            otherwise, the previous value, if available is used as fill-value.
            A similar function with more features is quality_and_self_correction()
            
            Args:
                v_min (float): minimum range, values < v_min are considered NaN. v_min==NaN means no lower limit
            
                v_max (float): maximum range, values > v_max are considered NaN. v_max==NaN means no upper limit
            
                dt_max (int): maximum time-range in seconds allowed for interpolating/extending values, default= max_utctime
            
            Returns:
                TimeSeries: min_max_check_linear_fill. Evaluated on demand time-series with NaN, out of range values filled in
            
        
        """
        ...
    @overload
    def min_max_check_linear_fill(self, v_min: object, v_max: object, dt_max: Union[time,float,int] = time.max) -> TimeSeries:
        """
        Create a min-max range checked ts with fill-values if value is NaN or outside range
            If the underlying time-series is point-instant, then fill-values are linear-interpolation,
            otherwise, the previous value, if available is used as fill-value.
            Similar and more parameterized function is quality_and_self_correction()
            
            Args:
                v_min (float): minimum range, values < v_min are considered NaN. v_min==NaN means no lower limit
            
                v_max (float): maximum range, values > v_max are considered NaN. v_max==NaN means no upper limit
            
                dt_max (int): maximum time-range in seconds allowed for interpolating/extending values, default= max_utctime
            
            Returns:
                TimeSeries: min_max_check_linear_fill. Evaluated on demand time-series with NaN, out of range values filled in
            
        """
        ...

    @overload
    def min_max_check_ts_fill(self, v_min: object, v_max: object, dt_max: object, cts: TimeSeries) -> TimeSeries:
        """
        Create a min-max range checked ts with cts-filled-in-values if value is NaN or outside range
            
            Args:
                v_min (float): minimum range, values < v_min are considered NaN. v_min==NaN means no lower limit
            
                v_max (float): maximum range, values > v_max are considered NaN. v_max==NaN means no upper limit
            
                dt_max (int): maximum time-range in seconds allowed for interpolating values
            
                cts (TimeSeries): time-series that keeps the values to be filled in at points that are NaN or outside min-max-limits
            
            Returns:
                TimeSeries: min_max_check_ts_fill. Evaluated on demand time-series with NaN, out of range values filled in
            
        
        """
        ...
    @overload
    def min_max_check_ts_fill(self, v_min: object, v_max: object, dt_max: Union[time,float,int], cts: TimeSeries) -> TimeSeries:
        """
        Create a min-max range checked ts with cts-filled-in-values if value is NaN or outside range
            
            Args:
                v_min (float): minimum range, values < v_min are considered NaN. v_min==NaN means no lower limit
            
                v_max (float): maximum range, values > v_max are considered NaN. v_max==NaN means no upper limit
            
                dt_max (int): maximum time-range in seconds allowed for interpolating values
            
                cts (TimeSeries): time-series that keeps the values to be filled in at points that are NaN or outside min-max-limits
            
            Returns:
                TimeSeries: min_max_check_ts_fill. Evaluated on demand time-series with NaN, out of range values filled in
            
        """
        ...

    def needs_bind(self) -> bool:
        """
        returns true if there are any unbound time-series in the expression
            this time-series represent
            These functions also supports symbolic time-series handling: .find_ts_bind_info(),bind() and bind_done()
            
        """
        ...

    def partition_by(self, calendar: Calendar, t: Union[time,float,int], partition_interval: Union[time,float,int], n_partitions: object, common_t0: Union[time,float,int]) -> TsVector:
        """
        DEPRECATED(replaced by .stack ) : from a time-series, construct a TsVector of n time-series partitions.
            The partitions are simply specified by calendar, delta_t(could be symbolic, like YEAR : MONTH:DAY) and n.
            To make yearly partitions, just pass Calendar.YEAR as partition_interval.
            The t - parameter set the start - time point in the source-time-series, e.g. like 1930.09.01
            The common_t0 - parameter set the common start - time of the new partitions, e.g. 2017.09.01
            
            The typical usage will be to use this function to partition years into a vector with
            80 years, where we can do statistics, percentiles to compare and see the different effects of
            yearly season variations.
            Note that the function is more general, allowing any periodic partition, like daily, weekly, monthly etc.
            that allows you to study any pattern or statistics that might be periodic by the partition pattern.
            Other related methods are time_shift,average,TsVector.
            
            Args:
                calendar (Calendar): The calendar to use, typically utc
            
                t (utctime): specifies where to pick the first partition
            
                partition_interval (utctimespan): the length of each partition, Calendar.YEAR,Calendar.DAY etc.
            
                n_partitions (int): number of partitions
            
                common_t0 (utctime): specifies the time to correlate all the partitions
            
            Returns:
                TsVector: ts-partitions. with length n_partitions, each ts is time-shifted to common_t0 expressions
            
        """
        ...

    def point_interpretation(self) -> point_interpretation_policy:
        """
        returns the point interpretation policy
        """
        ...

    @overload
    def pow(self, number: object) -> TimeSeries:
        """
        create a new ts that contains pow(py::self,number)
        
        """
        ...
    @overload
    def pow(self, ts_other: TimeSeries) -> TimeSeries:
        """
        create a new ts that contains pow(py::self,ts_other)
        """
        ...

    def quality_and_self_correction(self, parameters: QacParameter) -> TimeSeries:
        """
        returns a new time-series that applies quality checks accoring to parameters
            and fills in values according to rules specified in parameters.
            
            Args:
                parameter (QacParameter): Parameter with rules for quality and corrections
            
            Returns:
                TimeSeries: ts. a new time-series where the values are subject to quality and correction as specified
            
        """
        ...

    def quality_and_ts_correction(self, parameters: QacParameter, cts: TimeSeries) -> TimeSeries:
        """
        returns a new time-series that applies quality checks accoring to parameters
            and fills in values from the cts, according to rules specified in parameters.
            
            Args:
                parameter (QacParameter): Parameter with rules for quality and corrections
            
                cts (TimeSeries): is used to fill in correct values, as f(t) for values that fails quality-checks
            
            Returns:
                TimeSeries: ts. a new time-series where the values are subject to quality and correction as specified
            
        """
        ...

    def rating_curve(self, rc_param: RatingCurveParameters) -> TimeSeries:
        """
        Create a new TimeSeries that is computed using a RatingCurveParameter instance.
            
            Examples:
            
            
            >>> import numpy as np
            >>> from shyft.time_series import (
            ...     utctime_now, deltaminutes,
            ...     TimeAxis, TimeSeries,
            ...     RatingCurveFunction, RatingCurveParameters
            ... )
            >>>
            >>> # parameters
            >>> t0 = utctime_now()
            >>> dt = deltaminutes(30)
            >>> n = 48*2
            >>>
            >>> # make rating function, each with two segments
            >>> rcf_1 = RatingCurveFunction()
            >>> rcf_1.add_segment(0, 2, 0, 1)    # add segment from level 0, computing f(h) = 2*(h - 0)**1
            >>> rcf_1.add_segment(5.3, 1, 1, 1.4)  # add segment from level 5.3, computing f(h) = 1.3*(h - 1)**1.4
            >>> rcf_2 = RatingCurveFunction()
            >>> rcf_2.add_segment(0, 1, 1, 1)    # add segment from level 0, computing f(h) = 1*(h - 1)**1
            >>> rcf_2.add_segment(8.0, 0.5, 0, 2)  # add segment from level 8.0, computing f(h) = 0.5*(h - 0)**2
            >>>
            >>> # add rating curves to a parameter pack
            >>> rcp = RatingCurveParameters()
            >>> rcp.add_curve(t0, rcf_1)  # rcf_1 is active from t0
            >>> rcp.add_curve(t0+dt*n//2, rcf_2)  # rcf_2 takes over from t0 + dt*n/2
            >>>
            >>> # create a time-axis/-series
            >>> ta = TimeAxis(t0, dt, n)
            >>> ts = TimeSeries(ta, np.linspace(0, 12, n))
            >>> rc_ts = ts.rating_curve(rcp)  # create a new time series computed using the rating curve functions
            >>>
            
            Args:
                rc_param (RatingCurveParameter): RatingCurveParameter instance.
            
            Returns:
                TimeSeries: rcts. A new TimeSeries computed using self and rc_param.
            
        """
        ...

    def repeat(self, repeat_time_axis: TimeAxis) -> TimeSeries:
        """
        Repeat all time-series over the given repeat_time_axis periods
            
            Args:
                repeat_time_axis (TimeAxis): A time-axis that have the coarse repeat interval, like YEAR or similar
            
            Returns:
                TimeSeries: repeated_ts. time-series where pattern of self is repeated throughout the period of repeat_time_axis
            
        """
        ...

    def scale_by(self, v: object) -> None:
        """
        scale all values by the specified factor v
        """
        ...

    def serialize(self) -> ByteVector:
        """
        convert ts (expression) into a binary blob
            
        """
        ...

    def set(self, i: object, v: object) -> None:
        """
        set the i'th value
        """
        ...

    def set_point_interpretation(self, policy: point_interpretation_policy) -> None:
        """
        set new policy
        """
        ...

    def set_ts_id(self, ts_id: str) -> None:
        """
        Set a new ts_id of symbolic ts, requires unbound ts.
            To create symbolic time-series use TimeSeries('url://like/id') or with payload: TimeSeries('url://like/id',ts_with_values)
            
        """
        ...

    def size(self) -> int:
        """
        returns number of points
        """
        ...

    def slice(self, i0: object, n: object) -> TimeSeries:
        """
        Given that self is a concrete point-ts(not an expression), or empty ts,
            return a new TimeSeries containing the n values starting from index i0.
            
            Args:
                i0 (int): Index of first element to include in the slice
            
                n (int): Number of elements to include in the slice
            
        """
        ...

    def stack(self, calendar: Calendar, t0: Union[time,float,int], n_dt: object, dt: Union[time,float,int], n_partitions: object, target_t0: Union[time,float,int], dt_snap: Union[time,float,int]) -> TsVector:
        """
        stack time-series into a TsVector of n_partitions time-series, each with semantic calendar length n_dt x dt.
            The partitions are simply specified by calendar, n_dt x dt(could be symbolic, like YEAR : MONTH:DAY) and n.
            To make yearly partitions, just pass 1, Calendar.YEAR as n_dt and dt respectively.
            The t0 - parameter set the start - time point in the source-time-series, e.g. like 1930.09.01
            The target_t0 - parameter set the common start-time of the stack, e.g. 2017.09.01
            The dt_snap - parameter is useful to ensure that if target_to is a monday, then each partition is adjusted to neares monday.
            The snap mechanism could be useful if you would like to stack something like consumption, that would follow a weekly pattern.
            
            The typical usage will be to use this function to partition years into a vector with
            80 years, where we can do statistics, percentiles to compare and see the different effects of
            yearly season variations.
            Note that the function is more general, allowing any periodic partition, like daily, weekly, monthly etc.
            that allows you to study any pattern or statistics that might be periodic by the partition pattern.
            Other related methods are time_shift,average,TsVector.
            
            Args:
                calendar (Calendar): The calendar to use, typically utc
            
                t0 (utctime): specifies where to pick the first partition, e.g. 1930.09.01
            
                n_dt (int): number of calendar units for the length of the stride
            
                dt (utctimespan): the basic calendar length unit, Calendar.YEAR,Calendar.DAY
            
                n_partitions (int): number of partitions,e.g. length of the resulting TsVector
            
                target_t0 (utctime): specifies the common target time for the stack, e.g. 2017.09.01
            
                dt_snap (utctimespan): default 0, if set to WEEK, each stacked partition will be week-aligned.
            
            Returns:
                TsVector: stacked_ts. with length n_partitions, each ts is time-shifted (calendar n_dt x n) to common_t0 expressions
            
        """
        ...

    def statistics(self, ta: TimeAxis, p: object) -> TimeSeries:
        """
        Create a new ts that extract the specified statistics from `self` over the specified time-axis `ta`
            Statistics are created for the point values of the time-series that falls
            within each time-period of the time-axis.
            If there are no points within the period, nan will be the result.
            Tip: use `ts.average(ta_hourly_resolution).statistics(ta_weekly,p=50)`
            to get the functional true  hourly average statistics.
            
            
            Args:
                ta (TimeAxis): time-axis for the statistics
            
                p (int): percentile range [0..100], or statistical_property.AVERAGE|MIN_EXTREME|MAX_EXTREME
            
            Returns:
                TimeSeries: ts. a new time-series expression, will provide the statistics when requested
            
        """
        ...

    def stringify(self) -> str:
        """
        return human-readable string of ts or expression
        """
        ...

    def time(self, i: object) -> time:
        """
        returns the time at the i'th point
        """
        ...

    def time_shift(self, delta_t: Union[time,float,int]) -> TimeSeries:
        """
        create a new ts that is a the time-shift'ed  version of self
            
            Args:
                delta_t (int): number of seconds to time-shift, positive values moves forward
            
            Returns:
                TimeSeries: ts. a new time-series, that appears as time-shifted version of self
            
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        returns the total period covered by the time-axis of this time-series
        """
        ...

    def transform(self, points: object, method: interpolation_scheme) -> TimeSeries:
        """
        Create a transformed time-series, having values taken from pointwise function evaluation.
            Function values are determined by interpolating the given points, using the specified method.
            Valid method arguments are 'polynomial', 'linear' and 'catmull-rom'.
            
            Returns:
                TimeSeries: transform_ts. New TimeSeries where each element is an evaluated-on-demand transformed time-series.
            
        """
        ...

    def ts_id(self) -> str:
        """
        returns ts_id of symbolic ts, or empty string if not symbolic ts
            To create symbolic time-series use TimeSeries('url://like/id') or with payload: TimeSeries('url://like/id',ts_with_values)
            
            Returns:
                str: ts_id. url-like ts_id as passed to constructor or empty if the ts is not a ts with ts_id
            
        """
        ...

    def unbind(self) -> None:
        """
        Reset the  ts-expression to unbound state, discarding bound symbol references.
            For time-series, or expressions, that does not have symbolic references, no effect, see also .find_ts_bind_info(),bind() and bind_done()
            
        """
        ...

    def upper_half(self) -> TimeSeries:
        """
        Create a ts that contains non-negative values only.
            
            Returns:
                TimeSeries: upper_half_ts. Evaluated on demand inside time-series
            
        """
        ...

    def upper_half_mask(self) -> TimeSeries:
        """
        Create a ts that contains 1.0 in place of non-negative values, and 0.0 in case of negative values.
            
            Returns:
                TimeSeries: upper_half_mask_ts. Evaluated on demand inside time-series
            
        """
        ...

    def use_time_axis(self, time_axis: TimeAxis) -> TimeSeries:
        """
        Create a new ts that have the same values as self, but filtered to the time-axis points from
            from the supplied time-axis.
            This function migth be useful for making new time-series, that exactly matches
            the time-axis of another series.
            Values of the resulting time-series is like like: 
            [self(t) for t in time_axis.time_points[:-1]
            
            
            Args:
                time_axis (TimeAxis): the wanted time-axis
            
            Returns:
                TimeSeries: ts. a new time-series, that appears as resampled values of self
            
        """
        ...

    def use_time_axis_from(self, other: TimeSeries) -> TimeSeries:
        """
        Create a new ts that have the same values as self, but filtered to the time-axis points from
            from the other supplied time-series.
            This function migth be useful for making new time-series, that exactly matches
            the time-axis of another series.
            Values of the resulting time-series is like like: 
            [self(t) for t in other.time_axis.time_points[:-1]
            Notice that the `other` time-series can be an unbound (expression) in this case.
            
            Args:
                other (TimeSeries): time-series that provides the wanted time-axis
            
            Returns:
                TimeSeries: ts. a new time-series, that appears as resampled values of self
            
        """
        ...

    def value(self, i: object) -> float:
        """
        returns the value at the i'th time point
        """
        ...



class TsBindInfo:
    """
    TsBindInfo gives information about the time-series and it's binding
    represented by encoded string reference
    Given that you have a concrete ts,
    you can bind that the bind_info.ts
    using bind_info.ts.bind()
    see also Timeseries.find_ts_bind_info() and Timeseries.bind()
    """
    @property
    def id(self)->str:
        """
         a unique id/url that identifies a time-series in a ts-database/file-store/service
        """
        ...
    @id.setter
    def id(self, value:str)->None:
        ...
    @property
    def ts(self)->TimeSeries:
        """
         the ts, provides .bind(another_ts) to set the concrete values
        """
        ...
    @ts.setter
    def ts(self, value:TimeSeries)->None:
        ...
    def __init__(self) -> None:
        ...



class TsBindInfoVector:
    """
    A vector of TsBindInfo
    see also TsBindInfo
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class TsFactory:
    """
    TsFactory is used in specific contexts, to create point time-series that exposes the ITimeSeriesOfPoint interface, using the internal ts-implementations
    This class is intended for internal shyft-use, related to calibration/running etc.
    For geneneral time-series, please use TimeSeries() that have plenty well defined constructors
    """
    def __init__(self) -> None:
        ...

    @staticmethod
    def create_point_ts(*args, **kwargs):
        """
        creates a fixed interval time-series based on input parameters
            
            Args:
                n (int): number of points in time-series
            
                t (utctime): start of first point, as seconds since epoch
            
                dt (utctime): interval, in seconds
            
                values (DoubleVector): values as DoubleVector
            
                interpretation (ts_point_fx): point interpretation, default POINT_INSTANT_VALUE, other value is POINT_AVERAGE_VALUE 
            
            Returns:
                TimeSeries: ts. constructed time-series
            
        """
        ...

    @staticmethod
    def create_time_point_ts(*args, **kwargs):
        """
        creates a variable interval time-series based on input parameters
            
            Args:
                period (utcperiod):  where .start should be equal to the first point in the supplied time-vector,t, and .end should be the end-time of the last interval
            
                times (UtcTimeVector): start time of each interval
            
                values (DoubleVector): values as DoubleVector, one for each interval
            
                interpretation (ts_point_fx): point interpretation, default POINT_INSTANT_VALUE, other value is POINT_AVERAGE_VALUE 
            
            Returns:
                TimeSeries: ts. constructed time-series
            
        """
        ...



class TsFixed:
    """
    A time-series with a fixed delta t time-axis, used by the Shyft core,see also TimeSeries for end-user ts
    """
    @property
    def v(self)->DoubleVector:
        """
         the point vector<double>, same as .values, kept around for backward compatibility
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, ta: TimeAxisFixedDeltaT, v: Union[DoubleVector,list[float],np.ndarray], policy: point_interpretation_policy) -> None:
        """
        constructs a new timeseries from timeaxis, points and policy (how the points are to be interpreted, instant, or average of the interval)
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxisFixedDeltaT, fill_value: object, policy: point_interpretation_policy) -> None:
        """
        constructs a new timeseries from timeaxis, fill-value and policy
            
        """
        ...

    def __call__(self, t: Union[time,float,int]) -> float:
        """
        return the f(t) value for the time-series
        """
        ...

    def fill(self, v: object) -> None:
        """
        fill all values with v
        """
        ...

    def get(self, i: object) -> Point:
        """
        returns i'th point(t,v)
        """
        ...

    def get_time_axis(self) -> TimeAxisFixedDeltaT:
        """
        returns the time-axis
        """
        ...

    def index_of(self, t: Union[time,float,int]) -> int:
        """
        return the index of the intervall that contains t, or npos if not found
        """
        ...

    def point_interpretation(self) -> point_interpretation_policy:
        """
        returns the point interpretation policy
        """
        ...

    def scale_by(self, v: object) -> None:
        """
        scale all values by the specified factor v
        """
        ...

    def set(self, i: object, v: object) -> None:
        """
        set the i'th value
        """
        ...

    def set_point_interpretation(self, policy: point_interpretation_policy) -> None:
        """
        set new policy
        """
        ...

    def size(self) -> int:
        """
        returns number of points
        """
        ...

    def time(self, i: object) -> time:
        """
        returns the time at the i'th point
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        returns the total period covered by the time-axis of this time-series
        """
        ...

    def value(self, i: object) -> float:
        """
        returns the value at the i'th time point
        """
        ...



class TsInfo:
    """
    Gives some information from the backend ts data-store
    about the stored time-series, that could be useful in some contexts
    """
    @property
    def created(self)->Any:
        """
        when time-series was created, seconds 1970s utc
        """
        ...
    @created.setter
    def created(self, value:Any)->None:
        ...
    @property
    def data_period(self)->Any:
        """
        the period for data-stored, if applicable
        """
        ...
    @data_period.setter
    def data_period(self, value:Any)->None:
        ...
    @property
    def delta_t(self)->Any:
        """
        time-axis steps, in seconds, 0 if irregular time-steps
        """
        ...
    @delta_t.setter
    def delta_t(self, value:Any)->None:
        ...
    @property
    def modified(self)->Any:
        """
        when time-series was last modified, seconds 1970 utc
        """
        ...
    @modified.setter
    def modified(self, value:Any)->None:
        ...
    @property
    def name(self)->Any:
        """
        the unique name
        """
        ...
    @name.setter
    def name(self, value:Any)->None:
        ...
    @property
    def olson_tz_id(self)->Any:
        """
        empty or time-axis calendar for calendar,t0,delta_t type time-axis
        """
        ...
    @olson_tz_id.setter
    def olson_tz_id(self, value:Any)->None:
        ...
    @property
    def point_fx(self)->Any:
        """
        how to interpret the points, instant value, or average over period
        """
        ...
    @point_fx.setter
    def point_fx(self, value:Any)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, name: str, point_fx: point_interpretation_policy, delta_t: Union[time,float,int], olson_tz_id: str, data_period: UtcPeriod, created: Union[time,float,int], modified: Union[time,float,int]) -> None:
        """
        construct a TsInfo with all values specified
            
        """
        ...



class TsInfoVector:
    """
    A strongly typed list of TsInfo
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: TsInfoVector) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class TsPoint:
    """
    A time-series with a variable delta time-axis, used by the Shyft core,see also TimeSeries for end-user ts
    """
    @property
    def v(self)->DoubleVector:
        """
         the point vector<double>, same as .values, kept around for backward compatibility
        """
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, ta: TimeAxisByPoints, v: Union[DoubleVector,list[float],np.ndarray], policy: point_interpretation_policy) -> None:
        """
        constructs a new timeseries from timeaxis, points and policy (how the points are to be interpreted, instant, or average of the interval)
            
        
        """
        ...
    @overload
    def __init__(self, ta: TimeAxisByPoints, fill_value: object, policy: point_interpretation_policy) -> None:
        """
        constructs a new timeseries from timeaxis, fill-value and policy
            
        """
        ...

    def __call__(self, t: Union[time,float,int]) -> float:
        """
        return the f(t) value for the time-series
        """
        ...

    def fill(self, v: object) -> None:
        """
        fill all values with v
        """
        ...

    def get(self, i: object) -> Point:
        """
        returns i'th point(t,v)
        """
        ...

    def get_time_axis(self) -> TimeAxisByPoints:
        """
        returns the time-axis
        """
        ...

    def index_of(self, t: Union[time,float,int]) -> int:
        """
        return the index of the intervall that contains t, or npos if not found
        """
        ...

    def point_interpretation(self) -> point_interpretation_policy:
        """
        returns the point interpretation policy
        """
        ...

    def scale_by(self, v: object) -> None:
        """
        scale all values by the specified factor v
        """
        ...

    def set(self, i: object, v: object) -> None:
        """
        set the i'th value
        """
        ...

    def set_point_interpretation(self, policy: point_interpretation_policy) -> None:
        """
        set new policy
        """
        ...

    def size(self) -> int:
        """
        returns number of points
        """
        ...

    def time(self, i: object) -> time:
        """
        returns the time at the i'th point
        """
        ...

    def total_period(self) -> UtcPeriod:
        """
        returns the total period covered by the time-axis of this time-series
        """
        ...

    def value(self, i: object) -> float:
        """
        returns the value at the i'th time point
        """
        ...



class TsVector:
    """
    A vector, as in strongly typed list, array, of time-series that supports ts-math operations.
    You can create a TsVector from a list, or list generator of type TimeSeries.
    TsVector is to TimeSeries that a numpy array is to numbers, see also :class:`TimeSeries` 
    
    
    Math operations and their types transformations:
    
        * `number bin_op ts_vector -> ts_vector`
        * `ts_vector bin_op ts_vector -> ts_vector`
        * `ts bin_op ts_vector -> ts_vector`
    
    where bin_op is any of `(*,/,+,-)` and explicit forms of  binary functions like pow,log,min,max.
    
    
    In addition these are also available: :meth:`average`  :meth:`integral`  :meth:`accumulate`  :meth:`time_shift`  :meth:`percentiles` 
    
    All operation return a **new** object, usually a ts-vector, containing the resulting expressions
    
    
    Examples:
    
    >>> import numpy as np
    >>> from shyft.time_series import TsVector,Calendar,deltahours,TimeAxis,TimeSeries,POINT_AVERAGE_VALUE as fx_avg
    >>>
    >>> utc = Calendar()  # ensure easy consistent explicit handling of calendar and time
    >>> ta1 = TimeAxis(utc.time(2016, 9, 1, 8, 0, 0), deltahours(1), 10)  # create a time-axis for ts1
    >>> ts1 = TimeSeries(ta1, np.linspace(0, 10, num=len(ta)), fx_avg)
    >>> ta2 = TimeAxis(utc.time(2016, 9, 1, 8, 30, 0), deltahours(1), 5)  # create a time-axis to ts2
    >>> ts2 = TimeSeries(ta2, np.linspace(0,  1, num=len(ta)), fx_avg)
    >>> tsv = TsVector([ts1, ts2]) # create ts vector from list of time-series
    >>> c = tsv + tsv*3.0  # c is now an expression, time-axis is the overlap of a and b, lazy evaluation
    >>> c_values = c[0].values.to_numpy()  # compute and extract the values of the ith (here: 0) time-series, as numpy array
    >>>
    >>> # Calculate data for new time-points
    >>> value_1 = tsv(utc.time(2016, 9, 1, 8, 30)) # calculates value at a given time
    >>> ta_target = TimeAxis(utc.time(2016, 9, 1, 7, 30), deltahours(1), 12)  # create a target time_axis
    >>> tsv_new = tsv.average(ta_target) # new ts-vector with values on target time_axis
    >>> ts0_val = tsv_new[0].values.to_numpy() # access values of the ith (here: 0) time-series as a numpy array
    >>>
    """
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self, clone: TsVector) -> None:
        """
        Create a clone.
        
        """
        ...
    @overload
    def __init__(self) -> Any:
        """
        Create an empty TsVector
            
        
        """
        ...
    @overload
    def __init__(self, cloneme: TsVector) -> Any:
        """
        Create a shallow clone of  the TsVector
            
            Args:
                cloneme (TsVector): The TsVector to be cloned
            
        
        """
        ...
    @overload
    def __init__(self, ts_list: list) -> Any:
        """
        Create a TsVector from a python list of TimeSeries
            
            Args:
                ts_list (List[TimeSeries]): A list of TimeSeries
            
        """
        ...

    @overload
    def __add__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __add__(self, arg2: TsVector) -> TsVector:
        ...
    @overload
    def __add__(self, arg2: TimeSeries) -> TsVector:
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    @overload
    def __mul__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __mul__(self, arg2: TsVector) -> TsVector:
        ...
    @overload
    def __mul__(self, arg2: TimeSeries) -> TsVector:
        ...

    def __neg__(self) -> TsVector:
        ...

    @overload
    def __radd__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __radd__(self, arg2: TimeSeries) -> TsVector:
        ...

    @overload
    def __rmul__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __rmul__(self, arg2: TimeSeries) -> TsVector:
        ...

    @overload
    def __rsub__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __rsub__(self, arg2: TimeSeries) -> TsVector:
        ...

    @overload
    def __rtruediv__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __rtruediv__(self, arg2: TimeSeries) -> TsVector:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    @overload
    def __sub__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __sub__(self, arg2: TsVector) -> TsVector:
        ...
    @overload
    def __sub__(self, arg2: TimeSeries) -> TsVector:
        ...

    @overload
    def __truediv__(self, arg2: object) -> TsVector:
        ...
    @overload
    def __truediv__(self, arg2: TsVector) -> TsVector:
        ...
    @overload
    def __truediv__(self, arg2: TimeSeries) -> TsVector:
        ...

    def abs(self) -> TsVector:
        """
        create a new ts-vector, with all members equal to abs(py::self
            
            Returns:
                TsVector: tsv. a new TsVector expression, that will provide the abs-values of self.values
            
        """
        ...

    def accumulate(self, ta: TimeAxis) -> TsVector:
        """
        create a new  vector of time-series where the vaue of each i-th element is computed as:
            `integral f(t) *dt, from t0..ti`
            given the specified time-axis ta and point interpretation.
            
            
            Args:
                ta (TimeAxis): time-axis that specifies the periods where accumulated integral is applied
            
            Returns:
                TsVector: tsv. a new time-series expression, that will provide the accumulated values when requested
            
            Notes:
                Has a point-instant interpretation, see also note in :meth:`TimeSeries.accumulate` for possible consequences
        """
        ...

    def append(self, arg2: object) -> None:
        ...

    def average(self, ta: TimeAxis) -> TsVector:
        """
        create a new vector of ts that is the true average of self
            over the specified time-axis ta.
            
            Args:
                ta (TimeAxis): time-axis that specifies the periods where true-average is applied
            
            Returns:
                TsVector: tsv. a new time-series expression, that will provide the true-average when requested
            
            Notes:
                the self point interpretation policy is used when calculating the true average
            
        """
        ...

    def average_slice(self, lead_time: Union[time,float,int], delta_t: Union[time,float,int], n: object) -> TsVector:
        """
        Returns a ts-vector with the average time-series of the specified slice
            The slice for each ts is specified by the lead_time, delta_t and n
            parameters. 
            See also nash_sutcliffe,forecast_merge
            
            Args:
                lead_time (int): number of seconds lead-time offset from each ts .time(0)
            
                delta_t (int): delta-time seconds to average as basis for n.s. simulation and observation values
            
                n (int): number of time-steps of length delta_t to slice out of each forecast/simulation ts
            
            Returns:
                TsVector: ts_vector_sliced. a ts-vector with average ts of each slice specified.
            
        """
        ...

    def clone_expression(self) -> TsVector:
        """
        create a copy of the ts-expressions, except for the bound payload of the reference ts.
            For the reference terminals, those with `ts_id`, only the `ts_id` is copied.
            Thus, to re-evaluate the expression, those have to be bound.
            
            Notes:
                this function is only useful in context where multiple bind/rebind while keeping the expression is needed.
            
            Returns:
                TsVector: semantic_clone. returns a copy of the ts, except for the payload at reference/symbolic terminals, where only `ts_id`is copied
            
        """
        ...

    def derivative(self, method: derivative_method = derivative_method.DEFAULT) -> TsVector:
        """
        create a new  vector of ts where each i'th element is the
            `derivative of f(t)`
            
            Args:
                method (derivative_method): what derivative_method variant to use
            
            Returns:
                TsVector: tsv. where each member is the derivative of the source
            
        """
        ...

    def evaluate(self) -> TsVector:
        """
        Evaluates the expressions in TsVector multithreaded,
            and returns the resulting TsVector, where all items now
            are  concrete terminals, that is, not expressions anymore.
            Useful client-side if you have complex large expressions where
            all time-series are bound (not symbols)
            
            Returns:
                TsVector: evaluated_clone. returns the computed result as a new ts-vector
            
        """
        ...

    def extend(self, arg2: object) -> None:
        ...

    @overload
    def extend_ts(self, ts: TimeSeries, split_policy: extend_split_policy = extend_split_policy.LHS_LAST, fill_policy: extend_fill_policy = extend_fill_policy.FILL_NAN, split_at: Union[time,float,int] = "time(0)", fill_value: object = nan) -> TsVector:
        """
        create a new ats_vector where all time-series are extended by ts
            
            Args:
                ts (TimeSeries): time-series to extend each time-series in self with
            
                split_policy (extend_ts_split_policy): policy determining where to split between self and ts
            
                fill_policy (extend_ts_fill_policy): policy determining how to fill any gap between self and ts
            
                split_at (utctime): time at which to split if split_policy == EPS_VALUE
            
                fill_value (float): value to fill any gap with if fill_policy == EPF_FILL
            
            Returns:
                TsVector: new_ts_vec. a new time-series vector where all time-series in self have been extended by ts
            
        
        """
        ...
    @overload
    def extend_ts(self, ts: TsVector, split_policy: extend_split_policy = extend_split_policy.LHS_LAST, fill_policy: extend_fill_policy = extend_fill_policy.FILL_NAN, split_at: Union[time,float,int] = "time(0)", fill_value: object = nan) -> TsVector:
        """
        create a new ats_vector where all ts' are extended by the matching ts from ts_vec
            
            Args:
                ts_vec (TsVector): time-series vector to extend time-series in self with
            
                split_policy (extend_ts_split_policy): policy determining where to split between self and ts
            
                fill_policy (extend_ts_fill_policy): policy determining how to fill any gap between self and ts
            
                split_at (utctime): time at which to split if split_policy == EPS_VALUE
            
                fill_value (float): value to fill any gap with if fill_policy == EPF_FILL
            
            Returns:
                TsVector: new_ts_vec. a new time-series vector where all time-series in self have been extended by the corresponding time-series in ts_vec
            
        """
        ...

    def extract_as_table(self, cal: Calendar, time_scale: object) -> DoubleVectorVector:
        """
        Extract values in the ts-vector as a table, where columns:
            | [0]    is the distinct union of all time_scale*(time-points i, + cal.tz_offset(i))
            | [1..n] is the value contribution of the i'th ts, nan if no contribution at that time-point
            This function primary usage is within visual-layer of the shyft.dashboard package to speed up processing.
            The semantics and parameters reflects this.
            
            Args:
                cal (Calendar): Calendar to use for tz-offset of each time-point (to resolve bokeh lack of tz-handling)
            
                time_scale (float): time-scale to multiply the time from si-unit [s] to any scaled unit, typically ms
            
            Returns:
                DoubleVectorVector: table. A 2d vector where [0] contains time, [1..n] the values
            
        """
        ...

    def forecast_merge(self, lead_time: Union[time,float,int], fc_interval: Union[time,float,int]) -> TimeSeries:
        """
        merge the forecasts in this vector into a time-series that is constructed
            taking a slice of length fc_interval starting lead_time into each of the forecasts
            of this time-series vector.
            The content of the vector should be ordered in forecast-time, each entry at least
            fc_interval separated from the previous.
            If there is missing forecasts (larger than fc_interval between two forecasts) this is
            automagically repaired using extended slices from the existing forecasts
            
            Args:
                lead_time (int): start slice number of seconds from t0 of each forecast
            
                fc_interval (int): length of each slice in seconds, and thus also gives the forecast-interval separation
            
            Returns:
                TimeSeries: merged time-series. A merged forecast time-series
            
        """
        ...

    def inside(self, min_v: object, max_v: object, nan_v: object = nan, inside_v: object = 1.0, outside_v: object = 0.0) -> TsVector:
        """
        Create an inside min-max range ts-vector, that transforms the point-values
            that falls into the half open range [min_v .. max_v > to 
            the value of inside_v(default=1.0), or outside_v(default=0.0),
            and if the value considered is nan, then that value is represented as nan_v(default=nan)
            You would typically use this function to form a true/false series (inside=true, outside=false)
            
            
            Args:
                min_v (float): minimum range, values <  min_v are not inside min_v==NaN means no lower limit
            
                max_v (float): maximum range, values >= max_v are not inside. max_v==NaN means no upper limit
            
                nan_v (float): value to return if the value is nan
            
                inside_v (float): value to return if the ts value is inside the specified range
            
                outside_v (float): value to return if the ts value is outside the specified range
            
            Returns:
                TsVector: inside_tsv. New TsVector where each element is an evaluated-on-demand inside time-series
            
        """
        ...

    def integral(self, ta: TimeAxis) -> TsVector:
        """
        create a new vector of ts that is the true integral of self
            over the specified time-axis ta.
            defined as integral of the non-nan part of each time-axis interval
            
            Args:
                ta (TimeAxis): time-axis that specifies the periods where true-integral is applied
            
            Returns:
                TsVector: tsv. a new time-series expression, that will provide the true-integral when requested
            
            Notes:
                the self point interpretation policy is used when calculating the true average
            
        """
        ...

    def log(self) -> TsVector:
        """
        returns TsVector log(py::self)
        """
        ...

    @overload
    def max(self, number: object) -> TsVector:
        """
        returns max of vector and a number
        
        """
        ...
    @overload
    def max(self, ts: TimeSeries) -> TsVector:
        """
        returns max of ts-vector and a ts
        
        """
        ...
    @overload
    def max(self, tsv: TsVector) -> TsVector:
        """
        returns max of ts-vector and another ts-vector
        """
        ...

    @overload
    def min(self, number: object) -> TsVector:
        """
        returns min of vector and a number
        
        """
        ...
    @overload
    def min(self, ts: TimeSeries) -> TsVector:
        """
        returns min of ts-vector and a ts
        
        """
        ...
    @overload
    def min(self, tsv: TsVector) -> TsVector:
        """
        returns min of ts-vector and another ts-vector
        """
        ...

    def nash_sutcliffe(self, observation_ts: TimeSeries, lead_time: Union[time,float,int], delta_t: Union[time,float,int], n: object) -> float:
        """
        Computes the nash-sutcliffe (wiki nash-sutcliffe) criteria between the
            observation_ts over the slice of each time-series in the vector.
            The slice for each ts is specified by the lead_time, delta_t and n
            parameters. The function is provided to ease evaluation of forecast performance
            for different lead-time periods into each forecast.
            The returned value range is 1.0 for perfect match -oo for no match, or nan if observations is constant or data missing.
            See also nash_sutcliffe_goal_function
            
            Args:
                observation_ts (TimeSeries): the observation time-series
            
                lead_time (int): number of seconds lead-time offset from each ts .time(0)
            
                delta_t (int): delta-time seconds to average as basis for n.s. simulation and observation values
            
                n (int): number of time-steps of length delta_t to slice out of each forecast/simulation ts
            
            Returns:
                double: nash-sutcliffe value. the nash-sutcliffe criteria evaluated over all time-series in the TsVector for the specified lead-time, delta_t and number of elements
            
        """
        ...

    @overload
    def percentiles(self, time_axis: TimeAxis, percentiles: Union[IntVector,list[int],range]) -> TsVector:
        """
        
            Calculate the percentiles of all timeseries.
            over the specified time-axis. The definition is equal to e.g. NIST R7, excel, and in R.
            The time-series point_fx interpretation is used when performing
            the true-average over the time_axis periods.
            This functions works on bound expressions, for unbound expressions, use the `DtsClient.percentiles`.
            
            See also :meth:`DtsClient.percentiles` if you want to evaluate percentiles of an *unbound* expression.
            
            Args:
                percentiles (IntVector): A list of numbers,like `[ 0, 25,50,-1,75,100]` will return 6 time-series. Number with special sematics are: `-1 -> arithmetic average`, `-1000 -> min extreme value` `+1000 -> max extreme value`
            
                time_axis (TimeAxis): The time-axis used when applying true-average to the time-series
            
            Returns:
                TsVector: calculated_percentiles. Time-series list with evaluated percentile results, same length as input
            
        
        """
        ...
    @overload
    def percentiles(self, time_axis: TimeAxisFixedDeltaT, percentiles: Union[IntVector,list[int],range]) -> TsVector:
        """
        Calculate the percentiles of the timeseries. 
            over the specified time-axis. The definition is equal to e.g. NIST R7, excel, and in R.
            The time-series point_fx interpretation is used when performing
            the true-average over the time_axis periods.
            This functions works on bound expressions, for unbound expressions, use the `DtsClient.percentiles`.
            
            See also :meth:`DtsClient.percentiles` if you want to evaluate percentiles of an *unbound* expression.
            
            Args:
                percentiles (IntVector): A list of numbers,[ 0, 25,50,-1,75,100] will return 6 time-series,`-1 -> arithmetic average`, `-1000 -> min extreme value`, ` +1000 max extreme value`
            
                time_axis (TimeAxisFixedDeltaT): The time-axis used when applying true-average to the time-series
            
            Returns:
                TsVector: calculated_percentiles. Time-series list with evaluated percentile results, same length as input
            
        """
        ...

    @overload
    def pow(self, number: object) -> TsVector:
        """
        returns TsVector pow(py::self,number)
        
        """
        ...
    @overload
    def pow(self, ts: TimeSeries) -> TsVector:
        """
        returns TsVector pow(py::self,ts)
        
        """
        ...
    @overload
    def pow(self, tsv: TsVector) -> TsVector:
        """
        returns TsVector pow(py::self,tsv)
        """
        ...

    def repeat(self, repeat_time_axis: TimeAxis) -> TsVector:
        """
        Repeat all time-series over the given repeat_time_axis periods
            
            Args:
                repeat_time_axis (TimeAxis): A time-axis that have the coarse repeat interval, like YEAR or similar
            
            Returns:
                TsVector: tsv. time-series vector, where each element is repeated according to parameter
            
        """
        ...

    def slice(self, indexes: Union[IntVector,list[int],range]) -> TsVector:
        """
        returns a slice of self, specified by indexes
            
            Args:
                indexes (IntVector): the indicies to pick out from self, if indexes is empty, then all is returned
            
            Returns:
                TsVector: slice. a new TsVector, with content according to indexes specified
            
        """
        ...

    def statistics(self, ta: TimeAxis, p: object) -> TsVector:
        """
        create a new vector of ts where each element is ts.statistics(ta,p)
            
            Args:
                ta (TimeAxis): time-axis for the statistics
            
                p (int): percentile range [0..100], or statistical_property.AVERAGE|MIN_EXTREME|MAX_EXTREME
            
            Returns:
                TsVector: tsv. a new time-series expression, will provide the statistics when requested
            
        """
        ...

    def sum(self) -> TimeSeries:
        """
        returns sum of all ts in TsVector as ts as in reduce(add,..))
        """
        ...

    def time_shift(self, delta_t: Union[time,float,int]) -> TsVector:
        """
        create a new vector of ts that is a the time-shifted  version of self
            
            Args:
                delta_t (time): number of seconds to time-shift, positive values moves forward
            
            Returns:
                TsVector: tsv. a new time-series, that appears as time-shifted version of self
            
        """
        ...

    def transform(self, points: object, method: interpolation_scheme) -> TsVector:
        """
        Create a transformed ts-vector, having values taken from pointwise function evaluation.
            Function values are determined by interpolating the given points, using the specified method.
            Valid method arguments are 'polynomial', 'linear' and 'catmull-rom'.
            
            Returns:
                TsVector: transform_tsv. New TsVector where each element is an evaluated-on-demand transformed time-series.
            
        """
        ...

    def use_time_axis(self, time_axis: TimeAxis) -> TsVector:
        """
        Create a new ts-vector applying :meth:`TimeSeries.use_time_axis`  on each member, e.g. resampling instant values at specified time-points.
            
            
            Args:
                time_axis (TimeAxis): time-axis used to resample values from orignal ts
            
            Returns:
                TsVector: tsv. time-series vector, where each element have resampled time-axis values 
            
        """
        ...

    def use_time_axis_from(self, other: TimeSeries) -> TsVector:
        """
        Create a new ts-vector applying :meth:`TimeSeries.use_time_axis_from`  on each member
            
            
            Args:
                other (TimeSeries): time-series that provides the wanted time-axis
            
            Returns:
                TsVector: tsv. time-series vector, where each element have time-axis from other
            
        """
        ...

    def value_range(self, p: UtcPeriod) -> DoubleVector:
        """
        Computes min and max of all non-nan values in the period for bound expressions.
            
            Args:
                p (UtcPeriod): 
            
            Returns:
                DoubleVectorVector: values. Resulting [min_value, max_value]. If all values are equal, min = max = the_value
            
        """
        ...

    @overload
    def values_at(self, t: Union[time,float,int]) -> DoubleVector:
        """
        Computes the value at specified time t for all time-series
            
            Args:
                t (utctime): seconds since epoch 1970 UTC
            
        
        """
        ...
    @overload
    def values_at(self, t: object) -> DoubleVector:
        """
        Computes the value at specified time t for all time-series
            
            Args:
                t (int): seconds since epoch 1970 UTC
            
        """
        ...



class TsVectorSet:
    """
    A set of TsVector
    
    
    See also:
        quantile_map_forecast,TsVector
    """
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, clone: TsVectorSet) -> None:
        """
        Create a clone.
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...



class TzInfo:
    """
    The TzInfo class is responsible for providing information about the
    time-zone of the calendar.
    This includes:
    
     * name (olson identifier)
     * base_offset
     * utc_offset(t) time-dependent
    
    The Calendar class provides a shared pointer to it's TzInfo object
    """
    def __init__(self, base_tz: Union[time,float,int]) -> None:
        """
        creates a TzInfo with a fixed utc-offset(no dst-rules)
        """
        ...

    def base_offset(self) -> time:
        """
        returnes the time-invariant part of the utc-offset
        """
        ...

    def is_dst(self, t: Union[time,float,int]) -> bool:
        """
        returns true if DST is observed at given utc-time t
        """
        ...

    def name(self) -> str:
        """
        returns the olson time-zone identifier or name for the TzInfo
        """
        ...

    def utc_offset(self, t: Union[time,float,int]) -> time:
        """
        returns the utc_offset at specified utc-time, takes DST into account if applicable
        """
        ...



class UtcPeriod:
    """
    UtcPeriod defines the open utc-time range [start..end>
    where end is required to be equal or greater than start
    """
    @property
    def end(self)->time:
        """
         Defines the end of the period, not inclusive
        """
        ...
    @end.setter
    def end(self, value:time)->None:
        ...
    @property
    def start(self)->time:
        """
         Defines the start of the period, inclusive
        """
        ...
    @start.setter
    def start(self, value:time)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, start: Union[time,float,int], end: Union[time,float,int]) -> None:
        """
        Create utcperiod given start and end
        """
        ...

    @overload
    def contains(self, t: Union[time,float,int]) -> bool:
        """
        returns true if time t is contained in this utcperiod
        
        """
        ...
    @overload
    def contains(self, t: object) -> bool:
        """
        returns true if time t is contained in this utcperiod
        
        """
        ...
    @overload
    def contains(self, p: UtcPeriod) -> bool:
        """
        returns true if utcperiod p is contained in this utcperiod
        """
        ...

    @overload
    def diff_units(self, calendar: Calendar, delta_t: Union[time,float,int]) -> int:
        """
        Calculate the distance from start to end of UtcPeriod in specified units, taking dst into account if observed
            The function takes calendar semantics when delta_t is calendar::DAY,WEEK,MONTH,YEAR,
            and in addition also dst if observed.
            e.g. the diff_units of calendar::DAY over summer->winter shift is 1,
            even if the number of hours during those days are 23 and 25 summer and winter transition respectively
            
            Args:
                calendar (calendar): shyft calendar
            
                delta_t (time): timestep in seconds, with semantic interpretation of DAY,WEEK,MONTH,YEAR
            
            Returns:
                int: n_units. number of units in UtcPeriod
            
        
        """
        ...
    @overload
    def diff_units(self, calendar: Calendar, delta_t: object) -> int:
        """
        Calculate the distance from start to end of UtcPeriod in specified units, taking dst into account if observed
            The function takes calendar semantics when delta_t is calendar::DAY,WEEK,MONTH,YEAR,
            and in addition also dst if observed.
            e.g. the diff_units of calendar::DAY over summer->winter shift is 1,
            even if the number of hours during those days are 23 and 25 summer and winter transition respectively
            
            Args:
                calendar (calendar): shyft calendar
            
                delta_t (int): timestep in seconds, with semantic interpretation of DAY,WEEK,MONTH,YEAR
            
            Returns:
                int: n_units. number of units in UtcPeriod
            
        """
        ...

    @staticmethod
    def intersection(a: UtcPeriod, b: UtcPeriod) -> UtcPeriod:
        """
        Returns the intersection of two periods
            if there is an intersection, the resulting period will be .valid() and .timespan()>0
            If there is no intersection, an empty not .valid() period is returned
            
            Args:
                a (UtcPeriod): 1st UtcPeriod argument
            
                b (UtcPeriod): 2nd UtcPeriod argument
            
            Returns:
                UtcPeriod: intersection. The computed intersection, or an empty not .valid() UtcPeriod
            
        """
        ...

    def overlaps(self, p: UtcPeriod) -> bool:
        """
        returns true if period p overlaps this utcperiod
        """
        ...

    def timespan(self) -> time:
        """
        returns end-start, the timespan of the period
        """
        ...

    def to_string(self) -> str:
        """
        A readable representation in UTC
        """
        ...

    @overload
    def trim(self, calendar: Calendar, delta_t: Union[time,float,int], trim_policy: trim_policy = trim_policy.TRIM_IN) -> UtcPeriod:
        """
        Round UtcPeriod up or down to the nearest calendar time-unit delta_t
            taking the calendar time-zone and dst into account
            
            Args:
                calendar (calendar): shyft calendar
            
                delta_t (time): timestep in seconds, with semantic interpretation of Calendar.(DAY,WEEK,MONTH,YEAR)
            
                trim_policy (trim_policy): TRIM_IN if rounding period inwards, else rounding outwards
            
            Returns:
                UtcPeriod: trimmed_UtcPeriod. new trimmed UtcPeriod
            
        
        """
        ...
    @overload
    def trim(self, calendar: Calendar, delta_t: object, trim_policy: trim_policy = trim_policy.TRIM_IN) -> UtcPeriod:
        """
        Round UtcPeriod up or down to the nearest calendar time-unit delta_t
            taking the calendar time-zone and dst into account
            
            Args:
                calendar (calendar): shyft calendar
            
                delta_t (int): timestep in seconds, with semantic interpretation of Calendar.(DAY,WEEK,MONTH,YEAR)
            
                trim_policy (trim_policy): TRIM_IN if rounding period inwards, else rounding outwards
            
            Returns:
                UtcPeriod: trimmed_UtcPeriod. new trimmed UtcPeriod
            
        """
        ...

    def valid(self) -> bool:
        """
        returns true if start<=end otherwise false
        """
        ...



class UtcTimeVector:
    @overload
    def __init__(self, objects: List[Any]):
        """
        Constructs a strongly typed list from a list of objects convertible to the list
        """
        ...
    @overload
    def __init__(self) -> Any:
        """
        construct empty UtcTimeVecor
            
        
        """
        ...
    @overload
    def __init__(self, clone_me: Union[UtcTimeVector,list[time],list[float],np.ndarray]) -> Any:
        """
        construct a copy of supplied  UtcTimeVecor
            
            Args:
                clone_me (UtcTimeVector): to be cloned
            
        
        """
        ...
    @overload
    def __init__(self, seconds_vector: Union[IntVector,list[int],range]) -> Any:
        """
        construct a from seconds epoch utc as integer
            
            Args:
                seconds (IntVector): seconds
            
        
        """
        ...
    @overload
    def __init__(self, seconds_vector: Union[DoubleVector,list[float],np.ndarray]) -> Any:
        """
        construct a from seconds epoch utc as float
            
            Args:
                seconds (DoubleVector): seconds, up to us resolution epoch utc
            
        
        """
        ...
    @overload
    def __init__(self, times: list) -> Any:
        """
        construct a from a list of something that is convertible to UtcTime
            
            Args:
                times (list): a list with convertible times
            
        
        """
        ...
    @overload
    def __init__(self, np_times: object) -> Any:
        """
        construct a from a numpy array of int64 s epoch
            
            Args:
                np_times (list): a list with convertible times
            
        
        """
        ...
    @overload
    def __init__(self, np_times: object) -> Any:
        """
        construct a from a numpy array of float s epoch
            
            Args:
                np_times (list): a list with float convertible times
            
        """
        ...

    def __contains__(self, arg2: object) -> bool:
        ...

    def __delitem__(self, arg2: object) -> None:
        ...

    def __getitem__(self, arg2: object) -> Any:
        ...

    def __iter__(self) -> Any:
        ...

    def __len__(self) -> int:
        ...

    def __setitem__(self, arg2: object, arg3: object) -> None:
        ...

    def append(self, arg2: object) -> None:
        ...

    def extend(self, arg2: object) -> None:
        ...

    def from_numpy(self) -> UtcTimeVector:
        ...

    @staticmethod
    def push_back(*args, **kwargs):
        """
        appends a utctime like value to the vector
            
            Args:
                t (utctime): an int (seconds), or utctime
            
        """
        ...

    def to_numpy(self) -> Any:
        """
        convert to numpy array of type np.int64, seconds since epoch
            
        """
        ...

    def to_numpy_double(self) -> Any:
        """
        convert to numpy array of type np.float64, seconds since epoch
            
        """
        ...



class YMDhms:
    """
    Defines calendar coordinates as Year Month Day hour minute second and micro_second
    The intended usage is ONLY as result from the Calendar.calendar_units(t),
    to ensure a type-safe return of these entities for a given time.
    
        Please use this as a read-only return type from the Calendar.calendar_units(t)
    """
    @property
    def day(self)->int:
        """
         
        """
        ...
    @day.setter
    def day(self, value:int)->None:
        ...
    @property
    def hour(self)->int:
        """
         
        """
        ...
    @hour.setter
    def hour(self, value:int)->None:
        ...
    @property
    def micro_second(self)->int:
        """
         
        """
        ...
    @micro_second.setter
    def micro_second(self, value:int)->None:
        ...
    @property
    def minute(self)->int:
        """
         
        """
        ...
    @minute.setter
    def minute(self, value:int)->None:
        ...
    @property
    def month(self)->int:
        """
         
        """
        ...
    @month.setter
    def month(self, value:int)->None:
        ...
    @property
    def second(self)->int:
        """
         
        """
        ...
    @second.setter
    def second(self, value:int)->None:
        ...
    @property
    def year(self)->int:
        """
         
        """
        ...
    @year.setter
    def year(self, value:int)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, Y: object, M: object, D: object, h: object, m: object, s: object, us: object) -> None:
        """
        Creates calendar coordinates specifying Y,M,D,h,m,s,us
        """
        ...

    def is_null(self) -> bool:
        """
        returns true if all values are 0, - the null definition
        """
        ...

    def is_valid(self) -> bool:
        """
        returns true if YMDhms values are reasonable
        """
        ...

    @staticmethod
    def max() -> YMDhms:
        """
        returns the maximum representation
        """
        ...

    @staticmethod
    def min() -> YMDhms:
        """
        returns the minimum representation
        """
        ...



class YWdhms:
    """
    Defines calendar coordinates as iso Year Week week-day hour minute second and micro_second
    The intended usage is ONLY as result from the Calendar.calendar_week_units(t),
    to ensure a type-safe return of these entities for a given time.
    
    Notes:
        Please use this as a read-only return type from the Calendar.calendar_week_units(t)
    """
    @property
    def hour(self)->int:
        """
         
        """
        ...
    @hour.setter
    def hour(self, value:int)->None:
        ...
    @property
    def iso_week(self)->int:
        """
         
        """
        ...
    @iso_week.setter
    def iso_week(self, value:int)->None:
        ...
    @property
    def iso_year(self)->int:
        """
         
        """
        ...
    @iso_year.setter
    def iso_year(self, value:int)->None:
        ...
    @property
    def micro_second(self)->int:
        """
         
        """
        ...
    @micro_second.setter
    def micro_second(self, value:int)->None:
        ...
    @property
    def minute(self)->int:
        """
         
        """
        ...
    @minute.setter
    def minute(self, value:int)->None:
        ...
    @property
    def second(self)->int:
        """
         
        """
        ...
    @second.setter
    def second(self, value:int)->None:
        ...
    @property
    def week_day(self)->int:
        """
         week_day,[1..7]=[mo..sun]
        """
        ...
    @week_day.setter
    def week_day(self, value:int)->None:
        ...
    @overload
    def __init__(self) -> None:
        ...
    @overload
    def __init__(self, Y: object, W: object, wd: object, h: object, m: object, s: object, us: object) -> None:
        """
        Creates calendar coordinates specifying iso Y,W,wd,h,m,s
            
            Args:
                Y (int): iso-year
            
                W (int): iso week [1..53]
            
                wd (int): week_day [1..7]=[mo..sun]
            
                h (int): hour [0..23]
            
                m (int): minute [0..59]
            
                s (int): second [0..59]
            
                us (int): micro_second [0..999999]
            
        """
        ...

    def is_null(self) -> bool:
        """
        returns true if all values are 0, - the null definition
        """
        ...

    def is_valid(self) -> bool:
        """
        returns true if YWdhms values are reasonable
        """
        ...

    @staticmethod
    def max() -> YWdhms:
        """
        returns the maximum representation
        """
        ...

    @staticmethod
    def min() -> YWdhms:
        """
        returns the minimum representation
        """
        ...



class time:
    """
    **time** is represented as a number, in SI-unit seconds.
    
    For accuracy and performance, it's internally represented as 64bit integer, at micro-second resolution
    It is usually used in two roles:
    
    
     1) time measured in seconds since epoch (1970.01.01UTC)
          often constructed using the Calendar class that takes calendar-coordinates (YMDhms etc) and 
          returns the corresponding time-point, taking time-zone and dst etc. into account.
    
    >>>      utc = Calendar()
    >>>      t1 = utc.time(2018,10,15,16,30,15)
    >>>      t2 = time('2018-10-15T16:30:15Z')
    >>>      t3 = time(1539621015)
    
    
     2) time measure, in unit of seconds (resolution up to 1us)
          often constructed from numbers, always use SI-unit of seconds
    
    >>>      dt1 = time(3600)  # 3600 seconds
    >>>      dt2 = time(0.123456)  #  0.123456 seconds
    
    
    
    It can be constructed supplying number of seconds, or a well defined iso8601 string
    
    To convert it to a python number, use float() or int() to cast operations
    If dealing with time-zones/calendars conversion, use the calendar.time(..)/.calendar_units functions.
    If you want to use time-zone/calendar semantic add/diff/trim, use the corresponding Calendar methods.
    
    
    See also:
        Calendar,deltahours,deltaminutes
    """
    epoch: time
    max: time
    min: time
    undefined: time
    @property
    def seconds(self)->float:
        """
         returns time in seconds
        """
        ...
    @overload
    def __init__(self) -> Any:
        """
        construct a 0s
            
        
        """
        ...
    @overload
    def __init__(self, seconds: object) -> Any:
        """
        construct a from a precise time like item, e.g. time, float,int and iso8601 string
            
            Args:
                seconds (): A time like item, time, float,int or iso8601 YYYY-MM-DDThh:mm:ss[.xxxxxx]Z string
            
        """
        ...

    def __abs__(self) -> time:
        ...

    def __add__(self,other: Union[time,float,int,str]) -> time:
        """
        object __add__(tuple args, dict kwds)
        """
        ...

    def __float__(self) -> float:
        ...

    def __floordiv__(self,other: Union[time,float,int,str]) -> time:
        """
        object __floordiv__(tuple args, dict kwds)
        """
        ...

    @staticmethod
    def __int__(*args, **kwargs):
        """
        time as int seconds
            
        """
        ...

    @staticmethod
    def __long__(*args, **kwargs):
        """
        time as int seconds
            
        """
        ...

    def __mod__(self,other: Union[time,float,int,str]) -> time:
        """
        object __mod__(tuple args, dict kwds)
        """
        ...

    def __mul__(self,other: Union[time,float,int,str]) -> time:
        """
        object __mul__(tuple args, dict kwds)
        """
        ...

    def __neg__(self) -> time:
        """
        __neg__( (time)arg1) -> time
        
        __neg__( (time)arg1) -> object
        """
        ...

    def __radd__(self,other: Union[time,float,int,str]) -> time:
        """
        object __radd__(tuple args, dict kwds)
        """
        ...

    def __rfloordiv__(self,other: Union[time,float,int,str]) -> time:
        """
        object __rfloordiv__(tuple args, dict kwds)
        """
        ...

    def __rmul__(self,other: Union[time,float,int,str]) -> time:
        """
        object __rmul__(tuple args, dict kwds)
        """
        ...

    def __round__(self) -> time:
        ...

    def __rsub__(self,other: Union[time,float,int,str]) -> time:
        """
        object __rsub__(tuple args, dict kwds)
        """
        ...

    def __rtruediv__(self,other: Union[time,float,int,str]) -> time:
        """
        object __rtruediv__(tuple args, dict kwds)
        """
        ...

    def __sub__(self,other: Union[time,float,int,str]) -> time:
        """
        object __sub__(tuple args, dict kwds)
        """
        ...

    def __truediv__(self,other: Union[time,float,int,str]) -> time:
        """
        object __truediv__(tuple args, dict kwds)
        """
        ...

    @staticmethod
    def now() -> time:
        ...

    @staticmethod
    def sqrt(*args, **kwargs):
        ...


ALLOW_ANY_MISSING: ice_packing_temperature_policy
ALLOW_INITIAL_MISSING: ice_packing_temperature_policy
AT_VALUE: extend_split_policy
BACKWARD: derivative_method
CENTER: derivative_method
DEFAULT: derivative_method
DISALLOW_MISSING: ice_packing_temperature_policy
FILL_NAN: extend_fill_policy
FILL_VALUE: extend_fill_policy
FORWARD: derivative_method
LHS_LAST: extend_split_policy
POINT_AVERAGE_VALUE: point_interpretation_policy
POINT_INSTANT_VALUE: point_interpretation_policy
RHS_FIRST: extend_split_policy
SCHEME_CATMULL_ROM: interpolation_scheme
SCHEME_LINEAR: interpolation_scheme
SCHEME_POLYNOMIAL: interpolation_scheme
TRIM_IN: trim_policy
TRIM_OUT: trim_policy
TRIM_ROUND: trim_policy
USE_LAST: extend_fill_policy
max_utctime: time
min_utctime: time
no_utctime: time
npos: int
def _finalize() -> None:
    ...

def accumulate(ts: TimeSeries, time_axis: TimeAxis) -> TimeSeries:
    """
    create a new ts that is the integral f(t) *dt, t0..ti, the specified time-axis
    """
    ...

def average(ts: TimeSeries, time_axis: TimeAxis) -> TimeSeries:
    """
    creates a true average time-series of ts for intervals as specified by time_axis
    """
    ...

def byte_vector_from_file(path: str) -> ByteVector:
    """
    reads specified file and returns its contents as a ByteVector
    """
    ...

def byte_vector_from_hex_str(hex_str: str) -> ByteVector:
    """
    converts earlier byte-vector hex-string,[0-9a-f]x2 etc, to byte-vector
    """
    ...

def byte_vector_to_file(path: str, byte_vector: ByteVector) -> None:
    """
    write the supplied ByteVector to file as specified by path
    """
    ...

def byte_vector_to_hex_str(byte_vector: ByteVector) -> str:
    """
    return hex-string [0-9a-f]x2 etc of byte-vector
    """
    ...

def create_glacier_melt_ts_m3s(temperature: TimeSeries, sca_m2: TimeSeries, glacier_area_m2: object, dtf: object) -> TimeSeries:
    """
    create a ts that provide the glacier-melt algorithm based on the inputs
        
        Args:
            temperature (TimeSeries): a temperature time-series, unit [deg.Celcius]
        
            sca_m2 (TimeSeries): a snow covered area (sca) time-series, unit [m2]
        
            glacier_area_m2 (float): the glacier area, unit[m2]
        
            dtf (float): degree timestep factor [mm/day/deg.C]; lit. values for Norway: 5.5 - 6.4 in Hock, R. (2003), J. Hydrol., 282, 104-115
        
        Returns:
            TimeSeries: glacier_melt. an expression computing the glacier melt based on the inputs
        
    """
    ...

def create_periodic_pattern_ts(pattern: Union[DoubleVector,list[float],np.ndarray], dt: Union[time,float,int], t0: Union[time,float,int], ta: TimeAxis) -> TimeSeries:
    """
    Create a Timeseries by repeating the pattern-specification
        
        Args:
            pattern (DoubleVector): the value-pattern as a sequence of values
        
            dt (int): number of seconds between the pattern values, e.g. deltahours(3)
        
            t0 (utctime): specifies the start-time of the pattern
        
            ta (TimeAxis): the time-axis for which the pattern is repeated
            e.g. your pattern might be 8 3h values,and you could supply
            a time-axis 'ta' at hourly resolution
        
    """
    ...

def create_ts_vector_from_np_array(time_axis: TimeAxis, np_array: object, point_fx: point_interpretation_policy) -> TsVector:
    """
    Create a TsVector from specified time_axis, 2-d np_array and point_fx.
        
        Args:
            time_axis (TimeAxis): time-axis that matches in length to 2nd dim of np_array
        
            np_array (np.ndarray): numpy array of dtype=np.float64, and shape(n_ts,n_points)
        
            point_fx (point interpretation): one of POINT_AVERAGE_VALUE|POINT_INSTANT_VALUE
        
        Returns:
            TsVector: tsv. a TsVector of length first np_array dim, n_ts, each with time-axis, values and point_fx
        
    """
    ...

def deltahours(n: object) -> time:
    """
    returns time equal to specified n hours
    """
    ...

def deltaminutes(n: object) -> time:
    """
    returns time equal to specified n minutes
    """
    ...

def dtss_finalize() -> None:
    """
    dlib socket and timer cleanup before exit python(automatically called once at module exit)
    """
    ...

def ext_path_url(prefix: str, container: str, ts_path: str) -> str:
    """
    Construct a prefix container / urlencode(ts_name) fast
        
        
        Args:
            prefix (str): like fame:// mydb:// or similar
        
            container (str): TS container.
        
            ts_path (str): Time-series path.
        
        Returns:
            str: url. Constructed url with url-encoded ts_path
        
        See also:
            shyft_url, ext_query_url, urlencode, extract_shyft_url_container, extract_shyft_url_path, extract_shyft_url_query_parameters
        
    """
    ...

def ext_query_url(prefix: str, container: str, ts_path: str) -> str:
    """
    Construct a prefix container?urlencode(ts_name) fast
        
        
        Args:
            prefix (str): like fame:// mydb:// or similar
        
            container (str): TS container.
        
            ts_path (str): Time-series path.
        
        Returns:
            str: url. Constructed url with url-encoded ts_path
        
        See also:
            shyft_url,ext_path_url, urlencode, extract_shyft_url_container, extract_shyft_url_path, extract_shyft_url_query_parameters
        
    """
    ...

def extract_shyft_url_container(url: str) -> str:
    """
    Extract the container part from a Shyft URL.
        
        Args:
            url (str): Shyft URL to extract container from.
        
        Returns:
            str: container. Container part of `url`, if the string is invalid as a Shyft URL
            an empty string is retuned instead.
        
        See also:
            shyft_url
        
    """
    ...

def extract_shyft_url_path(url: str) -> str:
    """
    Extract the time-series path part from a Shyft URL.
        
        Args:
            url (str): Shyft URL to extract the TS-path from.
        
        Returns:
            str: ts_path. TS-path part of `url`, if the string is invalid as a Shyft URL
            an empty string is retuned instead.
        
        See also:
            shyft_url
        
    """
    ...

def extract_shyft_url_query_parameters(url: str) -> dict:
    """
    Extract query parameters from a Shyft URL.
        
        The query string is assumed to be on the format `?key1=value1&key2=value2&key3=&key4=value4`.
        This will be parsed into a map with four keys: `key1` through `key4`, where `key3` have a
        empty string value, while the rest have respectivly values `value1`, `value2`, and `value4`.
        
        Both query keys and values are assumed to be urlencoded, thus urlencode is called on every
        key and every value.
        
        Args:
            url (str): Shyft URL to extract the TS-path from.
        
        Returns:
            Dict[str,str]: queries. A dict with all queries defined in `url`. The dictionary is empty if `url` is invalid as a Shyft URL.
        
        See also:
            shyft_url, urldecode
        
    """
    ...

def integral(ts: TimeSeries, time_axis: TimeAxis) -> TimeSeries:
    """
    creates a true integral time-series of ts for intervals as specified by time_axis
    """
    ...

def intersection(a: UtcPeriod, b: UtcPeriod) -> UtcPeriod:
    """
    Returns the intersection of two periods
        if there is an intersection, the resulting period will be .valid() and .timespan()>0
        If there is no intersection, an empty not .valid() period is returned
        
        Args:
            a (UtcPeriod): 1st UtcPeriod argument
        
            b (UtcPeriod): 2nd UtcPeriod argument
        
        Returns:
            UtcPeriod: intersection. The computed intersection, or an empty not .valid() UtcPeriod
        
    """
    ...

def is_npos(n: object) -> bool:
    """
    returns true if n is npos, - meaning no position
    """
    ...

def kling_gupta(observation_ts: TimeSeries, model_ts: TimeSeries, time_axis: TimeAxis, s_r: object, s_a: object, s_b: object) -> float:
    """
    Computes the kling-gupta KGEs correlation for the two time-series over the specified time_axis
        
        Args:
            observed_ts (TimeSeries): the observed time-series
        
            model_ts (TimeSeries): the time-series that is the model simulated / calculated ts
        
            time_axis (TimeAxis): the time-axis that is used for the computation
        
            s_r (float): the kling gupta scale r factor(weight the correlation of goal function)
        
            s_a (float): the kling gupta scale a factor(weight the relative average of the goal function)
        
            s_b (float): the kling gupta scale b factor(weight the relative standard deviation of the goal function)
        
        Returns:
            float: KGEs. The  KGEs= 1-EDs that have a maximum at 1.0
        
    """
    ...

def log(ts: TimeSeries) -> TimeSeries:
    """
    returns a new ts as log(ts)
    """
    ...

def log_level(log_level: object) -> None:
    """
    Set log_level for **all**  loggers,
        Note: Ensure there is space on device to consume the logs, and ensure separate log partition for sensitive/critical systems
        
        Args:
            log_level (int):  -1000=everything, -100=TRACE, 0=DEBUG,100=INFO,200=WARN,300=ERROR,400=FATAL,1000=nologs
        
    """
    ...

def log_to_file(filename: str) -> None:
    """
    Direct all logs to specified file, leaving `filename`='' logs to stdout
        This affects all logging in Shyft.
        In particular the dtss backends, rocksdb and leveldb will generate verbose output
        when they are configured to log via the `DtssCfg` class.Note: Ensure there is space on device to consume the logs, and ensure separate log partition for sensitive/critical systems
        
        Args:
            filename (str): path to file
        
    """
    ...

@overload
def max(ts_a: TimeSeries, ts_b: TimeSeries) -> TimeSeries:
    """
    returns a new ts as max(ts_a,ts_b)
    
    """
    ...
@overload
def max(a: object, ts_b: TimeSeries) -> TimeSeries:
    """
    returns a new ts as max(a,ts_b)
    
    """
    ...
@overload
def max(ts_a: TimeSeries, b: object) -> TimeSeries:
    """
    returns a new ts as max(ts_a,b)
    
    """
    ...
@overload
def max(ts: TimeSeries, ts_vector: TsVector) -> TsVector:
    """
    return max of ts and ts_vector
    
    """
    ...
@overload
def max(ts_vector: TsVector, ts: TimeSeries) -> TsVector:
    """
    return max of ts_vector and ts
    
    """
    ...
@overload
def max(ts_vector: TsVector, number: object) -> TsVector:
    """
    return max of ts_vector and number
    
    """
    ...
@overload
def max(number: object, ts_vector: TsVector) -> TsVector:
    """
    return max of number and ts_vector
    
    """
    ...
@overload
def max(a: TsVector, b: TsVector) -> TsVector:
    """
    return max of ts_vectors a and b (requires equal size!)
    """
    ...

@overload
def min(ts_a: TimeSeries, ts_b: TimeSeries) -> TimeSeries:
    """
    returns a new ts as min(ts_a,ts_b)
    
    """
    ...
@overload
def min(a: object, ts_b: TimeSeries) -> TimeSeries:
    """
    returns a new ts as min(a,ts_b)
    
    """
    ...
@overload
def min(ts_a: TimeSeries, b: object) -> TimeSeries:
    """
    returns a new ts as min(ts_a,b)
    
    """
    ...
@overload
def min(ts: TimeSeries, ts_vector: TsVector) -> TsVector:
    """
    return minimum of ts and ts_vector
    
    """
    ...
@overload
def min(ts_vector: TsVector, ts: TimeSeries) -> TsVector:
    """
    return minimum of ts_vector and ts
    
    """
    ...
@overload
def min(ts_vector: TsVector, number: object) -> TsVector:
    """
    return minimum of ts_vector and number
    
    """
    ...
@overload
def min(number: object, ts_vector: TsVector) -> TsVector:
    """
    return minimum of number and ts_vector
    
    """
    ...
@overload
def min(a: TsVector, b: TsVector) -> TsVector:
    """
    return minimum of ts_vectors a and b (requires equal size!)
    """
    ...

def nash_sutcliffe(observation_ts: TimeSeries, model_ts: TimeSeries, time_axis: TimeAxis) -> float:
    """
    Computes the Nash-Sutcliffe model effiency coefficient (n.s) 
        for the two time-series over the specified time_axis
        
        Ref:  http://en.wikipedia.org/wiki/Nash%E2%80%93Sutcliffe_model_efficiency_coefficient 
        
        
        Args:
            observed_ts (TimeSeries): the observed time-series
        
            model_ts (TimeSeries): the time-series that is the model simulated / calculated ts
        
            time_axis (TimeAxis): the time-axis that is used for the computation
        
        Returns:
            float: ns. The  n.s performance, that have a maximum at 1.0
        
    """
    ...

@overload
def pow(ts_a: TimeSeries, ts_b: TimeSeries) -> TimeSeries:
    """
    returns a new ts as pow(ts_a,ts_b)
    
    """
    ...
@overload
def pow(a: object, ts_b: TimeSeries) -> TimeSeries:
    """
    returns a new ts as pow(a,ts_b)
    
    """
    ...
@overload
def pow(ts_a: TimeSeries, b: object) -> TimeSeries:
    """
    returns a new ts as pow(ts_a,b)
    
    """
    ...
@overload
def pow(ts: TimeSeries, ts_vector: TsVector) -> TsVector:
    """
    return pow(ts,ts_vector)->TsVector
    
    """
    ...
@overload
def pow(ts_vector: TsVector, ts: TimeSeries) -> TsVector:
    """
    return pow(ts_vector,ts)->TsVector
    
    """
    ...
@overload
def pow(ts_vector: TsVector, number: object) -> TsVector:
    """
    return pow(ts_vector,number)->TsVector
    
    """
    ...
@overload
def pow(number: object, ts_vector: TsVector) -> TsVector:
    """
    return pow(number,ts_vector)->TsVector
    
    """
    ...
@overload
def pow(a: TsVector, b: TsVector) -> TsVector:
    """
    return pow(a,b)->TsVector, (requires equal size!)
    """
    ...

@overload
def quantile_map_forecast(forecast_sets: TsVectorSet, set_weights: Union[DoubleVector,list[float],np.ndarray], historical_data: TsVector, time_axis: TimeAxis, interpolation_start: Union[time,float,int]) -> TsVector:
    """
    Computes the quantile-mapped forecast from the supplied input.
        ***TBD:detailed description with references***
        
        Args:
            forecast_sets (TsVectorSet): forecast sets, each of them a TsVector with n forecasts (might differ in size and length)
        
            set_weights (DoubleVector): a weight for each of the forecast set in forecast-sets,correlated by same index)
        
            historical_data (TsVector): historical time-series that should cover the requested time-axis
        
            time_axis (TimeAxis): the time-axis that the resulting time-series are mapped into
        
            interpolation_start (int): time where the historical to forecast interpolation should start, 1970 utc seconds since epoch
        
            interpolation_end (int): time where the interpolation should end, if no_utctime, use end of forecast-set
        
            interpolated_quantiles (bool): whether the quantile values should be interpolated or assigned the values lower than or equal to the current quantile
        
        Returns:
            TsVector: qm_forecast. quantile mapped forecast with the requested time-axis
        
    
    """
    ...
@overload
def quantile_map_forecast(forecast_sets: TsVectorSet, set_weights: Union[DoubleVector,list[float],np.ndarray], historical_data: TsVector, time_axis: TimeAxis, interpolation_start: Union[time,float,int], interpolation_end: Union[time,float,int]) -> TsVector:
    """
    Computes the quantile-mapped forecast from the supplied input.
        ***TBD:detailed description with references***
        
        Args:
            forecast_sets (TsVectorSet): forecast sets, each of them a TsVector with n forecasts (might differ in size and length)
        
            set_weights (DoubleVector): a weight for each of the forecast set in forecast-sets,correlated by same index)
        
            historical_data (TsVector): historical time-series that should cover the requested time-axis
        
            time_axis (TimeAxis): the time-axis that the resulting time-series are mapped into
        
            interpolation_start (int): time where the historical to forecast interpolation should start, 1970 utc seconds since epoch
        
            interpolation_end (int): time where the interpolation should end, if no_utctime, use end of forecast-set
        
            interpolated_quantiles (bool): whether the quantile values should be interpolated or assigned the values lower than or equal to the current quantile
        
        Returns:
            TsVector: qm_forecast. quantile mapped forecast with the requested time-axis
        
    
    """
    ...
@overload
def quantile_map_forecast(forecast_sets: TsVectorSet, set_weights: Union[DoubleVector,list[float],np.ndarray], historical_data: TsVector, time_axis: TimeAxis, interpolation_start: Union[time,float,int], interpolation_end: Union[time,float,int], interpolated_quantiles: object) -> TsVector:
    """
    Computes the quantile-mapped forecast from the supplied input.
        ***TBD:detailed description with references***
        
        Args:
            forecast_sets (TsVectorSet): forecast sets, each of them a TsVector with n forecasts (might differ in size and length)
        
            set_weights (DoubleVector): a weight for each of the forecast set in forecast-sets,correlated by same index)
        
            historical_data (TsVector): historical time-series that should cover the requested time-axis
        
            time_axis (TimeAxis): the time-axis that the resulting time-series are mapped into
        
            interpolation_start (int): time where the historical to forecast interpolation should start, 1970 utc seconds since epoch
        
            interpolation_end (int): time where the interpolation should end, if no_utctime, use end of forecast-set
        
            interpolated_quantiles (bool): whether the quantile values should be interpolated or assigned the values lower than or equal to the current quantile
        
        Returns:
            TsVector: qm_forecast. quantile mapped forecast with the requested time-axis
        
    """
    ...

def shyft_url(container: str, ts_path: str, queries: dict = {}) -> str:
    """
    Construct a Shyft URL from a container, a TS-path, and an optional collection of query flags.
        
        Query keys and values are always urlencoded. The separating `?`, `&`, and `=` are not encoded.
        
        Args:
            container (str): Shyft TS container.
        
            ts_path (str): Time-series path.
        
            queries (Dict[str,str]): Optional mapping from query keys to values. Defaults to an empty dictionary.
        
        Returns:
            str: url. Constructed Shyft URL.
        
        See also:
            shyft_url, urlencode, extract_shyft_url_container, extract_shyft_url_path, extract_shyft_url_query_parameters
        
    """
    ...

def time_axis_extract_time_points(time_axis: TimeAxis) -> IntVector:
    """
    Extract all time_axis.period(i).start plus time_axis.total_period().end into a UtcTimeVector
        
        Args:
            time_axis (TimeAxis): time-axis to extract all time-points from
        
        Returns:
            Int64Vector: time_points. all time_axis.period(i).start plus time_axis.total_period().end
        
    """
    ...

def time_axis_extract_time_points_as_utctime(time_axis: TimeAxis) -> UtcTimeVector:
    """
    Extract all time_axis.period(i).start plus time_axis.total_period().end into a UtcTimeVector
        
        Args:
            time_axis (TimeAxis): time-axis to extract all time-points from
        
        Returns:
            UtcTimeVector: time_points. all time_axis.period(i).start plus time_axis.total_period().end
        
    """
    ...

def time_axis_extract_time_points_as_utctime_tz(time_axis: TimeAxis, calendar: Calendar) -> UtcTimeVector:
    """
    Extract all time_axis.period(i).start plus time_axis.total_period().end into a UtcTimeVector
         - adding the calendar.tz_info.utc_offset(t) for all t in UtcTimeVector 
        Effectively you get out 'tz' version of the time-points, that might not be strictly ascending
        Intended usage is within the shyft.dashboard time-series rendering where we currently 
        need to pass tz-version of the time for rendering due to lack of proper view-handling of
        time-zones in bokeh
        NOT recommended for other usage!
        
        Args:
            time_axis (TimeAxis): time-axis to extract all time-points from
        
            calendar (Calendar): calendar with tz-info to use for adding tz_info.utc_offset(t)
        
        Returns:
            UtcTimeVector: time_points. all time_axis.period(i).start plus time_axis.total_period().end
        
    """
    ...

def time_series_to_bokeh_plot_data(ts: TimeSeries, calendar: Calendar, time_scale: object, force_linear: object, crop_trailing_nans: object) -> DoubleVectorVector:
    """
    To speedup and ease sending time-series data to bokeh, a all in one function to provide
        values from a time-series ready to be rendered by bokeh time-series plot
        Call t=r[0].to_numpy() f=r[1].to_numpy() to get numpy arrays, before sending those to bokeh
        
        Args:
            ts (TimeSeries): time-series to extract data from
        
            calendar (Calendar): calendar with time-zone to make tz-adjusted values for the time-points
        
            time_scale (float): the time-scale, usually 1000, as bokeh uses ms scaled time-values
        
            force_linear (bool): if set true, just provide the points as is, otherwise, ts.point_interpretation is considered
        
            crop_trailing_nans (bool): if true, crop away the trailing nan's from the time-series
        
        Returns:
            DoubleVectorVector: plot-data. [time-stamps, values] A strongly typed Vector of DoubleVector
        
    """
    ...

def time_shift(timeseries: TimeSeries, delta_t: Union[time,float,int]) -> TimeSeries:
    """
    returns a delta_t time-shifted time-series
         the values are the same as the original,
         but the time_axis equals the original + delta_t
        
    """
    ...

def ts_stringify(ts: TimeSeries) -> str:
    """
    Given a TimeSeries, return a string showing the details/expression
        
    """
    ...

def urldecode(encoded: str, space_pluss: object = True) -> str:
    """
    Decode a percent-encoded string to its original representation.
        
        All characters designated as unreserved in RFC3986 (Jan. 2005), sec. 2.3 are always passed
        through unmodified, except where they are encountered while parsing a percent-encoded value.
        
        The implementation only handles 8-bit character values. The behavior for multibyte characters
        is unspecified.
        Additionally it is undefined if characters outside the range `0-9A-Fa-f` are encountered as
        one of the two characters immidiatly succeeding a percent-sign.
        
        This is the reverse operation of `urlencode`.
        
        Args:
            encoded (str): Text string to decode.
        
            percent_plus (bool): When true `+` characters are decoded to `SP` characters (ASCII 0x20). When this is true and a `+` is encountered and exception is thrown. The default value is true.
        
        Returns:
            str: text. Original representation of the encoded input.
        
        
        Raises:
            RuntimeError: Thrown if unencoded characters outside the unreserved range is encountered, includes `+` when the `percent_plus` argument is false. The exception message contains the character and its location in the string.
        See also:
            urlencode
        
    """
    ...

def urlencode(text: str, space_pluss: object = True) -> str:
    """
    Percent-encode a string for use in URLs.
        
        All characters designated as reserved in RFC3986 (Jan. 2005), sec. 2.2 are always percent
        encoded, while all character explitily stated as unreserved (RFC3986, Jan. 2005, sec. 2.3)
        are never percent encoded. Space characters are encoded as `+` characters, while all other
        characters are precent-encoded.
        
        The implementation only handles 8-bit character values. The behavior for multibyte characters
        is unspecified.
        
        The reverse operation of this is `urldecode`.
        
        Args:
            text (str): Text string to encode.
        
            percent_plus (bool): When true the `SP` character (ASCII 0x20) is encoded as `+` instead of its percent encoding `%20`. Defaults to true.
        
        Returns:
            str: encoded. Percent-encoded representation if the input.
        
        See also:
            urldecode
        
    """
    ...

def utctime_now() -> time:
    """
    returns time now as seconds since 1970s
    """
    ...

def win_set_priority(p_class: object) -> None:
    """
    Win32 Api function to set normal (=0) or low(=-1) priority
        This is *very* specific to windows, and especially task-scheduler/bg.tasks
        get by default a complete garbled priority leaving the process you run
        close to useless when it comes to cpu,io and memory performance
        The UI does not help fixing this, and even the xml edit of files does not solve memory/io issues
        
        Args:
            p_class (int): priority class, 0=normal, -1=below normal(slightly above useless
        
    """
    ...

def win_short_path(path: str) -> str:
    """
    WinApi function GetShortPath exposed to python
        https://msdn.microsoft.com/en-us/library/windows/desktop/aa364989(v=vs.85).aspx
        Note that it only works for file-paths that exists, returns null string for not-existing files
        
        Args:
            path (str): a long path form
        
        Returns:
            str: short_path. windows 8.3 path string if on windows for *existing* files, otherwise same as input path
        
    """
    ...

